@@ -1,7 +1,7 @@
 #' @title Analyse post-IR IRSL measurement sequences
 #'
-#' @description The function performs an analysis of post-IR IRSL sequences
-#' including curve
+#' @description
+#' The function performs an analysis of post-IR IRSL sequences including curve
 #' fitting on [RLum.Analysis-class] objects.
 #'
 #' @details To allow post-IR IRSL protocol (Thomsen et al., 2008) measurement
@@ -12,12 +12,13 @@
 #'
 #' **Defining the sequence structure**
 #'
-#' The argument `sequence.structure` expects a shortened pattern of your sequence structure and was
-#' mainly introduced to ease the use of the function. For example: If your measurement data contains
-#' the following curves: `TL`, `IRSL`, `IRSL`, `TL`, `IRSL`, `IRSL`, the sequence pattern in `sequence.structure`
-#' becomes `c('TL', 'IRSL', 'IRSL')`. The second part of your sequence for one cycle should be
-#' similar and can be discarded. If this is not the case (e.g., additional hotbleach) such curves
-#' have to be removed before using the function.
+#' The argument `sequence.structure` expects a shortened pattern of your
+#' sequence structure and was mainly introduced to ease the use of the function.
+#' For example: If your measurement data contains the following curves: `TL`,
+#' `IRSL`, `IRSL`, `TL`, `IRSL`, `IRSL`, the pattern in `sequence.structure`
+#' becomes `c('TL', 'IRSL', 'IRSL')`. The second part of your sequence for one
+#' cycle should be similar and can be discarded. If this is not the case (e.g.,
+#' additional hotbleach) such curves must be removed before using the function.
 #'
 #' **If the input is a `list`**
 #'
@@ -66,7 +67,7 @@
 #' Ignored if `plot = FALSE`.
 #'
 #' @param ... further arguments that will be passed to
-#' [analyse_SAR.CWOSL] and [plot_GrowthCurve]. Furthermore, the
+#' [analyse_SAR.CWOSL] and [plot_DoseResponseCurve]. Furthermore, the
 #' arguments `main` (headers), `log` (IRSL curves), `cex` (control
 #' the size) and `mtext.outer` (additional text on the plot area) can be passed to influence the plotting. If the input
 #' is a list, `main` can be passed as [vector] or [list].
@@ -92,7 +93,7 @@
 #'
 #' `pdf(file = "<YOUR FILENAME>", height = 18, width = 18)`
 #'
-#' @section Function version: 0.2.5
+#' @section Function version: 0.2.6
 #'
 #' @author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
@@ -113,7 +114,6 @@
 #'
 #' @examples
 #'
-#'
 #' ### NOTE: For this example existing example data are used. These data are non pIRIR data.
 #' ###
 #' ##(1) Compile example data set based on existing example data (SAR quartz measurement)
@@ -166,7 +166,6 @@
 #'   dev.off()
 #' }
 #'
-#' @md
 #' @export
 analyse_pIRIRSequence <- function(
   object,
@@ -183,12 +182,16 @@ analyse_pIRIRSequence <- function(
   .set_function_name("analyse_pIRIRSequence")
   on.exit(.unset_function_name(), add = TRUE)
 
-# SELF CALL -----------------------------------------------------------------------------------
- if(is.list(object)){
+  ## store the graphical parameters: this is done here because the function
+  ## self calls, and this should ensure that we reset the global state to
+  ## what it was when we were first called
+  par.default <- .par_defaults()
+  on.exit(par(par.default), add = TRUE)
 
-   lapply(object, function(x) {
-     .validate_class(x, "RLum.Analysis", name = "All elements of 'object'")
-   })
+  ## Self-call --------------------------------------------------------------
+  if (inherits(object, "list")) {
+    lapply(object, .validate_class, "RLum.Analysis",
+           name = "All elements of 'object'")
 
     ## make life easy
     if(missing("signal.integral.min")){
@@ -197,10 +200,14 @@ analyse_pIRIRSequence <- function(
     }
 
     if(missing("signal.integral.max")){
-      signal.integral.max <- 2
-      .throw_warning("'signal.integral.max' missing, set to 2")
+      signal.integral.max <- signal.integral.min + 1
+      .throw_warning("'signal.integral.max' missing, set to ",
+                     signal.integral.max)
     }
 
+    .validate_class(background.integral.min, c("integer", "numeric"))
+    .validate_class(background.integral.max, c("integer", "numeric"))
+
    ## expand input arguments
    rep.length <- length(object)
 
@@ -211,7 +218,7 @@ analyse_pIRIRSequence <- function(
    sequence.structure <- .listify(sequence.structure, rep.length)
    dose.points <- .listify(dose.points, rep.length)
 
-   if ("main" %in% names(list(...))) {
+   if ("main" %in% ...names()) {
      main_list <- .listify(list(...)$main, rep.length)
    }
 
@@ -226,7 +233,7 @@ analyse_pIRIRSequence <- function(
                         sequence.structure = sequence.structure[[x]],
                         plot = plot,
                         plot_singlePanels = plot_singlePanels,
-                        main = ifelse("main"%in% names(list(...)), main_list[[x]], paste0("ALQ #",x)),
+                        main = ifelse("main"%in% ...names(), main_list[[x]], paste0("ALQ #",x)),
                         ...)
     }))
 
@@ -239,13 +246,16 @@ analyse_pIRIRSequence <- function(
     ##DO NOT use invisible here, this will stop the function from stopping
     if(length(results) == 0)
       return(NULL)
-    else
-      return(results)
+    return(results)
   }
 
   ## Integrity checks -------------------------------------------------------
 
   .validate_class(object, "RLum.Analysis", extra = "'list'")
+  .validate_class(signal.integral.min, c("integer", "numeric"))
+  .validate_class(signal.integral.max, c("integer", "numeric"))
+  .validate_class(background.integral.min, c("integer", "numeric"))
+  .validate_class(background.integral.max, c("integer", "numeric"))
   .validate_logical_scalar(plot)
   .validate_logical_scalar(plot_singlePanels)
 
@@ -255,55 +265,41 @@ analyse_pIRIRSequence <- function(
   }
 
   ## check allowed values in sequence structure
-  temp.collect.invalid.terms <- .collapse(
-      sequence.structure[!grepl("TL",  sequence.structure) &
-                         !grepl("IR",  sequence.structure) &
-                         !grepl("OSL", sequence.structure) &
-                         !grepl("EXCLUDE", sequence.structure)])
-
-  if (temp.collect.invalid.terms != "") {
-    .throw_error(temp.collect.invalid.terms,
-                 " not allowed in 'sequence.structure'")
+  invalid.terms <- .collapse(unique(grep("TL|IR|OSL|EXCLUDE", sequence.structure,
+                                         invert = TRUE, value = TRUE)),
+                             last_sep = " and ")
+  if (invalid.terms != "") {
+    .throw_error(invalid.terms, " not allowed in 'sequence.structure'")
   }
 
   ## deprecated argument
-  if ("plot.single" %in% names(list(...))) {
+  if ("plot.single" %in% ...names()) {
     plot_singlePanels <- list(...)$plot.single
     .throw_warning("'plot.single' is deprecated, use 'plot_singlePanels' ",
                    "instead")
   }
 
+  ## Deal with extra arguments
+  extraArgs <- list(...)
+  main <- extraArgs$main %||% "MEASUREMENT INFO"
+  cex <- extraArgs$cex %||% 0.7
+
   ## Enforce a minimum plot device size: this is necessary as otherwise users
   ## may experience "figure margins too large" errors when trying to draw all
   ## plots on a single page. We need to round the device size values because
   ## often they are values such as 15.99999999999 which would incorrectly
   ## trigger our check
-  min.size <- 16
+  min.size <- ceiling(11 * cex)
   dev.size <- round(grDevices::dev.size("in"), 5)
   if (plot && !plot_singlePanels && any(dev.size < min.size)) {
     plot <- FALSE
-    msg <- paste0("Argument 'plot' reset to 'FALSE'. The smallest plot ",
-                  "size required is IN x IN in.\nConsider plotting via ",
-                  "`pdf(..., width = IN, height = IN)` ",
-                  "or setting `plot_singlePanels = TRUE`")
+    msg <- paste0("Argument 'plot' reset to 'FALSE': the smallest plot ",
+                  "size required is IN x IN in (at cex = ", cex, "). ",
+                  "Consider plotting via `pdf(..., width = IN, height = IN)` ",
+                  "or setting `plot_singlePanels = TRUE`. ")
     .throw_warning(gsub(x = msg, "IN", min.size))
   }
 
-# Deal with extra arguments -------------------------------------------------------------------
-  ## default values
-  mtext.outer <- "MEASUREMENT INFO"
-  main <- ""
-  log <- ""
-  cex <- 0.7
-
-  ##deal with addition arguments
-  extraArgs <- list(...)
-  mtext.outer <- if ("mtext.outer" %in% names(extraArgs)) extraArgs$mtext.outer
-  main <- if ("main" %in% names(extraArgs)) extraArgs$main
-  log <- if ("log" %in% names(extraArgs)) extraArgs$log
-  cex <- if ("cex" %in% names(extraArgs)) extraArgs$cex
-
-
 # Protocol Integrity Checks --------------------------------------------------
   ##(1) Check structure and remove curves that fit not the recordType criteria
 
@@ -311,18 +307,15 @@ analyse_pIRIRSequence <- function(
   temp.sequence.structure  <- structure_RLum(object)
 
   ##remove data types that fit not to the allowed values
-  temp.sequence.rm.id <- temp.sequence.structure[
-    (!grepl("TL",temp.sequence.structure[["recordType"]])) &
-    (!grepl("OSL", temp.sequence.structure[["recordType"]])) &
-    (!grepl("IRSL", temp.sequence.structure[["recordType"]]))
-    ,"id"]
+  temp.sequence.rm.id <- temp.sequence.structure[["id"]][
+      !grepl("(T|OS|IRS)L", temp.sequence.structure[["recordType"]])]
 
   if(length(temp.sequence.rm.id)>0){
-
   ##removed record from data set
-  object <- get_RLum(object, record.id = -temp.sequence.rm.id,
-        drop = FALSE
-      )
+  object <- get_RLum(
+    object = object,
+    record.id = -temp.sequence.rm.id,
+    drop = FALSE)
 
   .throw_warning("The following unrecognised record types have been removed: ",
                  .collapse(temp.sequence.structure[temp.sequence.rm.id,
@@ -330,8 +323,6 @@ analyse_pIRIRSequence <- function(
   }
 
   ##(2) Apply user sequence structure
-
-  ##get sequence structure
   temp.sequence.structure  <- structure_RLum(object)
 
     ##try to account for a very common mistake
@@ -355,14 +346,11 @@ analyse_pIRIRSequence <- function(
   }
 
   ##remove values that have been excluded
-  temp.sequence.rm.id <- temp.sequence.structure[
-    temp.sequence.structure[,"protocol.step"] == "EXCLUDE" ,"id"]
-
-  if(length(temp.sequence.rm.id)>0){
+  rm.id <- temp.sequence.structure$id[temp.sequence.structure$protocol.step == "EXCLUDE"]
 
+  if (length(rm.id) > 0) {
     ##remove from object
-    object  <- get_RLum(
-      object, record.id = -temp.sequence.rm.id, drop = FALSE)
+    object  <- get_RLum(object, record.id = -rm.id, drop = FALSE)
 
     ##remove from sequence structure
     sequence.structure  <- sequence.structure[sequence.structure != "EXCLUDE"]
@@ -373,9 +361,7 @@ analyse_pIRIRSequence <- function(
     temp.sequence.structure[, "protocol.step"] <- rep(
       sequence.structure, nrow(temp.sequence.structure)/2/length(temp.sequence.structure))
 
-    ##print warning message
-    .throw_warning(length(temp.sequence.rm.id),
-                   " records have been removed due to EXCLUDE")
+    .throw_warning(length(rm.id), " records have been removed due to EXCLUDE")
   }
 
 ##============================================================================##
@@ -405,9 +391,8 @@ analyse_pIRIRSequence <- function(
   ## the order of the produced plots by the previous functions
 
   if (plot && !plot_singlePanels) {
-
     ##first (Tx,Tn, Lx,Ln)
-    temp.IRSL.layout.vector.first <- c(3,5,6,7,3,5,6,8)
+    temp.IRSL.layout.vector.first <- c(3, 5, 6, 7, 3, 5, 6, 8)
 
     ## middle (any other Lx,Ln)
     temp.IRSL.layout.vector.middle <- NULL
@@ -415,7 +400,7 @@ analyse_pIRIRSequence <- function(
     temp.IRSL.layout.vector.middle <-
       vapply(2:(n.loops - 1),
         FUN = function(x) 5 * x - 1 + c(0:3, 0:2, 4),
-        FUN.VALUE = vector(mode = "numeric", length = 8)
+        FUN.VALUE = numeric(8)
       )
     }
 
@@ -424,12 +409,10 @@ analyse_pIRIRSequence <- function(
       (if (n.loops > 2) max(temp.IRSL.layout.vector.middle)
        else max(temp.IRSL.layout.vector.first))
 
-    temp.IRSL.layout.vector <- c(temp.IRSL.layout.vector.first,
-                                 temp.IRSL.layout.vector.middle,
-                                 temp.IRSL.layout.vector.last)
-
-  ##get layout information
-  def.par <- par(no.readonly = TRUE)
+    temp.IRSL.layout.vector <- c(
+      temp.IRSL.layout.vector.first,
+      temp.IRSL.layout.vector.middle,
+      temp.IRSL.layout.vector.last)
 
   ##set up layout matrix linked to the number of plot areas needed
   layout.matrix  <- c(
@@ -437,22 +420,19 @@ analyse_pIRIRSequence <- function(
     temp.IRSL.layout.vector, #IRSL curves,
     rep((max(temp.IRSL.layout.vector)-3),8), #legend,
     rep((max(temp.IRSL.layout.vector)+1),1), #GC
-    rep((max(temp.IRSL.layout.vector)+2),1), #TnTc
+    rep((max(temp.IRSL.layout.vector)+2),1), #TnTx
     rep((max(temp.IRSL.layout.vector)+3),2), #Rejection criteria
     rep((max(temp.IRSL.layout.vector)+1),1), #GC
-    rep((max(temp.IRSL.layout.vector)+2),1), #TnTc
+    rep((max(temp.IRSL.layout.vector)+2),1), #TnTx
     rep((max(temp.IRSL.layout.vector)+3),2)) #Rejection criteria
 
-
   ##set layout
-  nf <- layout(
-    matrix(layout.matrix,(max(layout.matrix)/2 +
-                            ifelse(n.loops > 2, 0,2)), 4, byrow = TRUE),
-     widths = c(rep(c(1,1,1,.75),6),c(1,1,1,1)),
-     heights = c(rep(c(1),(2+2*n.loops)),c(0.20, 0.20)))
-
-  ## show the regions that have been allocated to each plot for debug
-  #layout.show(nf)
+  nrows <- max(layout.matrix) / 2 + ifelse(n.loops > 2, 0, 2)
+  mat <- matrix(layout.matrix, nrow = nrows, ncol = 4, byrow = TRUE)
+  graphics::layout(
+    mat = mat,
+    widths = c(rep(c(1, 1, 1, 0.75), 6), 1, 1, 1, 1),
+    heights = c(rep(1, 2 + 2 * n.loops), 0.20, 0.20))
   }
 
   ##(1) INFO PLOT
@@ -466,10 +446,8 @@ analyse_pIRIRSequence <- function(
     text(0.5,0.5, paste(sequence.structure, collapse = "\n"), cex = cex *2)
   }
 
-
   ##(2) set loop
   for(i in 1:n.loops){
-
     ##compile record ids
     temp.id.sel <-
       sort(c(TL.curves.id, IRSL.curves.id[seq(i,length(IRSL.curves.id),by=n.loops)]))
@@ -479,14 +457,12 @@ analyse_pIRIRSequence <- function(
 
     ##(b) grep integral limits as they might be different for different curves
     if(length(signal.integral.min)>1){
-
       temp.signal.integral.min <- signal.integral.min[i]
       temp.signal.integral.max <- signal.integral.max[i]
       temp.background.integral.min <- background.integral.min[i]
       temp.background.integral.max <- background.integral.max[i]
 
     }else{
-
       temp.signal.integral.min <- signal.integral.min
       temp.signal.integral.max <- signal.integral.max
       temp.background.integral.min <- background.integral.min
@@ -494,19 +470,18 @@ analyse_pIRIRSequence <- function(
     }
 
     ##(c) call analysis sequence and plot
-
     ## call single plots
-    if(i == 1){
-      temp.plot.single  <- c(1,2,3,4,6)
-
-    }else if(i == n.loops){
-      temp.plot.single  <- c(2,4,5,6)
-
-   }else{
-      temp.plot.single  <- c(2,4,6)
-   }
+     temp.plot.single <-
+    if(i == 1) {
+      c(1,2,3,4,6)
+    } else if(i == n.loops) {
+      c(2,4,5,6)
+    } else {
+      c(2,4,6)
+    }
 
     ##start analysis
+    par(cex = cex)
     temp.results <- analyse_SAR.CWOSL(
       temp.curves,
       signal.integral.min = temp.signal.integral.min,
@@ -516,7 +491,7 @@ analyse_pIRIRSequence <- function(
       plot = plot,
       dose.points = dose.points,
       plot_singlePanels = temp.plot.single,
-      cex.global = cex,
+      cex = cex,
       ...
     ) ##TODO should be replaced be useful explicit arguments
 
@@ -529,18 +504,15 @@ analyse_pIRIRSequence <- function(
       ##add signal information to the protocol step
       temp.results.pIRIR.De <- as.data.frame(c(
         get_RLum(temp.results, "data"),
-        data.frame(Signal = pIRIR.curve.names[i])
-      ))
+        data.frame(Signal = pIRIR.curve.names[i])))
 
       temp.results.pIRIR.LnLxTnTx <- as.data.frame(c(
         get_RLum(temp.results, "LnLxTnTx.table"),
-        data.frame(Signal = pIRIR.curve.names[i])
-      ))
+        data.frame(Signal = pIRIR.curve.names[i])))
 
       temp.results.pIRIR.rejection.criteria <- as.data.frame(c(
         get_RLum(temp.results, "rejection.criteria"),
-        data.frame(Signal = pIRIR.curve.names[i])
-      ))
+        data.frame(Signal = pIRIR.curve.names[i])))
 
       temp.results.pIRIR.formula <- list(get_RLum(temp.results,
                                                   "Formula"))
@@ -563,7 +535,6 @@ analyse_pIRIRSequence <- function(
       ##merge results
       if (exists("temp.results.final")) {
         temp.results.final <- merge_RLum(list(temp.results.final, temp.results))
-
       } else{
         temp.results.final <- temp.results
       }
@@ -589,36 +560,31 @@ if(plot){
        ylim = c(min(min.LxTx - max.LxTx.Error, 0), max.LxTx + max.LxTx.Error),
        xlab = "Dose [s]",
        ylab = expression(L[x]/T[x]),
+       mgp = c(2, 0.7, 0),
+       tcl = -0.4,
+       xpd = NA,
        main = "Summarised Dose Response Curves")
 
     ##set x for expression evaluation
     x <- seq(0,max(LnLxTnTx.table$Dose)*1.05, length.out = 100)
 
-    for(j in 1:length(pIRIR.curve.names)){
-     ##dose points
-     temp.curve.points <-  LnLxTnTx.table[,c("Dose", "LxTx", "LxTx.Error", "Signal")]
+  for (j in 1:length(pIRIR.curve.names)) {
+    ## dose points
+    curve.points <- LnLxTnTx.table[LnLxTnTx.table$Signal == pIRIR.curve.names[j],
+                                   c("Dose", "LxTx", "LxTx.Error")]
 
-     temp.curve.points <- temp.curve.points[
-       temp.curve.points[,"Signal"] == pIRIR.curve.names[j],
-       c("Dose", "LxTx", "LxTx.Error")]
+    points(curve.points[-1, c("Dose", "LxTx")], col = j, pch = j)
+    segments(x0 = curve.points$Dose[-1],
+             y0 = curve.points$LxTx[-1] - curve.points$LxTx.Error[-1],
+             x1 = curve.points$Dose[-1],
+             y1 = curve.points$LxTx[-1] + curve.points$LxTx.Error[-1],
+             col = j)
 
-     points(temp.curve.points[-1,c("Dose", "LxTx")], col = j, pch = j)
-     segments(x0 = temp.curve.points[-1,c("Dose")],
-              y0 = temp.curve.points[-1,c("LxTx")] -
-                temp.curve.points[-1,c("LxTx.Error")],
-              x1 = temp.curve.points[-1,c("Dose")],
-              y1 = temp.curve.points[-1,c("LxTx")] +
-                temp.curve.points[-1,c("LxTx.Error")],
-              col = j)
-
-     ##De values
-     lines(c(0, get_RLum(temp.results.final, "data")[j,1]),
-           c(temp.curve.points[1,c("LxTx")], temp.curve.points[1,c("LxTx")]),
-           col = j,
-           lty = 2)
-
-     lines(c(rep(get_RLum(temp.results.final, "data")[j,1], 2)),
-           c(temp.curve.points[1,c("LxTx")], 0),
+    ## De values
+    De <- get_RLum(temp.results.final, "data")[j, 1]
+    Lx <- curve.points$LxTx[1]
+    lines(c(-100, De, De, De),
+          c(Lx, Lx, Lx, -100),
            col = j,
            lty = 2)
 
@@ -627,16 +593,15 @@ if(plot){
         temp.results.final, "Formula")[[pIRIR.curve.names[j]]]
 
      try(lines(x, eval(temp.curve.formula), col = j), silent = TRUE)
-    }
+  }
 
     rm(x)
 
     ##plot legend
     legend("bottomright", legend = pIRIR.curve.names,
-           lty = 1, col = c(1:length(pIRIR.curve.names)),
+           lty = 1, col = 1:length(pIRIR.curve.names),
            bty = "n",
-           pch = c(1:length(pIRIR.curve.names))
-           )
+           pch = 1:length(pIRIR.curve.names))
 
     ##plot Tn/Tx curves
     ##select signal
@@ -649,7 +614,6 @@ if(plot){
 
     ##calculate normalised values
     for(j in 1:length(pIRIR.curve.names)){
-
       temp.curve.TnTx.sel <- temp.curve.TnTx[
         temp.curve.TnTx[,"Signal"] == pIRIR.curve.names[j]
         , "TnTx"]
@@ -658,11 +622,12 @@ if(plot){
     }
 
     plot(NA, NA,
-       xlim = c(0,nrow(LnLxTnTx.table)/
-                     n.loops),
+       xlim = c(0, nrow(LnLxTnTx.table) / n.loops),
        ylim = if (anyNA(range(temp.curve.TnTx.matrix))) c(0,1) else range(temp.curve.TnTx.matrix),
        xlab = "# Cycle",
        ylab = expression(T[x]/T[n]),
+       mgp = c(2, 0.7, 0),
+       tcl = -0.4,
        main = "Sensitivity change")
 
     ##zero line
@@ -678,10 +643,9 @@ if(plot){
 
    ##plot legend
    legend("bottomleft", legend = pIRIR.curve.names,
-         lty = 1, col = c(1:length(pIRIR.curve.names)),
+         lty = 1, col = 1:length(pIRIR.curve.names),
          bty = "n",
-         pch = c(1:length(pIRIR.curve.names))
-         )
+         pch = 1:length(pIRIR.curve.names))
 
    ##Rejection criteria
    temp.rejection.criteria <- get_RLum(temp.results.final,
@@ -707,12 +671,13 @@ if(plot){
    axis(side = 1, at = c(-0.2,-0.1,0,0.1,0.2), labels = c("- 0.2", "- 0.1","0/1","+ 0.1", "+ 0.2"))
    ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++##
    ##polygon for recycling ratio
-   text(x = -.4, y = 30, "Recycling ratio", pos = 1, srt = 0)
-   polygon(x = c(-as.numeric(as.character(temp.rc.reycling.ratio$Threshold))[1],
-                -as.numeric(as.character(temp.rc.reycling.ratio$Threshold))[1],
-                as.numeric(as.character(temp.rc.reycling.ratio$Threshold))[1],
-                as.numeric(as.character(temp.rc.reycling.ratio$Threshold))[1]),
-          y = c(21,29,29,21), col = "gray", border = NA)
+   text(x = -0.3, y = 25, "Recycling ratio", pos = 2, offset = 0.5, xpd = TRUE)
+   thres_range <- as.numeric(as.character(temp.rc.reycling.ratio$Threshold))[1] - 1
+   polygon(
+     x = c(-thres_range, -thres_range, thres_range, thres_range),
+     y = c(21, 29, 29, 21),
+     col = "gray",
+     border = NA)
     polygon(x = c(-0.3,-0.3,0.3,0.3) , y = c(21,29,29,21))
 
    ##consider possibility of multiple pIRIR signals and multiple recycling ratios
@@ -729,7 +694,6 @@ if(plot){
                y = 25,
                pch = col.id,
                col = col.id)
-
         }
         col.id <- col.id + 1
    }
@@ -739,7 +703,7 @@ if(plot){
 
    ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++##
    ##polygon for recuperation rate
-   text(x = -.4, y = 20, "Recuperation rate", pos = 1, srt = 0)
+   text(x = -0.3, y = 15, "Recuperation rate", pos = 2, offset = 0.5, xpd = TRUE)
 
    if(length(as.character(temp.rc.recuperation.rate$Threshold))>0){
    polygon(x = c(0,
@@ -751,7 +715,6 @@ if(plot){
    polygon(x = c(-0.3,-0.3,0.3,0.3) , y = c(11,19,19,11))
    polygon(x = c(-0.3,-0.3,0,0) , y = c(11,19,19,11), border = NA, density = 10, angle = 45)
 
-
   for(i in 1:nrow(temp.rc.recuperation.rate)){
 
     points(temp.rc.palaedose.error[i, "Value"],
@@ -763,7 +726,7 @@ if(plot){
 
    ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++##
    ##polygon for palaeodose error
-   text(x = -.4, y = 10, "Palaeodose error", pos = 1, srt = 0)
+   text(x = -0.3, y = 5, "Palaeodose error", pos = 2, offset = 0.5, xpd = TRUE)
    polygon(x = c(0,
                 0,
                 as.numeric(as.character(temp.rc.palaedose.error$Threshold))[1],
@@ -781,21 +744,18 @@ if(plot){
    }
 
    ##add 0 value
-   lines(x = c(0,0), y = c(0,19), lwd = 1.5*cex)
-   lines(x = c(0,0), y = c(20,29), lwd = 1.5*cex)
+   lines(x = c(0, 0), y = c(21, 29), lwd = 2)
+   lines(x = c(0, 0), y = c(11, 19), lwd = 2)
+   lines(x = c(0, 0), y = c(1, 9), lwd = 2)
 
   ##plot legend
-  legend("bottomright", legend = pIRIR.curve.names,
-         col = c(1:length(pIRIR.curve.names)),
+  legend(0.3, 15,
+         legend = pIRIR.curve.names,
+         col = 1:length(pIRIR.curve.names),
+         pch = 1:length(pIRIR.curve.names),
+         yjust = 0.5,
          bty = "n",
-         pch = c(1:length(pIRIR.curve.names)))
-
-
-  ##reset graphic settings
-  if (!plot_singlePanels) {
-    par(def.par)
-  }
-
+         xpd = TRUE)
 }##end plot == TRUE
 
 
@@ -803,5 +763,5 @@ if(plot){
 # Return Values -----------------------------------------------------------
 ##============================================================================##
 
-  return(temp.results.final)
+  temp.results.final
 }
