@@ -1,37 +1,38 @@
 #' @title Apply the Huntley (2006) model
 #'
 #' @description
-#' A function to calculate the expected sample specific fraction of saturation
-#' based on the model of Huntley (2006) using the approach as implemented
+#' The function calculates the expected sample specific fraction of saturation
+#' based on the model of Huntley (2006), using the approach as implemented
 #' in Kars et al. (2008) or Guralnik et al. (2015).
 #'
 #' @details
 #'
 #' This function applies the approach described in Kars et al. (2008) or Guralnik et al. (2015),
-#' which are both developed from the model of Huntley (2006) to calculate the expected sample
+#' which are both developed from the model of Huntley (2006), to calculate the expected sample
 #' specific fraction of saturation of a feldspar and also to calculate fading
 #' corrected age using this model. \eqn{\rho}' (`rhop`), the density of recombination
 #' centres, is a crucial parameter of this model and must be determined
 #' separately from a fading measurement. The function [analyse_FadingMeasurement]
 #' can be used to calculate the sample specific \eqn{\rho}' value.
 #'
-#' **Kars et al. (2008) - Single saturating exponential**
+#' **Kars et al. (2008) -- Single saturating exponential**
 #'
-#' To apply the approach after Kars et al. (2008) use `fit.method = "EXP"`.
+#' To apply the approach after Kars et al. (2008), use `fit.method = "EXP"`.
 #'
 #' Firstly, the unfaded \eqn{D_0} value is determined through applying equation 5 of
 #' Kars et al. (2008) to the measured \eqn{\frac{L_x}{T_x}} data as a function of irradiation
 #' time, and fitting the data with a single saturating exponential of the form:
 #'
-#' \deqn{\frac{L_x}{T_x}(t^*) = A  \phi(t^*) \{1 - exp(-\frac{t^*}{D_0}))\}}
+#' \deqn{\frac{L_x}{T_x}(t^*) = A  \phi(t^*) \{1 - \exp(-\frac{t^*}{D_0}))\}}
 #'
 #' where
 #'
-#' \deqn{\phi(t^*) = exp(-\rho' ln(1.8  \tilde{s} t^*)^3)}
+#' \deqn{\phi(t^*) = \exp(-\rho' \ln(1.8  \tilde{s} t^*)^3)}
 #'
 #' after King et al. (2016) where \eqn{A} is a pre-exponential factor,
 #' \eqn{t^*} (s) is the irradiation time, starting at the mid-point of
-#' irradiation (Auclair et al. 2003) and \eqn{\tilde{s}} (\eqn{3\times10^{15}} s\eqn{^{-1}}) is the athermal frequency factor after Huntley (2006). \cr
+#' irradiation (Auclair et al. 2003) and \eqn{\tilde{s}} (\eqn{3\times10^{15}} s\eqn{^{-1}})
+#' is the athermal frequency factor after Huntley (2006).
 #'
 #' Using fit parameters \eqn{A} and \eqn{D_0}, the function then computes a natural dose
 #' response curve using the environmental dose rate, \eqn{\dot{D}} (Gy/s) and equations
@@ -40,12 +41,12 @@
 #' be interpolated onto this curve to determine the fading corrected
 #' \eqn{D_e} value, from which the fading corrected age is calculated.
 #'
-#' **Guralnik et al. (2015) - General-order kinetics**
+#' **Guralnik et al. (2015) -- General-order kinetics**
 #'
 #' To apply the approach after Guralnik et al. (2015) use `fit.method = "GOK"`.
 #'
 #' The approach of Guralnik et al. (2015) is very similar to that of
-#' Kars et al. (2008), but instead of using a single saturating exponential
+#' Kars et al. (2008) but, instead of using a single saturating exponential,
 #' the model fits a general-order kinetics function of the form:
 #'
 #' \deqn{\frac{L_x}{T_x}(t^*) = A \phi (t^*)(1 - (1 + (\frac{1}{D_0}) t^* c)^{-1/c})}
@@ -68,9 +69,10 @@
 #' **Uncertainties**
 #'
 #' Uncertainties are reported at \eqn{1\sigma} and are assumed to be normally
-#' distributed and are estimated using Monte-Carlo re-sampling (`n.MC = 1000`)
-#' of \eqn{\rho}' and \eqn{\frac{L_x}{T_x}} during dose response curve fitting, and of \eqn{\rho}'
-#' in the derivation of (\eqn{n/N}) and (n/N)_SS.
+#' distributed and are estimated using Monte-Carlo re-sampling (`n.MC = 10000`
+#' by default) of \eqn{\rho}' and \eqn{\frac{L_x}{T_x}} during dose response
+#' curve fitting, and of \eqn{\rho}' #' in the derivation of (\eqn{n/N}) and
+#' (n/N)_SS.
 #'
 #' **Age calculated from 2D0 of the simulated natural DRC**
 #'
@@ -84,12 +86,13 @@
 #'
 #' @param data [data.frame] (**required**):
 #' A `data.frame` with one of the following structures:
-#' - A **three column** data frame with numeric values on a) dose (s), b) `LxTx` and
-#' c) `LxTx` error.
-#' - If a **two column** data frame is provided it is automatically
-#' assumed that errors on `LxTx` are missing. A third column will be attached
-#' with an arbitrary 5 % error on the provided `LxTx` values.
-#' - Can also be a **wide table**, i.e. a [data.frame] with a number of columns divisible by 3
+#' - **three columns** with numeric values for dose (s), `LxTx` and `LxTx`
+#' error, in this order.
+#' - **two columns** with numeric values for dose (s) and `LxTx`, in this order.
+#' This assumes that errors on `LxTx` are missing, and a third column will be
+#' automatically attached with an arbitrary 5 % error on the provided `LxTx`
+#' values.
+#' - **wide table**, i.e. a [data.frame] with a number of columns divisible by 3
 #' and where each triplet has the aforementioned column structure.
 #'
 #' ```
@@ -103,13 +106,12 @@
 #' [x, ]|  Rx     | LxTx | LxTx error |
 #'
 #' ```
-#' **NOTE:** The function assumes the first row of the function to be the
-#' `Ln/Tn`-value. If you want to provide more than one `Ln/Tn`-value consider
-#' using the argument `LnTn`.
+#' **NOTE:** The function assumes the first row of the data to be the
+#' `Ln/Tn`-value. If you want to provide more than one `Ln/Tn`-values, consider
+#' using argument `LnTn`.
 #'
 #' @param LnTn [data.frame] (*optional*):
-#' This argument should **only** be used to provide more than one `Ln/Tn`-value.
-#' It assumes a two column data frame with the following structure:
+#' A two column data frame with the following structure:
 #'
 #' ```
 #'      |  LnTn  |  LnTn error  |
@@ -127,33 +129,38 @@
 #' this value must be calculated beforehand and used in the first row in the
 #' data frame for argument `data`.
 #'
-#' **NOTE:** If you provide `LnTn`-values with this argument the data frame
-#' for the `data`-argument **must not** contain any `LnTn`-values!
+#' **NOTE:** This argument should **only** be used to provide more than one
+#' `Ln/Tn`-value. If you provide `LnTn`-values with this argument, the data
+#' frame for the `data`-argument **must not** contain any `LnTn`-values.
 #'
 #' @param rhop [numeric] (**required**):
-#' The density of recombination centres (\eqn{\rho}') and its error (see Huntley 2006),
-#' given as numeric vector of length two. Note that \eqn{\rho}' must **not** be
+#' A vector of length 2 for the density of recombination centres (\eqn{\rho}')
+#' and its error (see Huntley 2006). Note that \eqn{\rho}' must **not** be
 #' provided as the common logarithm. Example: `rhop = c(2.92e-06, 4.93e-07)`.
 #'
 #' @param ddot [numeric] (**required**):
-#' Environmental dose rate and its error, given as a numeric vector of length two.
+#' A vector of length 2 for the environmental dose rate and its error.
 #' Expected unit: Gy/ka. Example: `ddot = c(3.7, 0.4)`.
 #'
 #' @param readerDdot [numeric] (**required**):
-#' Dose rate of the irradiation source of the OSL reader and its error,
-#' given as a numeric vector of length two.
+#' A vector of length 2 for the dose rate of the irradiation source of the OSL
+#' reader and its error.
 #' Expected unit: Gy/s. Example: `readerDdot = c(0.08, 0.01)`.
 #'
+#' @param normalise [logical] (*with default*): If `TRUE` (the default) all
+#' measured and computed \eqn{\frac{L_x}{T_x}} values are normalised by the
+#' pre-exponential factor `A` (see details).
+#'
 #' @param fit.method [character] (*with default*):
-#' Fit function of the dose response curve. Can either be `EXP` (the default)
-#' or `GOK`. Note that `EXP` (single saturating exponential) is the original
+#' Fit function of the dose response curve. Can either be `"EXP"` (default) or
+#' `"GOK"`. Note that `"EXP"` (single saturating exponential) is the original
 #' function the model after Huntley (2006) and Kars et al. (2008) was
-#' designed to use. The use of a general-order kinetics function (`GOK`)
+#' designed to use. The use of a general-order kinetics function (`"GOK"`)
 #' is an experimental adaptation of the model and should be used
 #' with great care.
 #'
 #' @param lower.bounds [numeric] (*with default*):
-#' A vector of length 4 that contains the lower bound values to be applied
+#' A vector of length 4 for the values of the lower bounds to be applied
 #' when fitting the models with [minpack.lm::nlsLM]. In most cases, the
 #' default values (`c(-Inf, -Inf, -Inf, -Inf)`) are appropriate for finding
 #' a best fit, but sometimes it may be useful to restrict the lower bounds to
@@ -162,7 +169,9 @@
 #' when `fit.method = "EXP"`). More details can be found in
 #' [fit_DoseResponseCurve].
 #'
-#' @param normalise [logical] (*with default*): If `TRUE` (the default) all measured and computed \eqn{\frac{L_x}{T_x}} values are normalised by the pre-exponential factor `A` (see details).
+#' @param cores [integer] (*with default*):
+#' The number of cores to use. This will be capped to the number of available
+#' cores if set to too high.
 #'
 #' @param summary [logical] (*with default*):
 #' If `TRUE` (the default) various parameters provided by the user
@@ -174,14 +183,18 @@
 #'
 #' @param ...
 #' Further parameters:
-#' - `verbose` [logical]: Show or hide console output
-#' - `n.MC` [numeric]: Number of Monte Carlo iterations (default = `10000`).
+#' - `verbose` [logical]: Enable/disable output to the terminal (default = `TRUE`)
+#' - `n.MC` [numeric]: Number of Monte Carlo iterations (default = 10000)
+#' - `cex` [numeric]: Scaling of the plot (default = 1)
+#' - `maxiter` [numeric]: Number of iteration limits for nls fitting
+#' - `trace` [logical]: Enable/disable value tracing the terminal during fitting
 #' **Note** that it is generally advised to have a large number of Monte Carlo
 #' iterations for the results to converge. Decreasing the number of iterations
 #' will often result in unstable estimates.
 #'
-#' All other arguments are passed to [plot] and [fit_DoseResponseCurve] (in particular
-#' `mode` for the fit mode and `fit.force_through_origin`)
+#' All other arguments are passed to [plot] and [fit_DoseResponseCurve] (in
+#' particular `mode` for the De calculation mode, `fit.force_through_origin`,
+#' and `fit.bounds`).
 #'
 #' @return An [RLum.Results-class] object is returned:
 #'
@@ -194,7 +207,7 @@
 #' `Ln` \tab [numeric] \tab Ln and its error \cr
 #' `LxTx_tables` \tab `list` \tab A `list` of `data.frames` containing data on dose,
 #'  LxTx and LxTx error for each of the dose response curves.
-#'  Note that these **do not** contain the natural Ln signal, which is provided separately. \cr
+#'  Note that these **do not** contain the natural `Ln` signal, which is provided separately. \cr
 #' `fits` \tab `list` \tab A `list` of `nls` objects produced by [minpack.lm::nlsLM] when fitting the dose response curves \cr
 #' }
 #'
@@ -206,17 +219,18 @@
 #' `args` \tab `list` \tab arguments of the original function call \cr
 #' }
 #'
-#' @section Function version: 0.4.5
+#' @section Function version: 0.4.7
 #'
 #' @author
 #' Georgina E. King, University of Lausanne (Switzerland) \cr
 #' Christoph Burow, University of Cologne (Germany) \cr
-#' Sebastian Kreutzer, Ruprecht-Karl University of Heidelberg (Germany)
+#' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany) \cr
+#' Marco Colombo, Institute of Geography, Heidelberg University (Germany)
 #'
 #' @keywords datagen
 #'
 #' @note This function has BETA status, in particular for the GOK implementation. Please verify
-#' your results carefully
+#' your results carefully.
 #'
 #' @references
 #'
@@ -268,7 +282,6 @@
 #'  readerDdot = readerDdot,
 #'  n.MC = 25)
 #'
-#'
 #' \dontrun{
 #' # You can also provide LnTn values separately via the 'LnTn' argument.
 #' # Note, however, that the data frame for 'data' must then NOT contain
@@ -287,17 +300,17 @@
 #'  readerDdot = readerDdot,
 #'  n.MC = 25)
 #' }
-#' @md
 #' @export
 calc_Huntley2006 <- function(
     data,
     LnTn = NULL,
-    rhop,
-    ddot,
-    readerDdot,
+    rhop = NULL,
+    ddot = NULL,
+    readerDdot = NULL,
     normalise = TRUE,
     fit.method = c("EXP", "GOK"),
     lower.bounds = c(-Inf, -Inf, -Inf, -Inf),
+    cores = 1,
     summary = TRUE,
     plot = TRUE,
     ...
@@ -309,8 +322,11 @@ calc_Huntley2006 <- function(
 
   .validate_class(data, "data.frame")
   .validate_not_empty(data)
+  .validate_class(LnTn, "data.frame", null.ok = TRUE)
   fit.method <- .validate_args(fit.method, c("EXP", "GOK"))
-  .validate_length(lower.bounds, 4)
+  .validate_class(lower.bounds, "numeric", length = 4)
+  .validate_logical_scalar(summary)
+  .validate_logical_scalar(plot)
 
   ## Check 'data'
   if (ncol(data) == 2) {
@@ -323,7 +339,6 @@ calc_Huntley2006 <- function(
 
   ## Check if 'LnTn' is used and overwrite 'data'
   if (!is.null(LnTn)) {
-    .validate_class(LnTn, "data.frame")
     if (ncol(LnTn) != 2)
       .throw_error("'LnTn' should be a data frame with 2 columns")
     if (ncol(data) > 3)
@@ -379,44 +394,51 @@ calc_Huntley2006 <- function(
   # check if numeric
   if (is.numeric(rhop)) {
     .validate_length(rhop, 2)
-
-    # alternatively, an RLum.Results object produced by analyse_FadingMeasurement()
-    # can be provided
-  } else if (inherits(rhop, "RLum.Results")) {
-
-    if (rhop@originator == "analyse_FadingMeasurement")
-      rhop <- c(rhop@data$rho_prime$MEAN,
-                rhop@data$rho_prime$SD)
-    else
-      .throw_error("'rhop' accepts only RLum.Results objects produced ",
-                   "by 'analyse_FadingMeasurement()'")
+  } else {
+    ## alternatively, an RLum.Results object produced by
+    ## analyse_FadingMeasurement() can be provided
+    .validate_originator(rhop, "analyse_FadingMeasurement")
+    rhop <- c(rhop@data$rho_prime$MEAN, rhop@data$rho_prime$SD)
   }
 
   # check if 'rhop' is actually a positive value
-  if (anyNA(rhop) || !rhop[1] > 0 || any(is.infinite(rhop))) {
-    .throw_error("'rhop' must be a positive number. Provided value ",
-                 "was: ", signif(rhop[1], 3), " \u2213 ", signif(rhop[2], 3))
+  if (anyNA(rhop) || rhop[1] <= 0 || any(is.infinite(rhop))) {
+    .throw_error("'rhop' must be a positive number, the provided value ",
+                 "was ", signif(rhop[1], 3), " \u00B1 ", signif(rhop[2], 3))
   }
 
   ## Check ddot & readerDdot
-  .validate_class(ddot, "numeric")
-  .validate_length(ddot, 2)
-  .validate_class(readerDdot, "numeric")
-  .validate_length(readerDdot, 2)
+  .validate_class(ddot, "numeric", length = 2)
+  .validate_class(readerDdot, "numeric", length = 2)
+
+  ## set up the parallel cluster
+  .validate_positive_scalar(cores, int = TRUE)
+  available.cores <- parallel::detectCores()
+  cores <- min(cores, available.cores)
+  cl <- parallel::makeCluster(cores)
+  on.exit(parallel::stopCluster(cl), add = TRUE)
 
   ## Settings ------------------------------------------------------------------
+  extraArgs <- list(...)
   settings <- modifyList(
     list(
       verbose = TRUE,
-      n.MC = 10000),
-    list(...))
+      n.MC = 10000,
+      plot_all_DRC = plot,
+      maxiter = 500,
+      trace = FALSE),
+    extraArgs)
+
+  .validate_positive_scalar(settings$n.MC, int = TRUE, name = "'n.MC'")
+  if (settings$n.MC == 1) {
+    settings$n.MC <- max(settings$n.MC, 2)
+    extraArgs$n.MC <- settings$n.MC
+  }
 
   ## Define Constants ----------------------------------------------------------
-  kb <- 8.617343 * 1e-5
-  alpha <- 1
+
   Hs <- 3e15 # s value after Huntley (2006)
-  Ma <- 1e6 * 365.25 * 24 * 3600 #in seconds
-  ka <- Ma / 1000 #in seconds
+  ka <- 1e3 * .const$year_s # in seconds
 
   ## Define Functions ----------------------------------------------------------
   # fit data using using Eq 5. from Kars et al (2008) employing
@@ -446,13 +468,12 @@ calc_Huntley2006 <- function(
   ## around 2.2, so setting it to 3 should be enough in general, and 1000
   ## points seem also enough; in any case, we let the user override it
   rprime <- seq(0.01, 3, length.out = 1000)
-  if ("rprime" %in% names(list(...))) {
-    rprime <- list(...)$rprime
+  if ("rprime" %in% names(extraArgs)) {
+    rprime <- extraArgs$rprime
     .validate_class(rprime, "numeric")
   }
 
   ## (1) MEASURED ----------------------------------------------------
-
   data.tmp <- data
   data.tmp[ ,1] <- data.tmp[ ,1] * readerDdot
 
@@ -463,23 +484,28 @@ calc_Huntley2006 <- function(
     fit.force_through_origin = FALSE,
     verbose = FALSE)
 
-  GC.settings <- modifyList(GC.settings, list(...))
+  GC.settings <- modifyList(GC.settings, extraArgs)
   GC.settings$object <- data.tmp
   GC.settings$verbose <- FALSE
 
-  ## take of force_through origin settings
+  ## unset the n.MC argument so that fit_DoseResponseCurve() won't use it,
+  ## which would result in a large performance slowdown (#867)
+  GC.settings$n.MC <- NULL
+
+  fit.bounds <- GC.settings$fit.bounds
   force_through_origin <- GC.settings$fit.force_through_origin
   mode_is_extrapolation <- GC.settings$mode == "extrapolation"
 
   ## call the fitting
   GC.measured <- try(do.call(fit_DoseResponseCurve, GC.settings))
 
-  if (inherits(GC.measured$Fit, "try-error"))
-    .throw_error("Unable to fit growth curve to measured data, try setting ",
-                 "'fit.bounds = FALSE'")
-  if (plot) {
+  if (inherits(GC.measured$Fit, "try-error")) {
+    .throw_error("Unable to fit growth curve to measured data",
+                 ifelse(fit.bounds, ", try setting 'fit.bounds = FALSE'", ""))
+  }
+  if (settings$plot_all_DRC) {
     plot_DoseResponseCurve(GC.measured, main = "Measured dose response curve",
-                           xlab = "Dose (Gy)", verbose = FALSE)
+                           xlab = "Dose [Gy]", verbose = FALSE)
   }
 
   # extract results and calculate age
@@ -523,29 +549,74 @@ calc_Huntley2006 <- function(
   }
 
   ## do the fitting
-  fitcoef <- do.call(rbind, sapply(rhop_MC, function(rhop_i) {
+  fitcoef <- do.call(rbind, parallel::parLapply(cl, rhop_MC, function(rhop_i) {
     fit_sim <- try({
       minpack.lm::nlsLM(
        formula = model,
        start = start,
        lower = lower.bounds,
        upper = upper.bounds,
+       trace = settings$trace,
        control = list(maxiter = settings$maxiter))
     }, silent = TRUE)
 
     if (!inherits(fit_sim, "try-error"))
       coefs <- coef(fit_sim)
-    else
-      .throw_error("Could not fit simulated curve, check suitability of ",
-                   "model and parameters")
+    else {
+      ## As the fit from the given starting values failed, try again by fixing
+      ## D0 at different values to make the fitting a bit easier. Values are
+      ## spaced logarithmically so that the ratio between consecutive values
+      ## is constant.
+      # all.D0 <- c(start$D0, exp(seq(log(start$D0 / 10), log(start$D0 * 10),
+      #                               length.out = 99)))
+      ## ALTERNATIVE: sample from a gamma distribution; with only 10, this should
+      ## be enough based on tests RLumSK (2025-03-29)
+      all.D0 <- stats::rgamma(10, shape = start$D0)
+      fit.D0 <- lapply(1:length(all.D0), function(idx) {
+        D0 <- all.D0[idx]
+        t <- try(minpack.lm::nlsLM(
+                  formula = model,
+                  start = start[-2],
+                  lower = lower.bounds[-2],
+                  upper = upper.bounds[-2],
+                  control = list(maxiter = settings$maxiter)),
+                 silent = TRUE)
+
+        if (inherits(t, "try-error"))
+          return(NULL)
+        return(t)
+      })
+
+      ## pick the one with the best fit after removing those that didn't fit
+      fit.D0 <- .rm_NULL_elements(fit.D0)
+
+      ## if also this fails, we should throw an error, but as we are inside
+      ## a parallel region, we cannot do that cleanly, so we return NA and
+      ## only afterwards we'll throw the error
+      if (length(fit.D0) == 0)
+        return(NA)
+
+      ## extract the coefficients from the one with the best fit
+      fit.D0 <- fit.D0[[which.min(vapply(fit.D0, stats::deviance, numeric(1)))]]
+      coefs <- coef(fit.D0)
+
+      ## add back the coefficient for D0
+      D0 <- environment(fit.D0$m$predict)$env$D0
+      coefs <- c(coefs[1], D0 = D0, coefs[2:length(coefs)])
+    }
 
     return(coefs)
-  }, simplify = FALSE))
+  }))
+
+  ## check if errors had occurred during model fitting
+  if (anyNA(fitcoef)) {
+    .throw_error("Could not fit simulated curve, check suitability of ",
+                 "model and parameters")
+  }
 
   # final fit for export
   # fit_simulated <- minpack.lm::nlsLM(LxTx.measured ~ a * theta(dosetime, rhop[1]) * (1 - exp(-dosetime / D0)),
   #                      start = list(a = max(LxTx.measured), D0 = D0.measured / readerDdot))
-
   # scaling factor
   A <- mean(fitcoef[, "a"], na.rm = TRUE)
   A.error <- sd(fitcoef[ ,"a"], na.rm = TRUE)
@@ -559,10 +630,10 @@ calc_Huntley2006 <- function(
   # natdosetime <- seq(0, 1e14, length.out = settings$n.MC)
   # natdosetimeGray <- natdosetime * ddot / ka
 
-  # calculate D0 dose in seconds
-  computedD0 <- (fitcoef[ ,"D0"] * readerDdot) / (ddot / ka)
-
   # Legacy code:
+  # calculate D0 dose in seconds
+  # computedD0 <- (fitcoef[ ,"D0"] * readerDdot) / (ddot / ka)
+  #
   # This is an older approximation to calculate the natural dose response curve,
   # which sometimes tended to slightly underestimate nN_ss. This is now replaced
   # with the newer approach below.
@@ -576,7 +647,6 @@ calc_Huntley2006 <- function(
   natdosetime <- natdosetimeGray
   pr <- 3 * rprime^2 * exp(-rprime^3) # Huntley 2006, eq. 3
   K <- Hs * exp(-rhop[1]^-(1/3) * rprime)
-  TermA <- matrix(NA, nrow = length(rprime), ncol = length(natdosetime))
   UFD0 <- mean(fitcoef[ ,"D0"], na.rm = TRUE) * readerDdot
 
   c_val <- mean(fitcoef[, "c"], na.rm = TRUE)
@@ -587,23 +657,35 @@ calc_Huntley2006 <- function(
     d_gok <- mean(fitcoef[ ,"d"], na.rm = TRUE)
   }
 
-  for (k in 1:length(rprime)) {
-    if (fit.method[1] == "EXP") {
-      TermA[k, ] <- A * pr[k] *
-        ((ddots / UFD0) / (ddots / UFD0 + K[k]) *
-         (1 - exp(-(natdosetime + c_val) * (1 / UFD0 + K[k] / ddots))))
-    } else if (fit.method[1] == "GOK") {
-      TermA[k, ] <- A * pr[k] * (ddots / UFD0) / (ddots / UFD0 + K[k]) *
-        (d_gok-(1+(1 / UFD0 + K[k] / ddots) * natdosetime * c_val)^(-1 / c_val))
-    }
+  ## the original formulation used:
+  ##  (1) (ddots / UFD0) / (ddots / UFD0 + K[k])
+  ##  (2) 1 / UFD0 + K[k] / ddots
+  ## which are algebraically equivalent to:
+  ##  (1) ddots / (ddots + UFD0 * K[k]) -> scaled.ddots
+  ##  (2) 1 / scaled.dots / UFD0
+  scaled.ddots <- ddots / (ddots + UFD0 * K)
+  A.pr.ddots <- A * pr * scaled.ddots
+  inv.UFD0.K <- 1 / scaled.ddots / UFD0
+  if (fit.method == "EXP") {
+    fun <- function(k) A.pr.ddots[k] *
+        (1 - exp(-(natdosetime + c_val) * inv.UFD0.K[k]))
+  } else if (fit.method == "GOK") {
+    fun <- function(k) A.pr.ddots[k] *
+        (d_gok - (1 + inv.UFD0.K[k] * natdosetime * c_val)^(-1 / c_val))
   }
-
+  TermA <- t(vapply(seq_along(rprime), fun, USE.NAMES = FALSE,
+                    FUN.VALUE = numeric(length(natdosetime))))
 
   LxTx.sim <- colSums(TermA) / sum(pr)
   # warning("LxTx Curve (new): ", round(max(LxTx.sim) / A, 3), call. = FALSE)
 
   # calculate Age
   positive <- which(diff(LxTx.sim) > 0)
+  if (length(positive) == 0) {
+    .throw_error("All simulated Lx/Tx values are identical and approximately ",
+                 "zero. Please verify the accuracy of your rho' value, as this ",
+                 "is likely too large and may not be realistic")
+  }
 
   data.unfaded <- data.frame(
     dose = c(0, natdosetimeGray[positive]),
@@ -615,15 +697,13 @@ calc_Huntley2006 <- function(
   GC.settings$object <- data.unfaded
 
   ## calculate simulated DE
-  suppressWarnings(
-    GC.simulated <- try(do.call(fit_DoseResponseCurve, GC.settings))
-  )
+  GC.simulated <- suppressWarnings(try(do.call(fit_DoseResponseCurve, GC.settings)))
 
   fit_simulated <- NA
   De.sim <- De.error.sim <- D0.sim.Gy <- D0.sim.Gy.error <- NA
   Age.sim <- Age.sim.error <- Age.sim.2D0 <- Age.sim.2D0.error <- NA
   if (!inherits(GC.simulated, "try-error")) {
-    if (plot) {
+    if (settings$plot_all_DRC) {
       plot_DoseResponseCurve(GC.simulated, main = "Simulated dose response curve",
                              xlab = "Dose (Gy)", verbose = FALSE)
     }
@@ -649,8 +729,8 @@ calc_Huntley2006 <- function(
   }
 
   if (Ln > max(LxTx.sim) * 1.1)
-    .throw_warning("Ln is >10 % larger than the maximum computed LxTx value.",
-                   " The De and age should be regarded as infinite estimates.")
+    .throw_warning("Ln is >10 % larger than the maximum computed LxTx value, ",
+                   "the De and age should be regarded as infinite estimates")
 
   if (Ln < min(LxTx.sim) * 0.95 && !mode_is_extrapolation)
     .throw_warning("Ln/Tn is smaller than the minimum computed LxTx value: ",
@@ -680,16 +760,16 @@ calc_Huntley2006 <- function(
   ##
   ## Now, substituting (2') into (3) we get:
   ##
-  ##  (3') tau <- ((1 / Hs) * exp(1)^(rprime / (rhop_MC[i]^(1 / 3))) / ka
+  ##  (3') tau <- (1 / Hs) * exp(rprime / (rhop_MC[i]^(1 / 3))) / ka
   ##
   ## The current formulation then follows:
   ##
   ##  rho_i <- rhop_MC[i]^(1 / 3)
-  ##  tau <- ((1 / Hs) * exp(1)^(rprime / rho_i)) / ka
+  ##  tau <- (1 / Hs) * exp(rprime / rho_i) / ka
 
   rho_MC <- rhop_MC^(1 / 3)
   nN_SS_MC <- mapply(function(rho_i, ddot_i, UFD0_i) {
-    tau <- ((1 / Hs) * exp(1)^(rprime / rho_i)) / ka
+    tau <- (1 / Hs) * exp(rprime / rho_i) / ka
     Ls <- 1 / (1 + UFD0_i / (ddot_i * tau))
     Lstrap <- (pr * Ls) / sum(pr)
 
@@ -710,14 +790,14 @@ calc_Huntley2006 <- function(
 
   ## (3) UNFADED ---------------------------------------------------------------
   LxTx.unfaded <- LxTx.measured / theta(dosetime, rhop[1])
-  LxTx.unfaded[is.nan((LxTx.unfaded))] <- 0
-  LxTx.unfaded[is.infinite(LxTx.unfaded)] <- 0
+
+  ## set Inf and NaN values to 0
+  LxTx.unfaded[!is.finite(LxTx.unfaded)] <- 0
   dosetimeGray <- dosetime * readerDdot
 
   ## run this first model also for GOK as in general it provides more
   ## stable estimates that can be used as starting point for GOK
-  if (fit.method[1] == "EXP" || fit.method[1] == "GOK") {
-    fit_unfaded <- try(minpack.lm::nlsLM(
+  fit_unfaded <- try(minpack.lm::nlsLM(
       LxTx.unfaded ~ a * (1 - exp(-(dosetimeGray + c) / D0)),
       start = list(
         a = coef(fit_simulated)[["a"]],
@@ -729,8 +809,8 @@ calc_Huntley2006 <- function(
            c(Inf, max(dosetimeGray), Inf)
           },
         lower = lower.bounds[1:3],
+        trace = settings$trace,
       control = list(maxiter = settings$maxiter)), silent = TRUE)
-  }
 
   ## if this fit has failed, what we do depends on fit.method:
   ## - for EXP, this error is irrecoverable
@@ -749,7 +829,7 @@ calc_Huntley2006 <- function(
       start = list(
         a = coef(fit_start)[["a"]],
         D0 = coef(fit_start)[["D0"]],
-        c = coef(fit_start)[["c"]],
+        c = max(coef(fit_start)[["c"]], 1),
         d = coef(fit_simulated)[["d"]]),
       upper = if(force_through_origin) {
         c(a = Inf, D0 = max(dosetimeGray), c = Inf, d = 1)
@@ -769,16 +849,14 @@ calc_Huntley2006 <- function(
   ## Create LxTx tables --------------------------------------------------------
   # normalise by A (saturation point of the un-faded curve)
   if (normalise) {
-    LxTx.measured.relErr <- (LxTx.measured.error / LxTx.measured)
     LxTx.measured <- LxTx.measured / A
-    LxTx.measured.error <- LxTx.measured * LxTx.measured.relErr
+    LxTx.measured.error <- LxTx.measured.error / A
 
     LxTx.sim <- LxTx.sim / A
     LxTx.unfaded <- LxTx.unfaded / A
 
-    Ln.relErr <- Ln.error / Ln
     Ln <- Ln / A
-    Ln.error <- Ln * Ln.relErr
+    Ln.error <- Ln.error / A
   }
 
   # combine all computed LxTx values
@@ -800,21 +878,20 @@ calc_Huntley2006 <- function(
   ## Plot settings -------------------------------------------------------------
   plot.settings <- modifyList(list(
     main = "Dose response curves",
-    xlab = "Dose (Gy)",
-    ylab = ifelse(normalise, "normalised LxTx (a.u.)", "LxTx (a.u.)")
-  ), list(...))
+    cex = 1,
+    xlab = "Dose [Gy]",
+    ylab = ifelse(normalise, "normalised LxTx [a.u.]", "LxTx [a.u.]")
+  ), extraArgs)
 
   ## Plotting ------------------------------------------------------------------
   if (plot) {
     ### par settings ---------
-    # set plot parameters
-    par.old.full <- par(no.readonly = TRUE)
+    par.default <- .par_defaults()
+    on.exit(par(par.default), add = TRUE)
 
     # set graphical parameters
-    par(mfrow = c(1,1), mar = c(4.5, 4, 4, 4), cex = 0.8,
-        oma = c(0, 9, 0, 9))
-    if (summary)
-      par(oma = c(0, 3, 0, 9))
+    par(mfrow = c(1,1), mar = c(4.5, 4, 4, 4), cex = 0.8 * plot.settings$cex,
+        oma = c(0, 0, 0, if (summary) 12 / plot.settings$cex else 0))
 
     # Find a good estimate of the x-axis limits
     if (mode_is_extrapolation && !force_through_origin) {
@@ -822,9 +899,7 @@ calc_Huntley2006 <- function(
       De.measured <- -De.measured
     }
 
-    xlim <- range(pretty(dosetimeGray))
-    if (!is.na(De.sim) & De.sim > xlim[2])
-      xlim <- range(pretty(c(min(dosetimeGray), De.sim)))
+    xlim <- range(pretty(c(dosetimeGray, De.sim), n = 15))
 
     # Create figure after Kars et al. (2008) contrasting the dose response curves
     ## open plot window ------------
@@ -839,9 +914,8 @@ calc_Huntley2006 <- function(
       xlim = xlim
     )
 
-    ##add ablines for extrapolation
-    if (mode_is_extrapolation)
-      abline(v = 0, h = 0, col = "gray")
+    ## add horizontal line at zero
+    abline(v = 0, h = 0, col = "gray")
 
     # LxTx error bars
     segments(x0 = dosetimeGray[dosetimeGray >= 0],
@@ -879,7 +953,7 @@ calc_Huntley2006 <- function(
 
     # Ln and DE as points
     points(x = if (mode_is_extrapolation)
-                rep(De.measured, 2)
+                 rep(De.measured, 2)
                else
                  c(0, De.measured),
            y = if (mode_is_extrapolation)
@@ -898,7 +972,7 @@ calc_Huntley2006 <- function(
     lines(x = if (mode_is_extrapolation)
                 c(0, min(c(De.measured, De.sim), na.rm = TRUE))
               else
-                c(par()$usr[1], max(c(De.measured, De.sim), na.rm = TRUE)),
+                c(par()$usr[1], max(c(De.measured, De.sim, 0), na.rm = TRUE)),
           y = c(Ln, Ln),
           col = "red ", lty = 3)
 
@@ -914,6 +988,7 @@ calc_Huntley2006 <- function(
              "Unfaded DRC",
              "Measured DRC",
              "Simulated natural DRC"),
+           inset = c(0, 0.04),
            lty = c(5, 1, 3),
            bty = "n",
            cex = 0.8)
@@ -962,29 +1037,18 @@ calc_Huntley2006 <- function(
         bquote(dot(D)["Reader"] == .(format(readerDdot, digits = 2, nsmall = 2)) %+-% .(round(as.numeric(format(readerDdot.error, digits = 3, nsmall = 3)), 3)) ~ frac(Gy, s)),
         bquote(log[10]~(rho~"'") == .(format(log10(rhop[1]), digits = 2, nsmall = 2)) %+-% .(round(as.numeric(format(rhop[2] / (rhop[1] * log(10, base = exp(1))), digits = 2, nsmall = 2)), 2)) ),
         bquote(bgroup("(", frac(n, N), ")") == .(format(nN, digits = 2, nsmall = 2)) %+-% .(round(as.numeric(format(nN.error, digits = 2, nsmall = 2)), 2)) ),
-        bquote(bgroup("(", frac(n, N), ")")[SS] == .(format(nN_SS, digits = 2, nsmall = 2)) %+-% .(round(as.numeric(format(nN_SS.error, digits = 2, nsmall = 2)), 2)) ),
+        bquote(bgroup("(", frac(n, N), ")")[SS] == .(format(nN_SS, digits = 2, nsmall = 2)) %+-% .(round(nN_SS.error, 2))),
         bquote(D["E,sim"] == .(format(De.sim, digits = 1, nsmall = 0)) %+-% .(format(De.error.sim, digits = 1, nsmall = 0)) ~ Gy),
         bquote(D["0,sim"] == .(format(D0.sim.Gy, digits = 1, nsmall = 0)) %+-% .(format(D0.sim.Gy.error, digits = 1, nsmall = 0)) ~ Gy),
         bquote(Age["sim"] == .(format(Age.sim, digits = 1, nsmall = 0)) %+-% .(format(Age.sim.error, digits = 1, nsmall = 0)) ~ ka)
       )
 
       # each of the labels is positioned at 1/10 of the available y-axis space
-      ypos <- seq(range(axTicks(2))[2], range(axTicks(2))[1], length.out = 10)[1:length(labels.text)]
-
-      # allow overprinting
-      par(xpd = NA)
-
-      # add labels iteratively
-      mapply(function(label, pos) {
-        text(x = par("usr")[2] * 1.05,
-             y = pos,
-             labels = label,
-             pos = 4)
-      }, labels.text, ypos)
+      ypos <- seq(range(axTicks(2))[2], range(axTicks(2))[1], length.out = 10)
+      for (i in seq_along(labels.text))
+        mtext(labels.text[[i]], at = ypos[i],
+              side = 4, line = 1, las = 1, padj = 1)
     }
-
-    # recover plot parameters
-    on.exit(par(par.old.full), add = TRUE)
   }
 
   ## Results -------------------------------------------------------------------
@@ -992,26 +1056,26 @@ calc_Huntley2006 <- function(
     class = "RLum.Results",
     data = list(
       results = data.frame(
-        "nN" = nN,
-        "nN.error" = nN.error,
-        "nN_SS" = nN_SS,
-        "nN_SS.error" = nN_SS.error,
-        "Meas_De" = abs(De.measured),
-        "Meas_De.error" = De.measured.error,
-        "Meas_D0" =  D0.measured,
-        "Meas_D0.error" = D0.measured.error,
-        "Meas_Age" = Age.measured,
-        "Meas_Age.error" = Age.measured.error,
-        "Sim_De" = De.sim,
-        "Sim_De.error" = De.error.sim,
-        "Sim_D0" = D0.sim.Gy,
-        "Sim_D0.error" = D0.sim.Gy.error,
-        "Sim_Age" = Age.sim,
-        "Sim_Age.error" = Age.sim.error,
-        "Sim_Age_2D0" = Age.sim.2D0,
-        "Sim_Age_2D0.error" = Age.sim.2D0.error,
-        "Unfaded_D0" = D0.unfaded,
-        "Unfaded_D0.error" = D0.error.unfaded,
+        nN = nN,
+        nN.error = nN.error,
+        nN_SS = nN_SS,
+        nN_SS.error = nN_SS.error,
+        Meas_De = abs(De.measured),
+        Meas_De.error = De.measured.error,
+        Meas_D0 =  D0.measured,
+        Meas_D0.error = D0.measured.error,
+        Meas_Age = Age.measured,
+        Meas_Age.error = Age.measured.error,
+        Sim_De = De.sim,
+        Sim_De.error = De.error.sim,
+        Sim_D0 = D0.sim.Gy,
+        Sim_D0.error = D0.sim.Gy.error,
+        Sim_Age = Age.sim,
+        Sim_Age.error = Age.sim.error,
+        Sim_Age_2D0 = Age.sim.2D0,
+        Sim_Age_2D0.error = Age.sim.2D0.error,
+        Unfaded_D0 = D0.unfaded,
+        Unfaded_D0.error = D0.error.unfaded,
         row.names = NULL),
       data = data,
       Ln = c(Ln, Ln.error),
