@@ -1,4 +1,3 @@
-
 #' Apply the maximum age model to a given De distribution
 #'
 #' Function to fit the maximum age model to De data. This is a wrapper function
@@ -31,9 +30,8 @@
 #' for [data.frame]: two columns with De `(data[ ,1])` and De error `(data[ ,2])`.
 #'
 #' @param sigmab [numeric] (**required**):
-#' additional spread in De values.
-#' This value represents the expected overdispersion in the data should the sample be
-#' well-bleached (Cunningham & Walling 2012, p. 100).
+#' additional spread in De values, representing the expected overdispersion in
+#' the data should the sample be well-bleached (Cunningham & Wallinga 2012, p. 100).
 #' **NOTE**: For the logged model (`log = TRUE`) this value must be
 #' a fraction, e.g. 0.2 (= 20 %). If the un-logged model is used (`log = FALSE`),
 #' sigmab must be provided in the same absolute units of the De values (seconds or Gray).
@@ -60,13 +58,12 @@
 #'
 #' @return Please see [calc_MinDose].
 #'
-#' @section Function version: 0.3.1
+#' @section Function version: 0.3.2
 #'
 #' @author
 #' Christoph Burow, University of Cologne (Germany) \cr
 #' Based on a rewritten S script of Rex Galbraith, 2010
 #'
-#'
 #' @seealso [calc_CentralDose], [calc_CommonDose], [calc_FiniteMixture],
 #' [calc_FuchsLang2001], [calc_MinDose]
 #'
@@ -121,7 +118,6 @@
 #' # apply the maximum dose model
 #' calc_MaxDose(ExampleData.DeValues$CA1, sigmab = 0.2, par = 3)
 #'
-#' @md
 #' @export
 calc_MaxDose<- function(
   data,
@@ -129,7 +125,7 @@ calc_MaxDose<- function(
   log=TRUE,
   par=3,
   bootstrap=FALSE,
-  init.values,
+  init.values = NULL,
   plot=TRUE,
   ...
 ){
