@@ -27,7 +27,6 @@
 #' \eqn{s} the frequency factor in 1/s,
 #' \eqn{T} the temperature in K and \eqn{k} the Boltzmann constant in eV/K (cf. Furetta, 2010).
 #'
-#'
 #' @param E [numeric] (**required**):
 #' vector of trap depths in eV,
 #' if `profiling = TRUE` only the first two elements are considered
@@ -139,7 +138,6 @@
 #'   output_unit = "Ma"
 #')
 #'
-#' @md
 #' @export
 calc_ThermalLifetime <- function(
   E,
@@ -147,7 +145,7 @@ calc_ThermalLifetime <- function(
   T = 20,
   output_unit = "Ma",
   profiling = FALSE,
-  profiling_config = NULL,
+  profiling_config = list(),
   verbose = TRUE,
   plot = TRUE,
   ...
@@ -155,32 +153,27 @@ calc_ThermalLifetime <- function(
   .set_function_name("calc_ThermalLifetime")
   on.exit(.unset_function_name(), add = TRUE)
 
-  ## Integrity tests --------------------------------------------------------
+  ## Integrity checks -------------------------------------------------------
 
   .validate_class(E, "numeric")
   .validate_class(s, "numeric")
+  .validate_class(T, c("numeric", "integer"))
 
 # Set variables -------------------------------------------------------------------------------
 
-  ##Boltzmann constant
-  k <- 8.6173324e-05 #eV/K
-
-  ##recalculate temparature
-  T.K <- T + 273.15 #K
+  ## Boltzmann constant (ev/K)
+  kB <- .const$kB
 
+  ## recalculate temperature in Kelvin (K)
+  T.K <- T + .const$C2K
 
   ##SETTINGS FOR PROFILING
   ##profiling settings
-  profiling_settings <- list(
+  profiling_settings <- modifyList(list(
     n = 1000,
     E.distribution = "norm",
     s.distribution = "norm"
-  )
-
-  ##replace if set
-  if(!is.null(profiling_config)){
-    profiling_settings <- modifyList(profiling_settings, profiling_config)
-  }
+  ), profiling_config)
 
   ##check for odd input values
   if (profiling_settings$n < 1000){
@@ -193,7 +186,7 @@ calc_ThermalLifetime <- function(
 
  ##set function for the calculation
  f <- function(E, s, T.K) {
-    1 / s * exp(E / (k * T.K))
+    1 / s * exp(E / (kB * T.K))
  }
 
  ##PROFILING
@@ -213,7 +206,6 @@ calc_ThermalLifetime <- function(
         .throw_error("Unknown distribution setting for E profiling")
       }
 
-
     ##s
     profiling_matrix[, 2] <-
       if (profiling_settings$s.distribution == "norm") {
@@ -251,35 +243,32 @@ calc_ThermalLifetime <- function(
           X = E,
           FUN = function(j) {
             f(E = j, s = s, T.K = i)
-
           },
-          FUN.VALUE = vector(mode = "numeric", length = length(s))
-        )
-
+          FUN.VALUE = numeric(length(s)))
       },
       FUN.VALUE = matrix(numeric(), ncol = length(E), nrow = length(s))
     )
 
-    ##transform to an arry in either case to have the same output
-    if (!is(lifetimes, "array")) {
+    ##transform to an array in either case to have the same output
+    if (!inherits(lifetimes, "array")) {
       lifetimes <-
         array(lifetimes, dim = c(length(s), length(E), length(T)))
     }
 
     ##set dimnames to make reading more clear
-    dimnames(lifetimes) <- list(s, E, paste0("T = ", T, " \u00B0C"))
+    dimnames(lifetimes) <- list(s, E, paste("T =", T, "\u00B0C"))
   }
 
- ##re-calculate lifetimes accourding to the chosen output unit
- temp.lifetimes <- switch (
+  ## re-calculate lifetimes according to the chosen output unit
+  temp.lifetimes <- switch(
     output_unit,
-    "s" = lifetimes,
-    "min" = lifetimes / 60,
-    "h" = lifetimes / 60 / 60,
-    "d" = lifetimes / 60 / 60 / 24,
-    "a" = lifetimes / 60 / 60 / 24 / 365,
-    "ka" = lifetimes / 60 / 60 / 24 / 365 / 1000,
-    "Ma" = lifetimes / 60 / 60 / 24 / 365 / 1000 / 1000
+    s = lifetimes,
+    min = lifetimes / 60,
+    h = lifetimes / 60 / 60,
+    d = lifetimes / 60 / 60 / 24,
+    a = lifetimes / .const$year_s,
+    ka = lifetimes / (1e3 * .const$year_s),
+    Ma = lifetimes / (1e6 * .const$year_s)
   )
 
   ##check for invalid values
@@ -291,34 +280,30 @@ calc_ThermalLifetime <- function(
     rm(temp.lifetimes)
   }
 
-
   # Terminal output -----------------------------------------------------------------------------
 
   if(verbose){
     cat("\n[calc_ThermalLifetime()]\n\n")
 
     if(profiling){
-
-
     cat("\tprofiling = TRUE")
     cat("\n\t--------------------------\n")
     }
-    cat(paste("\tmean:\t", format(mean(lifetimes), scientific = TRUE), output_unit))
-    cat(paste("\n\tsd:\t", format(sd(lifetimes), scientific = TRUE), output_unit))
-    cat(paste("\n\tmin:\t", format(min(lifetimes), scientific = TRUE), output_unit))
+    cat("\tmean:\t", format(mean(lifetimes), scientific = TRUE), output_unit)
+    cat("\n\tsd:\t", format(sd(lifetimes), scientific = TRUE), output_unit)
 
+    cat("\n\tmin:\t", format(min(lifetimes), scientific = TRUE), output_unit)
     if(!profiling){
-      cat(paste0(" (@",T[which(lifetimes == min(lifetimes), arr.ind = TRUE)[3]], " \u00B0C)"))
+      cat(" (@", T[which(lifetimes == min(lifetimes), arr.ind = TRUE)[3]], "\u00B0C)")
     }
 
-    cat(paste("\n\tmax:\t", format(max(lifetimes), scientific = TRUE), output_unit))
-
+    cat("\n\tmax:\t", format(max(lifetimes), scientific = TRUE), output_unit)
     if(!profiling){
-      cat(paste0(" (@",T[which(lifetimes == max(lifetimes), arr.ind = TRUE)[3]], " \u00B0C)"))
+      cat(" (@", T[which(lifetimes == max(lifetimes), arr.ind = TRUE)[3]], "\u00B0C)")
     }
 
     cat("\n\t--------------------------")
-    cat(paste0("\n\t(", length(lifetimes), " lifetimes calculated in total)"))
+    cat(paste0("\n\t(", length(lifetimes), " lifetimes calculated in total)\n"))
   }
 
 
@@ -326,7 +311,7 @@ calc_ThermalLifetime <- function(
   if(plot & profiling){
 
     ##plot settings
-    plot.settings <- list(
+    plot.settings <- modifyList(list(
       main = "Thermal Lifetime Density Plot",
       xlab = paste0("Thermal lifetime [",output_unit,"]"),
       ylab = "Density",
@@ -336,10 +321,7 @@ calc_ThermalLifetime <- function(
       lwd = 1,
       lty = 1,
       col = rgb(0, 0, 0, 0.25)
-    )
-
-    ##modify on request
-    plot.settings <-   modifyList(plot.settings, list(...))
+    ), list(...))
 
     ##split data and calculate density
     ##set seq
@@ -353,14 +335,12 @@ calc_ThermalLifetime <- function(
       lapply(1:(length(id_seq) - 1),
              function(x) {
                density(lifetimes[id_seq[x]:id_seq[x+1]])
-
              })
 
     ##get x values
     lifetimes_density.x <- matrix(unlist(lapply(1:length(lifetimes_density), function(i){
       lifetimes_density[[i]]$x
 
-
     })), nrow = length(lifetimes_density[[1]]$x))
 
     ##get y values
@@ -388,10 +368,10 @@ calc_ThermalLifetime <- function(
   }
 
   # Return values -------------------------------------------------------------------------------
- return(set_RLum(
+  set_RLum(
    class = "RLum.Results",
    data = list(lifetimes = lifetimes,
                profiling_matrix = profiling_matrix),
    info = list(call = sys.call())
- ))
+  )
 }
