@@ -7,8 +7,8 @@
 #'(Gy/ka) and corrects for grain size attenuation and water content
 #'
 #'Dose rate conversion factors can be chosen from Adamiec and Aitken (1998),
-#'Guerin et al. (2011), Liritzis et al. (201) and Cresswell et al. (2018).
-#'Default is Guerin et al. (2011).
+#'Guérin et al. (2011), Liritzis et al. (201) and Cresswell et al. (2018).
+#'Default is Guérin et al. (2011).
 #'
 #'Grain size correction for beta dose rates is achieved using the correction
 #'factors published by Guérin et al. (2012).
@@ -26,8 +26,8 @@
 #'`K` \tab `numeric` \tab K nuclide content in %\cr
 #'`K_SE` \tab `numeric` \tab error on K nuclide content in %\cr
 #'`Th` \tab `numeric` \tab Th nuclide content in ppm\cr
-#'`Th_SE` \tab `numeric` error on Th nuclide content in ppm\cr
-#'`U` \tab `numeric` U nuclide content in ppm\cr
+#'`Th_SE` \tab `numeric` \tab error on Th nuclide content in ppm\cr
+#'`U` \tab `numeric` \tab U nuclide content in ppm\cr
 #'`U_SE` \tab `numeric` \tab error on U nuclide content in ppm\cr
 #'`GrainSize` \tab `numeric` \tab average grain size in µm\cr
 #'`WaterContent` \tab `numeric` \tab mean water content in %\cr
@@ -42,12 +42,14 @@
 #'
 #'The unit for the weight is gram (g).
 #'
-#'@param input [data.frame] (*optional*): a table containing all relevant information
-#' for each individual layer if nothing is provided, the function returns a template [data.frame]
-#' Please note that until one dataset per input is supported!
+#' @param input [data.frame] (*optional*): a table containing all relevant
+#' information for each individual layer. If nothing is provided, the function
+#' returns a template data frame, the values of which need to be filled in by
+#' the user. Please note that only one dataset per input is supported.
 #'
-#'@param conversion [character] (*with default*): which dose rate conversion factors to use,
-#' defaults uses Guérin et al. (2011). For accepted values see [BaseDataSet.ConversionFactors]
+#' @param conversion [character] (*with default*):
+#' dose rate conversion factors to use, by default those by Guérin et al. (2011).
+#' For accepted values see [BaseDataSet.ConversionFactors].
 #'
 #'@return The function returns an [RLum.Results-class] object for which the first
 #'element is [matrix] with the converted values. If no input is provided, the
@@ -64,10 +66,10 @@
 #'Cresswell., A.J., Carter, J., Sanderson, D.C.W., 2018. Dose rate conversion parameters:
 #'Assessment of nuclear data. Radiation Measurements 120, 195-201.
 #'
-#'Guerin, G., Mercier, N., Adamiec, G., 2011. Dose-rate conversion factors: update.
+#'Guérin, G., Mercier, N., Adamiec, G., 2011. Dose-rate conversion factors: update.
 #'Ancient TL, 29, 5-8.
 #'
-#'Guerin, G., Mercier, N., Nathan, R., Adamiec, G., Lefrais, Y., 2012. On the use
+#'Guérin, G., Mercier, N., Nathan, R., Adamiec, G., Lefrais, Y., 2012. On the use
 #'of the infinite matrix assumption and associated concepts: A critical review.
 #'Radiation Measurements, 47, 778-785.
 #'
@@ -97,7 +99,6 @@
 #'## convert
 #'convert_Concentration2DoseRate(input)
 #'
-#'@md
 #'@export
 convert_Concentration2DoseRate <- function(
   input,
@@ -106,11 +107,7 @@ convert_Concentration2DoseRate <- function(
   .set_function_name("convert_Concentration2DoseRate")
   on.exit(.unset_function_name(), add = TRUE)
 
-# Alternate mode ----------------------------------------------------------
-  if(missing(input)){
-    message("[convert_Concentration2DoseRate()] Input template returned. Please fill this data.frame and use it as input to the function!")
-
-    df <- data.frame(
+  template <- data.frame(
       Mineral = NA_character_,
       K = NA_integer_,
       K_SE = NA_integer_,
@@ -122,15 +119,18 @@ convert_Concentration2DoseRate <- function(
       WaterContent = NA_integer_,
       WaterContent_SE = NA_integer_)
 
-    return(df)
+  ## return a template if no input is given
+  if (missing(input)) {
+    .throw_message("Input template returned, please fill this data frame ",
+                   "and use it as input to the function", error = FALSE)
+    return(template)
   }
 
+  ## Load datasets ----------------------------------------------------------
 
-# Load datasets -----------------------------------------------------------
-  ## fulfil CRAN checks
+  ## silence notes raised by R CMD check
   BaseDataSet.ConversionFactors <- BaseDataSet.GrainSizeAttenuation <- NA
 
-  ## load datasets
   load(system.file("data", "BaseDataSet.ConversionFactors.rda",
                    package = "Luminescence"))
   load(system.file("data", "BaseDataSet.GrainSizeAttenuation.rda",
@@ -139,25 +139,34 @@ convert_Concentration2DoseRate <- function(
   ## we do this to be consistent with the code written by Svenja and Martin
   GSA <- BaseDataSet.GrainSizeAttenuation
 
-  ## Integrity tests --------------------------------------------------------
+  ## Integrity checks -------------------------------------------------------
+
   .validate_class(input, c("data.frame", "matrix"))
 
-  if(ncol(input) != ncol(suppressMessages(convert_Concentration2DoseRate())) || nrow(input) > 1)
-    .throw_error("Number of rows/columns in input does not match the requirements")
+  if (ncol(input) != ncol(template))
+    .throw_error("'input' should have ", ncol(template), " columns")
+  if (nrow(input) > 1)
+    .throw_error("'input' should have only one row")
+  if (anyNA(input))
+    .throw_error("'input' should not contain NA values")
+  for (idx in 2:ncol(input)) {
+    .validate_class(input[, idx], c("numeric", "integer"),
+                    name = "Each element of 'input' other than the first")
+  }
 
   ## conversion factors: we do not use BaseDataSet.ConversionFactors directly
   ## as it is in alphabetical level, but we want to have 'Guerinetal2011'
   ## in first position, as that is our default value
   valid_conversion_factors <- c("Guerinetal2011", "Cresswelletal2018",
                                 "AdamiecAitken1998", "Liritzisetal2013")
-  stopifnot(all(names(BaseDataSet.ConversionFactors) %in%
-                valid_conversion_factors))
+  stopifnot(names(BaseDataSet.ConversionFactors) %in% valid_conversion_factors)
   conversion <- .validate_args(conversion, valid_conversion_factors)
 
   if(!any(input[,1] %in% c("FS","Q")))
     .throw_error("As mineral only 'FS' or 'Q' is supported")
 
-# Convert -----------------------------------------------------------------
+  ## Convert ----------------------------------------------------------------
+
     InfDR <- matrix(data = NA, nrow = 2, ncol = 6)
     colnames(InfDR) <- c("K","SE","Th","SE","U","SE")
     rownames(InfDR) <- c("Beta","Gamma")
@@ -173,23 +182,19 @@ convert_Concentration2DoseRate <- function(
       }
 
       for (j in 1:2){
-        if (j== 1){
-          Temp = "beta"
-        } else {
-          Temp = "gamma"
-        }
+        Temp <- if (j == 1) "beta" else "gamma"
 
+        ConvFactor <- BaseDataSet.ConversionFactors[[conversion]][[Temp]][[Col]]
         Nuclide <- i * 2
         N <- 2 * i - 1
         Error <- Nuclide + 1
-        InfDR[j, N] <-
-          input[1, Nuclide] * BaseDataSet.ConversionFactors[[conversion]][[Temp]][[Col]][1]  # Calculate Dose Rate
-        InfDR[j, Nuclide] <-
-          sqrt((input[1, Error] / input[1, Nuclide]) ^ 2 + (
-            BaseDataSet.ConversionFactors[[conversion]][[Temp]][[Col]][2] /
-              BaseDataSet.ConversionFactors[[conversion]][[Temp]][[Col]][1]
-          ) ^ 2
-          ) # Calculate Error
+
+        ## calculate dose rate
+        InfDR[j, N] <- input[1, Nuclide] * ConvFactor[1]
+
+        # calculate error
+        InfDR[j, Nuclide] <- sqrt((input[1, Error] / input[1, Nuclide])^2 +
+                                  (ConvFactor[2] / ConvFactor[1])^2)
       }
     }
 
@@ -200,23 +205,22 @@ convert_Concentration2DoseRate <- function(
       ThFit <- approx(GSA$GrainSize, GSA$FS_Th,n = 981, method = "linear")
       UFit <- approx(GSA$GrainSize, GSA$FS_U, n = 981, method = "linear")
 
-      Temp <- which(KFit$x == input[1, 8])
-
-      InfDR[1, 1] <- InfDR[1, 1] * (1 - KFit$y[Temp])                     # K
-      InfDR[1, 3] <- InfDR[1, 3] * (1 - ThFit$y[Temp])                    # Th
-      InfDR[1, 5] <- InfDR[1, 5] * (1 - UFit$y[Temp])                     # U
-
     } else if (input[1,1] == "Q")  {                                # QUARTZ
       KFit <- approx(GSA$GrainSize, GSA$Q_K, n = 981, method = "linear")
       ThFit <- approx(GSA$GrainSize, GSA$Q_Th, n = 981, method = "linear")
       UFit <- approx(GSA$GrainSize, GSA$Q_U, n = 981, method = "linear")
-
-      Temp <- which(KFit$x == input[1, 8])
-      InfDR[1, 1] <- InfDR[1, 1] * (1 - KFit$y[Temp])                     # K
-      InfDR[1, 3] <- InfDR[1, 3] * (1 - ThFit$y[Temp])                    # Th
-      InfDR[1, 5] <- InfDR[1, 5] * (1 - UFit$y[Temp])                     # U
     }
 
+    Temp <- which(KFit$x == input[1, 8])
+    if (length(Temp) == 0) {
+      .throw_error("No attenuation data available for the grain size provided (",
+                   input[1, 8], ")")
+    }
+
+    InfDR[1, 1] <- InfDR[1, 1] * (1 - KFit$y[Temp])   # K
+    InfDR[1, 3] <- InfDR[1, 3] * (1 - ThFit$y[Temp])  # Th
+    InfDR[1, 5] <- InfDR[1, 5] * (1 - UFit$y[Temp])   # U
+
     ##### --- Correct beta sediment dose rate for water content --- #####
     InfDRG <- matrix(data = NA, nrow = 2, ncol = 6)
     colnames(InfDRG) <- c("K", "SE", "Th", "SE", "U", "SE")
@@ -228,9 +232,9 @@ convert_Concentration2DoseRate <- function(
     for (i in 1:6){
       for (j in 1:2){
         if (j == 1){
-          k = 1.25 #Water content correction for beta
+          k <- 1.25 #Water content correction for beta
         } else {
-          k = 1.14 #Water content correction for gamma
+          k <- 1.14 #Water content correction for gamma
         }
 
         Remain <- i %% 2
@@ -246,8 +250,7 @@ convert_Concentration2DoseRate <- function(
     InfDRG <- round(InfDRG, digits = 3)
 
 # Return ------------------------------------------------------------------
-  return(
-    set_RLum(
+  set_RLum(
       class = "RLum.Results",
       data = list(
         InfDRG = InfDRG,
@@ -255,5 +258,5 @@ convert_Concentration2DoseRate <- function(
       ),
       info = list(
         call = sys.call()
-      )))
+      ))
 }
