@@ -33,18 +33,19 @@
 #' **Parameter** \tab **Type** \tab **Description**\cr
 #' `p` \tab [numeric] \tab controls the probability for the F statistic reference values. For a significance level of 5 % a value of 0.95 (the default) should be added, for 1 %, a value of 0.99 is sufficient: 1 > p > 0 (cf. [stats::FDist])\cr
 #' `seed` \tab [numeric] \tab set the seed for the random number generator, provide a value here to get reproducible results \cr
-#' `DEoptim.trace` \tab [logical] \tab enables/disables the tracing of the differential evolution (cf. [DEoptim::DEoptim.control]) \cr
-#' `DEoptim.itermax` \tab [logical] \tab controls the number of the allowed generations (cf. [DEoptim::DEoptim.control]) \cr
-#' `weights` \tab [logical] \tab enables/disables the weighting for the start parameter estimation and fitting (see equations above).
+#' `DEoptim.trace` \tab [logical] \tab enable/disable the tracing of the differential evolution (cf. [DEoptim::DEoptim.control]) \cr
+#' `DEoptim.itermax` \tab [logical] \tab control the number of the allowed generations (cf. [DEoptim::DEoptim.control]) \cr
+#' `weights` \tab [logical] \tab enable/disable the weighting for the start parameter estimation and fitting (see equations above).
 #' The default values is `TRUE` \cr
-#' `nlsLM.trace` \tab [logical] \tab enables/disables trace mode for the nls fitting ([minpack.lm::nlsLM]), can be used to identify convergence problems, default is `FALSE` \cr
-#' `nlsLM.upper` \tab [logical] \tab enables/disables upper parameter boundary, default is `TRUE` \cr
-#' `nlsLM.lower` \tab [logical] \tab enables/disables lower parameter boundary, default is `TRUE`
+#' `nlsLM.trace` \tab [logical] \tab enable/disable trace mode for the nls fitting ([minpack.lm::nlsLM]), can be used to identify convergence problems, default is `FALSE` \cr
+#' `nlsLM.upper` \tab [logical] \tab enable/disable upper parameter boundary, default is `TRUE` \cr
+#' `nlsLM.lower` \tab [logical] \tab enable/disable lower parameter boundary, default is `TRUE`
 #' }
 #'
-#' @param object [RLum.Data.Curve-class], [RLum.Analysis-class], [data.frame] or [matrix] **(required)**:
+#' @param object [RLum.Data.Curve-class], [RLum.Analysis-class], [data.frame] or [matrix] (**required**):
 #' Input object containing the data to be analysed. All objects can be provided also as list for an automated
-#' processing. Please note: `NA` values are automatically removed and the dataset should comprise at least 5 data points.
+#' processing. Please note: `NA` values are automatically removed and the dataset should comprise at least 5 data points (possibly more if `n.components` is
+#' set to a value greater than 1)
 #'
 #' @param tp [numeric] (*with default*): option to account for the stimulation pulse width. For off-time measurements
 #' the default value is 0. `tp` has the same unit as the measurement data, e.g., µs. Please set this parameter
@@ -61,13 +62,15 @@
 #' @param method_control [list] (*optional*): Named to allow a more fine control of the fitting process. See details
 #' for allowed options.
 #'
-#' @param plot [logical] (*with default*): Enable/disable plot output
+#' @param plot [logical] (*with default*): Enable/disable the plot output.
 #'
-#' @param plot_simple [logical] (*with default*): Enable/disable reduced plot output. If `TRUE`, no
+#' @param plot_simple [logical] (*with default*): Enable/disable the reduced
+#' plot output. If `TRUE`, no
 #' residual plot is shown, however, plot output can be combined using the standard R layout options,
 #' such as `par(mfrow = c(2,2))`.
 #'
-#' @param verbose [logical] (*with default*): Enable/disable terminal feedback
+#' @param verbose [logical] (*with default*): Enable/disable output to the
+#' terminal.
 #'
 #' @param ... parameters passed to [plot.default] to control the plot output, supported are:
 #' `main`, `xlab`, `ylab`, `log`, `xlim`, `ylim`, `col`, `lty`, `legend.pos`, `legend.text`. If the input
@@ -111,7 +114,7 @@
 #'
 #' *(3) Further information*\cr
 #' - The photon count sum
-#' - Durbin-Watson residual statistic to asses whether the residuals are correlated, ideally
+#' - Durbin-Watson residual statistic to assess whether the residuals are correlated, ideally
 #' the residuals should be not correlated at all. Rough measures are: \cr
 #' D = 0: the residuals are systematically correlated \cr
 #' D = 2: the residuals are randomly distributed \cr
@@ -165,7 +168,6 @@
 #' object = ExampleData.TR_OSL)
 #' }
 #'
-#'@md
 #'@export
 fit_OSLLifeTimes <- function(
   object,
@@ -177,55 +179,49 @@ fit_OSLLifeTimes <- function(
   plot_simple = FALSE,
   verbose = TRUE,
   ...
-  ){
+) {
+  .set_function_name("fit_OSLLifeTimes")
+  on.exit(.unset_function_name(), add = TRUE)
 
+  ## Self-call --------------------------------------------------------------
+  if (inherits(object, c("list", "RLum.Analysis"))) {
+    .validate_not_empty(object)
 
-# Self-call -----------------------------------------------------------------------------------
-if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
   ##allow RLum.Analysis objects
-  if(all(vapply(object, function(x){
-    inherits(x, "RLum.Analysis")}, logical(1)))){
+  if (all(vapply(object, inherits, "RLum.Analysis", FUN.VALUE = logical(1)))) {
     object <- lapply(object, function(x){x@records})
     object <- .unlist_RLum(object)
-
   }
 
-  ##expand parameters
-  ##n.components
-  if(!is.null(n.components))
-    n.components <- as.list(rep(n.components, length(object)))
+  ## expand input arguments
+  rep.length <- length(object)
 
-  ##tp
-  tp <- as.list(rep(tp, length(object)))
+  if (!is.null(n.components))
+    n.components <- .listify(n.components, rep.length)
+  tp <- .listify(tp, rep.length)
 
   ## names of extra arguments
-  arg_names <- names(list(...))
+  arg_names <- ...names()
 
   ##pretreat some of the ... settings to avoid
   ## expand all arguments
+  arg_list <- NULL
   if(!is.null(arg_names)){
     arg_list <- lapply(arg_names , function(x){
-      unlist(rep(list(...)[[x]], length.out = length(object)))
+      unlist(rep_len(list(...)[[x]], length(object)))
     })
 
     ## make sure we organise this list (not nice but it works)
     arg_list <- lapply(1:length(object), function(x){
-      args <- lapply(1:length(arg_names), function(y){
-        arg_list[[y]][[x]]
-
-      })
+      args <- lapply(arg_list, function(y) y[[x]])
       names(args) <- arg_names
       args
-
     })
-  } else{
-    arg_list <- NULL
-
   }
 
   ##run function
   temp_results <- lapply(1:length(object), function(x){
-      temp <- try(do.call(what = fit_OSLLifeTimes,
+    temp <- try(do.call(what = fit_OSLLifeTimes,
         c(list(
          object = object[[x]],
          tp = tp[[x]],
@@ -237,17 +233,12 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
          verbose = verbose
          ),
          arg_list[[x]])
-
-     ), silent = FALSE)
+     ), outFile = stdout()) # redirect error messages so they can be silenced
 
      if(inherits(temp, "try-error")){
        return(NULL)
-
-     }else{
-       return(temp)
-
      }
-
+    return(temp)
   })
 
   ##combine results and return
@@ -258,52 +249,48 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
 
   ##return
   return(results)
-
 }
 
-# Input integrity tests ------------------------------------------------------------------
-  if(inherits(object, "RLum.Data.Curve")){
-   if(!grepl(pattern = "POSL", x = object@recordType, fixed = TRUE))
-     stop(paste0("[fit_OSLLifeTime()] recordType ",object@recordType, " not supported for input object!"),
-          call. = FALSE)
+  ## Integrity checks -------------------------------------------------------
 
-    df <- as.data.frame(object@data)
-
-  }else if(inherits(object, "data.frame")){
-    df <- object[,1:2]
-
-  } else if(inherits(object, "matrix")){
-    df <- as.data.frame(object[,1:2])
-
-  }else{
-    try(stop(paste0("[fit_OSLLifeTime()] Class '",class(object), "' not supported as input, NULL returned!"),
-             call. = FALSE))
+  is.valid <- .validate_class(object,
+                              c("RLum.Data.Curve", "data.frame", "matrix"),
+                              extra = "a 'list' of such objects",
+                              throw.error = FALSE)
+  if (!is.valid)
     return(NULL)
 
+  .validate_not_empty(object)
+  if(inherits(object, "RLum.Data.Curve")){
+   if(!grepl(pattern = "POSL", x = object@recordType, fixed = TRUE))
+     .throw_error("recordType '", object@recordType,
+                  "' not supported for input object")
+
+    object <- as.data.frame(object@data)
+
+  } else if(inherits(object, "matrix")){
+    object <- as.data.frame(object)
   }
 
-  ##remove NA values, whatever it is worth for
-  if(any(is.na(df))){
-    df <- na.exclude(df)
-    warning("[fit_OSLLifeTimes()] NA values detected and removed from dataset.",call. = TRUE)
+  if (ncol(object) < 2) {
+    .throw_error("'object' should have at least two columns")
+  }
+  df <- object[, 1:2]
 
+  ##remove NA values, whatever it is worth for
+  if (anyNA(df)) {
+    df <- na.exclude(df)
+    .throw_warning("NA values detected and removed from dataset")
   }
 
   ##rename columns for data.frame
   colnames(df) <- c("x","y")
 
-  ##make sure that we have a minimum of data points available
-  if(nrow(df) < 5){
-    try(stop("[fit_OSLLifeTimes()] Your input dataset has less than 5 data points. NULL returned!", call. = FALSE))
-    return(NULL)
-
-  }
-
   #check for 0 data in dataset ... we opt for hard stop
   if(any(df[[2]] == 0)){
-    warning("[fit_OSLLifeTimes()] Your dataset contains 0. A value of 0.1 has been added to your count values!",call. = TRUE)
+    .throw_warning("The dataset contains 0, a value of 0.1 ",
+                   "has been added to your count values")
     df[[2]] <- df[[2]] + 0.1
-
   }
 
   ##save original data for later
@@ -311,29 +298,53 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
 
   ##signal_range
   if(!is.null(signal_range)){
-    if(!inherits(signal_range, "numeric"))
-      stop("[fit_OSLLifeTimes()] Argument 'signal_range' must by of type numeric!", call. = FALSE)
+    .validate_class(signal_range, "numeric")
+
+    ## format the extremes of the signal range
+    reset_msg <- function(sr) {
+      sprintf("'signal_range' reset to c(%d, %d)", sr[1], sr[2])
+    }
 
     ##check lengths
     if(length(signal_range) == 1)
       signal_range <- c(signal_range, nrow(df))
+    else if (length(signal_range) > 2)
+      .throw_warning("'signal_range' has more than 2 elements, ",
+                     "only the first 2 will be used, ", reset_msg(signal_range))
 
-    if(length(signal_range) > 2)
-      warning("[fit_OSLLifeTimes()] 'signal_range' has more than two elements, take only the first two!", call. = FALSE)
+    if (any(signal_range < 1)) {
+      signal_range <- pmax(signal_range, 1)
+      .throw_warning("'signal_range' accepts only positive values, ",
+                     reset_msg(signal_range))
+    }
 
     if(signal_range[2] > nrow(df)){
-      warning("[fit_OSLLifeTimes()] 'signal_range' > number of channels, reset to maximum!", call. = FALSE)
       signal_range[2] <- nrow(df)
+      .throw_warning("The last element of 'signal_range' exceeds the number ",
+                     "of channels, ", reset_msg(signal_range))
     }
 
     if(signal_range[1] > signal_range[2]){
-      warning("[fit_OSLLifeTimes()] 'signal_range' first element > last element, reset to default", call. = FALSE)
       signal_range <- c(1, nrow(df))
+      .throw_warning("The first element of 'signal_range' exceeds the last ",
+                     "element, ", reset_msg(signal_range))
     }
 
     ##set range
     df <- df[signal_range[1]:signal_range[2],]
+  }
 
+  ## number of components requested
+  m <- max(.validate_positive_scalar(n.components, int = TRUE, null.ok = TRUE), 1)
+
+  ## ensure that we have a minimum of data points available: the minimum
+  ## is computed so that the degrees of freedom for the F distribution is
+  ## positive (see `qf()` in the `while` loop further down at (B))
+  min.num.signals <- 2 * m + 2 + 1
+  if (nrow(df) < min.num.signals) {
+    .throw_message("For ", m, " components the dataset must have at least ",
+                   min.num.signals, " signal points, NULL returned")
+    return(NULL)
   }
 
 
@@ -349,7 +360,6 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
     nlsLM.trace = FALSE,
     nlsLM.upper = TRUE,
     nlsLM.lower = TRUE
-
   )
 
   ##udpate list if the user did something
@@ -361,58 +371,37 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
   ##
   ##(1) >> set fitting function for minpack.lm
   x <- 0 #cheat R check routine
-  fit_forumla <- function(n.components, tp){
-    A <- paste0("A.",1:n.components)
-    tau <- paste0("tau.",1:n.components)
-    as.formula(paste0("y ~ ", paste(A," * exp(- x/(",tau," + ", tp, "))", collapse = " + ")))
-
+  fit_formula <- function(n.components, tp) {
+    comps <- 1:n.components
+    stats::reformulate(paste0("A.", comps, " * exp(-x / (tau.", comps,
+                              " + ", tp, "))", collapse = " + "),
+                       response = "y")
   }
   ##
   ##
   ##(2) create formula for differential evolution run
-    fn_constructor <- function(m){
-    ##get length of x-vector
-    x_len <- 1:(2 * m)
-
+  fn_constructor <- function(m) {
     ##generate term
-    term <- vapply(seq(1,length(x_len), by = 2), function(i){
-      paste0("(x[", i, "] * exp(-t/(x[", i + 1, "] + tp)))")
-
-    },character(1))
-
-    ##parse
-    term <- paste(term, collapse = " + ")
+    idx <- seq(1, 2 * m, by = 2)
+    term <- paste0("(x[", idx, "] * exp(-t / (x[", idx + 1, "] + tp)))",
+                   collapse = " + ")
 
     ##set weight (should be given as character)
-    if(method_control_setting$weights){
-      w <- "c^2/n"
-
-    }else{
-      w <- "1"
-
-    }
+    w <- if (method_control_setting$weights) "c^2 / n" else "1"
 
     ##combine
     term <- paste0("sum(",w," * ((n/c) - (",term,"))^2)")
 
     ##parse ... if we do it here, we boost the speed of the evaluation
     parse(text = eval(term))
-
   }
   ##
   ##
-  ##(3) initialse objects
+  ##(3) initialise objects
   chi_squared <- c(NA, NA)
   F <- c(Inf, Inf)
   start <- NULL
 
-  if(is.null(n.components)){
-    m <- 1
-
-  }else{
-    m <- n.components
-
-  }
   ##
   ##
   ##(4) set seed
@@ -426,11 +415,10 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
       cat("\n[fit_OSLLifeTime()]\n")
       cat("\n(1) Start parameter and component adapation")
       cat("\n---------------------(start adaption)------------------------------------")
-
     }
 
-    while(!is.na(suppressWarnings(qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2))) && (
-          F[2] > qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2) & F[1] >= F[2])){
+    while(!is.na(suppressWarnings(stats::qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2))) && (
+          F[2] > stats::qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2) & F[1] >= F[2])){
 
       ##set F
       F[1] <- F[2]
@@ -459,41 +447,42 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
         start_parameters <- start$optim$bestmem
 
       ##run differential evolution
-      start <- DEoptim::DEoptim(
+      start <- try(DEoptim::DEoptim(
         fn = fn,
         lower = rep(0, 2 * m),
         upper = rep(c(10 * sum(df[[2]]), 10000), m),
         control = DEoptim::DEoptim.control(
            trace = method_control_setting$DEoptim.trace,
            itermax = method_control_setting$DEoptim.itermax,
-           c = .5,
+           c = 0.5,
            strategy = 2,
            parallelType = 0 #Does it make sense to use parallel processing here: no, it does not scale well
          )
-      )
+      ), silent = TRUE)
+
+      if (inherits(start, "try-error")) {
+        .throw_error("Failed to optimize the function, check the input data")
+      }
 
       ##set chi^2 value and calculate F for the 2nd run
       chi_squared[2] <- start$optim$bestval
       if(!is.na(chi_squared[1])){
         F[2] <- (abs(diff(chi_squared))/2) /
           (chi_squared[2]/(nrow(df) - 2 * m  - 2))
-
       }
 
       ##terminal feedback
       if(verbose){
         cat("\n>> + adaption for",m, "comp.", ": ", round(F[2],2), "(calc.) <> ",
-            round(qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2), 2), "(ref.)")
+            round(stats::qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2), 2), "(ref.)")
 
-        if(F[2] > qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2) & F[1] >= F[2]){
+        if(F[2] > stats::qf(method_control_setting$p, df1 = 2, df2 = length(df[[2]]) - 2 * m - 2) & F[1] >= F[2]){
           cat(" >> [add comp.]")
 
         }else{
           cat(" >> [stop]\n")
           cat("---------------------(end adaption)--------------------------------------\n\n")
-
         }
-
       }
 
       ##break here if n.components was set others than NULL, in such case we force the number
@@ -505,13 +494,11 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
 
         start_parameters <- start$optim$bestmem
         break()
-
       }
 
       ##update objects
       chi_squared[1] <- chi_squared[2]
       m <- m + 1
-
     }
 
     ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -519,17 +506,13 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
     ##
     ##reduce m by 2, why 2?
     ##  - the last component violated the F statistic, so was obviously not the best call
-    ##  - the loop adds everytime another component
+    ##  - the loop adds every time another component
     if(is.null(n.components)){
-      ##this covers the extrem case that the process stops after the first run
+      ## this covers the extreme case that the process stops after the first run
       if(m == 2){
-        m <- 1
         start_parameters <- start$optim$bestmem
-
-      }else{
-        m <- m - 2
-
       }
+      m <- max(m - 2, 1)
     }
 
     A <- start_parameters[seq(1,length(start_parameters), by = 2)]
@@ -547,12 +530,11 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
       cat("\n>> Applied component matrix\n")
       print(start_matrix)
       cat("\n\n")
-
     }
 
     ##run fitting using the Levenberg-Marquardt algorithm
     fit <- try(minpack.lm::nlsLM(
-      formula = fit_forumla(n.components = m, tp = tp),
+      formula = fit_formula(n.components = m, tp = tp),
       data = df,
       start = c(A, tau),
       upper = if(method_control_setting$nlsLM.upper){
@@ -561,7 +543,7 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
          NULL
        },
       lower = if(method_control_setting$nlsLM.lower){
-        c(rep(0,2*length(A)))
+         rep(0, 2 * length(A))
        }else{
          NULL
        },
@@ -573,10 +555,14 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
       },
       trace = method_control_setting$nlsLM.trace,
       control = minpack.lm::nls.lm.control(maxiter = 500)
-    ), silent = FALSE)
+    ), silent = TRUE)
 
 # Post-processing -----------------------------------------------------------------------------
 
+  A <- NA
+  tau <- NA
+  summary_matrix <- NA
+  D <- NA
   if (!inherits(fit, 'try-error')) {
     ##extract coefficients
     A <- coef(fit)[1:(m)]
@@ -593,7 +579,8 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
     ##return warning if one parameter is negative, this can happen if the user let the boundaries
     ##free float
     if(any(summary_matrix[,1]<0))
-      warning("[fit_OSLLifeTimes()] At least one parameter is negative. Please carefully check your results!", call. = FALSE)
+      .throw_warning("At least one parameter is negative, ",
+                     "please check carefully your results")
 
     ##order matrix by tau, but keep the rownames
     temp_rownames <- rownames(summary_matrix)
@@ -605,15 +592,9 @@ if(inherits(object, "list") || inherits(object, "RLum.Analysis")){
     R <- residuals(fit)
     D <- round(sum((R - c(0,R[-length(R)]))^2) / sum(R^2),2)
     rm(R)
-
-
-  }else{
+  } else {
+    .throw_warning("Fitting failed: ", attr(fit, "condition")$message)
     m <- 1
-    A <- NA
-    tau <- NA
-    summary_matrix <- NA
-    D <- NA
-
   }
 
 # Terminal output -----------------------------------------------------------------------------
@@ -626,8 +607,7 @@ if(verbose){
     cat("-------------------------------------------------------------------------\n")
 
   }else{
-    try(stop("[fit_OSLLifeTimes()] The fitting was not sucessful, consider to try again!", call. = FALSE))
-
+    .throw_message("The fitting was not successful, consider trying again")
   }
 
   cat("\n(3) Further information\n")
@@ -635,16 +615,12 @@ if(verbose){
   cat("Photon count sum: ", sum(df[[2]]),"\n")
   cat("Durbin-Watson residual statistic: ", D,"")
 
+  string <- NA
   if(!is.na(D)){
     string <- c("\u005b",rep(" ",(D * 10)/4),"\u003c\u003e",rep(" ",10 - (D * 10)/4),"\u005d\n")
-
-  }else{
-    string <- NA
-
   }
   cat(paste(string, collapse = ""))
   cat("\n")
-
 }
 
 # Plotting ------------------------------------------------------------------------------------
@@ -662,43 +638,37 @@ if(plot) {
     lty = rep(1, (m + 1)),
     legend.pos = "topright",
     legend.text = c("sum", paste0("comp. ", 1:m))
-
   )
 
     ##modify settings on request
     plot_settings <- modifyList(x = plot_settings, val = list(...))
 
     ##catch log scale
-    if(grepl(pattern = "x", plot_settings$log, fixed = TRUE)){
-      if(plot_settings$xlim[1] == 0){
-        plot_settings$xlim[1] <- if(min(df_raw[[1]]) == 0) 1e-04 else min(df_raw[[1]])
-        warning(paste0("[fit_OSLLifeTime()] log-scale requires x-values > 0, set min xlim to ",
-                       round(plot_settings$xlim[1],4), "!"), immediate. = TRUE, call. = FALSE)
-      }
+    if (is.list(plot_settings$log))
+      plot_settings$log <- unlist(plot_settings$log)
+    if (grepl(pattern = "x", plot_settings$log, fixed = TRUE) &&
+        plot_settings$xlim[1] == 0) {
+        plot_settings$xlim[1] <- max(min(df_raw[[1]]), 1e-4)
+        .throw_warning("log-scale requires x-values > 0, set min xlim to ",
+                       round(plot_settings$xlim[1], 4))
     }
 
-    if(grepl(pattern = "y", plot_settings$log, fixed = TRUE)){
-      if(plot_settings$ylim[1] == 0){
-        plot_settings$ylim[1] <- if(min(df_raw[[2]]) == 0) 1e-04 else min(df_raw[[2]])
-        warning(paste0("[fit_OSLLifeTime()] log-scale requires y-values > 0, set min ylim to ",
-                       round(plot_settings$ylim[1],4), "!"), immediate. = TRUE, call. = FALSE)
-      }
+    if (grepl(pattern = "y", plot_settings$log, fixed = TRUE) &&
+        plot_settings$ylim[1] == 0) {
+        plot_settings$ylim[1] <- max(min(df_raw[[2]]), 1e-04)
+        .throw_warning("log-scale requires y-values > 0, set min ylim to ",
+                       round(plot_settings$ylim[1], 4))
     }
 
+  par.default <- .par_defaults()
+  on.exit(par(par.default), add = TRUE)
 
   ##plot if the fitting was a success
   if (!inherits(fit, 'try-error')) {
 
     if(!plot_simple){
-      ##make sure that the screen closes if something is wrong
-      on.exit(close.screen(all.screens = TRUE))
-
-      split.screen(rbind(
-        c(0.1,1,0.32, 0.98),
-        c(0.1,1,0.1, 0.32)))
-
-      screen(1)
-      par(mar = c(0, 4, 3, 4))
+      graphics::layout(matrix(c(1, 2), 2, 1), height = c(0.7, 0.3))
+      par(mar = c(0, 4.5, 3, 2))
     }
 
     plot(NA,NA,
@@ -721,7 +691,7 @@ if(plot) {
     ##+ add some curve
     lines(
       df$x,
-      fitted(fit),
+      stats::fitted(fit),
       col = plot_settings$col[1],
       lwd = 1.3,
       lty = plot_settings$lty[1]
@@ -741,7 +711,6 @@ if(plot) {
         col = plot_settings$col[i + 1],
         lty = plot_settings$lty[i + 1]
       )
-
     }
 
     ##+ add legend
@@ -753,10 +722,8 @@ if(plot) {
       bty = "n"
     )
 
-
     if(!plot_simple){
-      screen(2)
-      par(mar = c(4, 4, 0, 4))
+      par(mar = c(5, 4.5, 0, 2))
       plot(
         x = df[[1]],
         y = residuals(fit),
@@ -764,12 +731,13 @@ if(plot) {
         type = "b",
         pch = 20,
         xlim = plot_settings$xlim,
-        log = if(plot_settings$log == "x"){"x"}else{""},
+        log = gsub("y", "", plot_settings$log),
         ylab = "Resid."
       )
     }
 
   }else{
+      ## fitting failed
       plot(
         df,
         xlab = plot_settings$xlab,
@@ -780,16 +748,13 @@ if(plot) {
         ylim = plot_settings$ylim,
         log = plot_settings$log
       )
-
   }
-
 }#if plot
 
 # Return --------------------------------------------------------------------------------------
 
   ##create return object
-  return(
-    set_RLum(
+  set_RLum(
       class = "RLum.Results",
       data = list(
         data = summary_matrix,
@@ -800,7 +765,5 @@ if(plot) {
       info = list(
         call = sys.call()
       )
-    )
   )
-
 }
