@@ -61,12 +61,13 @@
 #' Example: `sigmaphi = 5e-10`
 #'
 #' @param mu [numeric] (*optional*):
-#' A numeric value for mu, i.e. the light attenuation coefficient.
-#' Example: `mu = 0.9`
+#' A numeric value for `mu`, i.e. the light attenuation coefficient. If `data`
+#' is a list of *n* samples, then `mu` must be a vector of length 1 or *n*.
+#' Example: `mu = 0.9`, or `mu = c(0.9, 0.8, 0.9)`.
 #'
 #' @param age [numeric] (*optional*):
-#' The age (a) of the sample, if known. If `data` is a [list] of *x* samples,
-#' then `age` must be a numeric vector of length *x*.
+#' The age (a) of the sample, if known. If `data` is a [list] of *n* samples,
+#' then `age` must be a numeric vector of length *n*.
 #' Example: `age = 10000`, or `age = c(1e4, 1e5, 1e6)`.
 #'
 #' @param Ddot [numeric] (*optional*):
@@ -79,20 +80,20 @@
 #' to be considered a value for `Ddot` must also be provided; otherwise it will be
 #' ignored.
 #'
-#' @param weights [logical] (*optional*):
+#' @param weights [logical] (*with default*):
 #' If `TRUE` the fit will be weighted by the inverse square of the error.
 #' Requires `data` to be a [data.frame] with three columns.
 #'
-#' @param plot [logical] (*optional*):
+#' @param plot [logical] (*with default*):
 #' enable/disable the plot output.
 #'
-#' @param legend [logical] (*optional*):
+#' @param legend [logical] (*with default*):
 #' Show or hide the equation inside the plot.
 #'
-#' @param error_bars [logical] (*optional*):
+#' @param error_bars [logical] (*with default*):
 #' Show or hide error bars (only applies if errors were provided).
 #'
-#' @param coord_flip [logical] (*optional*):
+#' @param coord_flip [logical] (*with default*):
 #' Flip the coordinate system.
 #'
 #' @param ... Further parameters passed to [plot].
@@ -210,7 +211,6 @@
 #'  D0 = 40)
 #'get_RLum(results)
 #'
-#' @md
 #' @export
 fit_SurfaceExposure <- function(
     data,
@@ -258,6 +258,11 @@ fit_SurfaceExposure <- function(
       .throw_error("If 'data' is a list of data sets for global fitting, ",
                    "'age' must be of the same length.")
 
+    ## if mu is specified, it must have length 1 or match the number of data sets
+    if (!is.null(mu) && length(mu) != 1 && length(data) != length(mu))
+      .throw_error("If 'data' is a list of data sets for global fitting, ",
+                   "'mu' must either be of the same length or of length 1.")
+
     # TODO: Support weighted fitting for global fit
     if (weights) {
       if (settings$verbose)
@@ -301,8 +306,10 @@ fit_SurfaceExposure <- function(
   ## remove rows with NA
   if (anyNA(data)) {
     data <- data[stats::complete.cases(data), ]
+    if (nrow(data) == 0)
+      .throw_error("After NA removal, nothing is left from the data set")
     if (settings$verbose)
-      message("[fit_SurfaceExposure()] NA values in 'data' were removed")
+      .throw_message("NA values in 'data' were removed", error = FALSE)
   }
 
   ## extract errors into separate variable
@@ -327,16 +334,17 @@ fit_SurfaceExposure <- function(
 
   ## Functions
   # w/o dose rate
-  fun <- y ~ exp(-sigmaphi * age * 365.25*24*3600 * exp(-mu * x))
-  fun_global <- y ~ exp(-sigmaphi * age[group] * 365.25*24*3600 * exp(-mu * x))
+  year_s <- .const$year_s
+  fun <- y ~ exp(-sigmaphi * age * year_s * exp(-mu * x))
+  fun_global <- y ~ exp(-sigmaphi * age[group] * year_s * exp(-mu * x))
 
   # w/ dose rate (Sohbati et al. 2012, eq 12)
   if (!is.null(Ddot))
-    Ddot <- Ddot / 1000 / 365.25 / 24 / 60 / 60
+    Ddot <- Ddot / (1000 * year_s)
 
-  fun_w_dr <- y ~ (sigmaphi * exp(-mu * x) * exp(-(age * 365.25*24*3600) * (sigmaphi * exp(-mu * x) + Ddot/D0)) + Ddot/D0) /
+  fun_w_dr <- y ~ (sigmaphi * exp(-mu * x) * exp(-(age * year_s) * (sigmaphi * exp(-mu * x) + Ddot/D0)) + Ddot/D0) /
                          (sigmaphi * exp(-mu * x) + Ddot/D0)
-  fun_global_w_dr <- y ~ (sigmaphi * exp(-mu * x) * exp(-(age[group] * 365.25*24*3600) * (sigmaphi * exp(-mu * x) + Ddot/D0)) + Ddot/D0) /
+  fun_global_w_dr <- y ~ (sigmaphi * exp(-mu * x) * exp(-(age[group] * year_s) * (sigmaphi * exp(-mu * x) + Ddot/D0)) + Ddot/D0) /
                                 (sigmaphi * exp(-mu * x) + Ddot/D0)
 
   ## start parameter
@@ -360,10 +368,9 @@ fit_SurfaceExposure <- function(
       use_fun <- fun_global_w_dr
     else
       use_fun <- fun_w_dr
-  } else {
-    if (global_fit)
+  } else if (global_fit) {
       use_fun <- fun_global
-    else
+  } else {
       use_fun <- fun
   }
 
@@ -390,7 +397,7 @@ fit_SurfaceExposure <- function(
 
     # Fill with NA values
     coef <- data.frame(
-      "Estimate" = rep(NA, 3),
+      Estimate = rep(NA, 3),
       "Std. Error" = rep(NA, 3),
       row.names = c("age", "sigmaphi", "mu"), check.names = FALSE
     )
@@ -398,11 +405,11 @@ fit_SurfaceExposure <- function(
 
   ## RESULTS ----
   summary <- data.frame(
-    age = if (is.null(age)) coef["age", "Estimate"] else age,
+    age = age %||% coef["age", "Estimate"],
     age_error = coef["age", "Std. Error"],
-    sigmaphi = if (is.null(sigmaphi)) coef["sigmaphi", "Estimate"] else sigmaphi,
+    sigmaphi = sigmaphi %||% coef["sigmaphi", "Estimate"],
     sigmaphi_error = coef["sigmaphi", "Std. Error"],
-    mu = if (is.null(mu)) coef["mu", "Estimate"] else mu,
+    mu = mu %||% coef["mu", "Estimate"],
     mu_error = coef["mu", "Std. Error"]
   )
 
@@ -419,6 +426,8 @@ fit_SurfaceExposure <- function(
 
   ## PLOT ----
   if (plot) {
+    par.default <- .par_defaults()
+    on.exit(par(par.default), add = TRUE)
 
     # remove $group column for easier data handling
     if (global_fit)
@@ -435,13 +444,13 @@ fit_SurfaceExposure <- function(
       pch = 21,
       col = "black",
       bg = "red",
-      xlab = if (!coord_flip) "Depth (mm)" else "OSL intensity (Ln/Tn)",
-      ylab = if (!coord_flip) "OSL intensity (Ln/Tn)" else "Depth (mm)",
+      xlab = if (coord_flip) "OSL intensity (Ln/Tn)" else "Depth (mm)",
+      ylab = if (coord_flip) "Depth (mm)" else "OSL intensity (Ln/Tn)",
       cex = 1.0,
       lty = 1,
       lwd = 1,
       log = "",
-      ylim = if (!coord_flip) range(pretty(data[ ,2])) else rev(range(pretty(data[ ,2]))),
+      ylim = if (coord_flip) rev(range(pretty(data[, 2]))) else range(pretty(data[, 2])),
       xlim = range(pretty(data[ ,1]))
     )
 
@@ -474,7 +483,7 @@ fit_SurfaceExposure <- function(
     }
 
     ## add fitted curve
-    if (!inherits(fit, "error") && !inherits(fit, "simpleError")) {
+    if (!inherits(fit, c("error", "simpleError"))) {
 
       if (coord_flip) {
         oldx <- data[ ,2]
@@ -494,17 +503,17 @@ fit_SurfaceExposure <- function(
       if (!global_fit) {
         points(newx, newy,
                type = "l",
-               col = ifelse("line_col" %in% names(list(...)), list(...)$line_col, "blue"),
-               lty = ifelse("line_lty" %in% names(list(...)), list(...)$line_lty, 1),
-               lwd = ifelse("line_lwd" %in% names(list(...)), list(...)$line_lwd, 1))
+               col = list(...)$line_col %||% "blue",
+               lty = list(...)$line_lty %||% 1,
+               lwd = list(...)$line_lwd %||% 1)
       } else {
         for (i in 1:length(data_list)) {
           seg <- seq(i * 101 - 100, 10000, nrow(data))
           points(newx[seg], newy[seg],
                  type = "l",
-                 col = ifelse("line_col" %in% names(list(...)), list(...)$line_col, i),
-                 lty = ifelse("line_lty" %in% names(list(...)), list(...)$line_lty, 1),
-                 lwd = ifelse("line_lwd" %in% names(list(...)), list(...)$line_lwd, 1))
+                 col = list(...)$line_col %||% i,
+                 lty = list(...)$line_lty %||% 1,
+                 lwd = list(...)$line_lwd %||% 1)
         }
       }
 
@@ -519,7 +528,7 @@ fit_SurfaceExposure <- function(
 
     }
 
-    # add formula
+    ## show the formula in the legend
     if (legend && !inherits(fit, "simpleError")) {
       formula_text <- paste0("y = ", as.character(fit$m$formula())[3])
 
@@ -533,8 +542,13 @@ fit_SurfaceExposure <- function(
       }
       if (!is.null(sigmaphi))
         formula_text <- gsub("sigmaphi", sigmaphi, formula_text)
-      if (!is.null(mu))
-        formula_text <- gsub("mu", mu, formula_text)
+      if (!is.null(mu)) {
+        if (!global_fit) {
+          formula_text <- gsub("mu", mu, formula_text)
+        } else {
+          formula_text <- gsub("mu", paste0("[", paste(mu, collapse = "|"), "]"), formula_text)
+        }
+      }
 
       legend(ifelse(coord_flip, "bottomleft", "bottomright"), legend = formula_text, cex = 0.8, bty = "n")
     }
@@ -544,50 +558,40 @@ fit_SurfaceExposure <- function(
   if (settings$verbose) {
     cat("\n [fit_SurfaceExposure()] \n\n")
 
-    if (!global_fit) {
-      ## STANDARD OUTPUT
-      cat(" Estimated paramater(s):\n",
-          "-----------------------\n")
-      if (is.null(age))
-        cat(paste0(" age (a):\t", signif(results$summary$age, 3), " \u00B1 ",
-                   signif(results$summary$age_error, 3), "\n"))
-      if (is.null(sigmaphi))
-        cat(paste0(" sigmaphi:\t", signif(results$summary$sigmaphi, 3), " \u00B1 ",
-                   signif(results$summary$sigmaphi_error, 3), "\n"))
-      if (is.null(mu))
-        cat(paste0(" mu:\t\t", signif(results$summary$mu, 3), " \u00B1 ",
-                   signif(results$summary$mu_error, 3), "\n"))
-      cat("\n")
+    if (global_fit) {
+      cat(" Shared estimated parameter(s):\n")
     } else {
-      ## GLOBAL FIT OUTPUT
-      cat(" Shared estimated paramater(s):\n",
-          "-----------------------\n")
-      if (is.null(sigmaphi))
-        cat(paste0(" sigmaphi:\t", signif(unique(results$summary$sigmaphi), 3), " \u00B1 ",
-                   signif(unique(results$summary$sigmaphi_error), 3), "\n"))
-      if (is.null(mu))
-        cat(paste0(" mu:\t\t", signif(unique(results$summary$mu), 3), " \u00B1 ",
-                   signif(unique(results$summary$mu_error), 3), "\n"))
-      cat("\n")
+      cat(" Estimated parameter(s):\n")
     }
+    cat(" ---------------------\n")
+    if (is.null(age))
+      cat(" age (a):\t", signif(results$summary$age, 3), "\u00B1",
+          signif(results$summary$age_error, 3), "\n")
+    if (is.null(sigmaphi))
+      cat(" sigmaphi:\t", signif(unique(results$summary$sigmaphi), 3), "\u00B1",
+          signif(unique(results$summary$sigmaphi_error), 3), "\n")
+    if (is.null(mu))
+      cat(" mu:\t\t", signif(unique(results$summary$mu), 3), "\u00B1",
+          signif(unique(results$summary$mu_error), 3), "\n")
+    cat("\n")
 
     ## STANDARD OUTPUT
-    cat(" Fixed parameters(s):\n",
-        "--------------------\n")
+    cat(" Fixed parameter(s):\n",
+        "-------------------\n")
     if (!is.null(age))
-      cat(paste0(" age (a):\t", .collapse(age, quote = FALSE), "\n"))
+      cat(" age (a):\t", .collapse(age, quote = FALSE), "\n")
     if (!is.null(sigmaphi))
-      cat(paste0(" sigmaphi:\t", sigmaphi, "\n"))
+      cat(" sigmaphi:\t", sigmaphi, "\n")
     if (!is.null(mu))
-      cat(paste0(" mu:\t\t", mu, "\n"))
+      cat(" mu:\t\t", .collapse(mu, quote = FALSE), "\n")
     cat("\n")
 
     if (!is.null(age)) {
-      message(paste0("To apply the estimated parameters to a sample of unknown age run:\n\n",
-                     "fit_SurfaceExposure(data = ", capture.output(results$args[[1]]),
-                     ", sigmaphi = ", signif(unique(results$summary$sigmaphi), 3),
-                     ", mu = ", signif(unique(results$summary$mu), 3),
-                     ")\n\n"))
+      message("To apply the estimated parameters to a sample of unknown age run:\n\n",
+              "fit_SurfaceExposure(data = ", capture.output(results$args[[1]]),
+              ", sigmaphi = ", signif(unique(results$summary$sigmaphi), 3),
+              ", mu = c(", .collapse(signif(results$summary$mu, 3), quote = FALSE),
+              "))\n")
     }
   }
 
