@@ -8,24 +8,25 @@
 #'
 #' The equation used for the fitting is
 #'
-#' \deqn{y = (A / (1 + C * (exp(-W / (k * x))))) + c}
+#' \deqn{y = A / (1 + C * exp(-W / (kB * x))) + c}
 #'
-#' *W* is the energy depth in eV and *C* is dimensionless constant. *A* and *c* are used to
-#' adjust the curve for the given signal. *k* is the Boltzmann in eV/K and *x* is the absolute
-#' temperature in K.
+#' *W* is the energy depth in eV, *C* is a dimensionless constant, *A* and *c*
+#' are used to adjust the curve for the given signal, *kB* is the Boltzmann
+#' constant in eV/K and *x* is the absolute temperature in K.
 #'
 #' **Error estimation**\cr
 #'
-#' The error estimation is done be varying the input parameters using the given uncertainties in
-#' a Monte Carlo simulation. Errors are assumed to follow a normal distribution.
+#' The error estimation is done by varying the input parameters using the
+#' given uncertainties in a Monte Carlo simulation. Errors are assumed to
+#' follow a normal distribution.
 #'
 #' **`start_param`** \cr
 #'
-#' The function allows the injection of own start parameters via the argument `start_param`. The
-#' parameters needs to be provided as names list. The names are the parameters to be optimised.
+#' The function allows the injection of starting values for the parameters to
+#' be optimised via the `start_param` argument. The parameters must be provided
+#' as a named list.
 #' Examples: `start_param = list(A = 1, C = 1e+5, W = 0.5, c = 0)`
 #'
-#'
 #' **`method_control`** \cr
 #'
 #' The following arguments can be provided via `method_control`. Please note that arguments provided
@@ -53,8 +54,8 @@
 #' the fitting, see details for
 #' further information
 #'
-#' @param n.MC [numeric] (*with default*): number of Monte Carlo runs for the error estimation. If `n.MC` is
-#' `NULL` or `<=1`, the error estimation is skipped
+#' @param n.MC [integer] (*with default*): number of Monte Carlo runs for the
+#' error estimation. If `n.MC` is `NULL` or 1, the error estimation is skipped.
 #'
 #' @param verbose [logical] (*with default*): enable/disable output to the
 #' terminal.
@@ -78,7 +79,8 @@
 #'
 #' `[.. $data : data.frame]`\cr
 #'
-#'  A table with all fitting parameters and the number of Monte Carlo runs used for the error estimation.
+#' A table with all fitting parameters and the number of Monte Carlo runs used
+#' for the error estimation (this may be smaller that `n.MC`).
 #'
 #' `[.. $fit : nls object]` \cr
 #'
@@ -100,10 +102,11 @@
 #' The fit is shown as dashed-line (can be modified). Please note that for the fitting the absolute
 #' temperature values are used but are re-calculated to deg. C for the plot.
 #'
+#' @section Function version: 0.2
 #'
-#' @section Function version: 0.1.0
-#'
-#' @author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
+#' @author
+#' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany) \cr
+#' Marco Colombo, Institute of Geography, Heidelberg University (Germany)
 #'
 #' @references
 #'
@@ -124,7 +127,6 @@
 #'  data = data,
 #'  n.MC = NULL)
 #'
-#' @md
 #' @export
 fit_ThermalQuenching <- function(
   data,
@@ -140,8 +142,6 @@ fit_ThermalQuenching <- function(
 
   # Self-call -----------------------------------------------------------------------------------
   if(inherits(data, "list")){
-
-    ##get arguments
     args <- as.list(match.call())
     args[[1]] <- NULL
     args$data <- NULL
@@ -155,9 +155,7 @@ fit_ThermalQuenching <- function(
     return(merge_RLum(results_list))
   }
 
-
-  ## Integrity tests --------------------------------------------------------
-
+  ## Integrity checks -------------------------------------------------------
   .validate_class(data, "data.frame",
                   extra = "a 'list' of such objects")
 
@@ -172,62 +170,81 @@ fit_ThermalQuenching <- function(
   if (anyNA(data)) {
     .throw_warning("NA values in 'data' automatically removed")
     data <- na.exclude(data)
+    if (nrow(data) == 0)
+      .throw_error("After NA removal, nothing is left from the data set")
   }
 
-  # Prepare data --------------------------------------------------------------------------------
-  ##set formula for quenching accordingt to Wintle 1973
-  ##we here add a constant, otherwise the fit will not really work
-  k <- 8.6173303e-05
-  f <- y ~ (A / (1 + C * (exp(-W / (k * x))))) + c
+  .validate_class(start_param, "list")
+  .validate_class(method_control, "list")
+  n.MC <- max(.validate_positive_scalar(n.MC, int = TRUE, null.ok = TRUE), 1)
+
+  ## Prepare data -----------------------------------------------------------
+  ##
+  ## Set formula for quenching according to Wintle 1973.
+  ## Here we add a constant, otherwise the fit will not really work.
+  ##
+  ## The actual implementation adopts a slight reformulation, which is
+  ## mathematically equivalent to the one presented in the documentation
+  ## above, but is numerically more stable:
+  ##
+  ## y = A / (1 + C * exp(-W / (kB * x))) + c
+  ##   = A / (1 + exp(log(C)) * exp(-W / (kB * x))) + c
+  ##   = A / (1 + exp(log(C) - W / (kB * x))) + c
+  ##   = A / (1 + exp(C' - W / (kB * x))) + c
+  ##
+  ## For backward compatibility, we keep using the name C for the logged
+  ## parameter, so that the user will keep seeing values in the original scale,
+  ## although internally we operate in log-space for C. When we show the
+  ## final results, we'll have to exponentiate the fitted parameter.
+  ##
+  kB <- .const$kB
+  f <- y ~ A / (1 + exp(C - W / (kB * x))) + c
 
   ##set translate values in data.frame to absolute temperature
-  data_raw <- data
-  data[[1]] <- data[[1]] + 273.15
+  data[[1]] <- data[[1]] + .const$C2K
 
   ##start parameter
   start_param <- modifyList(x = list(
     A = max(data[[2]]),
-    C = max(data[[1]] * 10e+5),
+    C = log(max(data[[1]]) * 1e5),
     W = 0.5,
     c = 0),
     val = start_param)
 
-
   ##method control
   method_control <- modifyList(
     x = list(
-     lower = c(A = 0, C = 0, W = 0, c = 0),
-     upper = c(A = 10 * start_param$A, C = Inf, W = 10, c = start_param$A),
+     lower = c(A = 0, C = 0, W = 0, c = -Inf),
+     upper = c(A = 10 * start_param$A,
+               C = log(max(data[[1]]) * 1e12),
+               W = 10, c = 10 * start_param$A),
      trace = FALSE,
      weights = data[[3]]
     ),
     val = method_control)
 
+  weights <- method_control$weights %||% rep(1, nrow(data))
+
   # Fitting -------------------------------------------------------------------------------------
   ##guine fitting
   fit <- try(minpack.lm::nlsLM(
     formula = f,
     data = data.frame(x = data[[1]], y = data[[2]]),
-    weights = if(is.null(method_control$weights)){
-      rep(1, length(data[[2]]))
-      } else {
-        method_control$weights},
+    weights = weights,
     control = list(
       maxiter = 500,
       maxfev = 1000,
-      trace = method_control$weights
-      ),
+      trace = method_control$trace),
     start = start_param,
     lower = method_control$lower,
     upper = method_control$upper
   ), silent = TRUE)
 
   ##only continue if the first fitting worked out
-  if(!inherits(fit, "try-error")){
-
-    ##reset n.MC
-    if(is.null(n.MC) || n.MC < 1)
-      n.MC <- 1
+  if (inherits(fit, "try-error")) {
+    .throw_message("Fitting failed, NULL returned")
+    return(NULL)
+  }
 
     ##Prepare MC runs for the fitting
     x_MC <- data[[1]]
@@ -237,15 +254,12 @@ fit_ThermalQuenching <- function(
       ncol = n.MC)
     y_MC[y_MC < 0] <- 0
 
-    ##run fitting
-    fit_MC <- lapply(1:n.MC, function(x){
+  ## run fitting
+  fit_MC <- lapply(1:n.MC, function(x) {
       temp <- try(minpack.lm::nlsLM(
         formula = f,
         data = data.frame(x = x_MC, y = y_MC[,x]),
-        weights = if(is.null(method_control$weights)){
-          rep(1, length(data[[2]]))
-        } else {
-          method_control$weights},
+        weights = weights,
         control = list(
           maxiter = 500,
           maxfev = 1000
@@ -256,23 +270,16 @@ fit_ThermalQuenching <- function(
       ), silent = TRUE)
 
       ##return value
-      if(inherits(temp, 'try-error')) {
-        return(NULL)
-      } else{
-        temp
-      }
-    })
-
-  }else{
-    .throw_message("Fitting failed, NULL returned")
-    return(NULL)
-  }
+      if (inherits(temp, "try-error"))
+        return(NULL) # nocov
+      temp
+  })
 
   ## remove NULL (the fit was not successful)
   fit_MC <- .rm_NULL_elements(fit_MC)
   n.MC <- length(fit_MC)
 
-# Extract values ------------------------------------------------------------------------------
+  ## Extract values ---------------------------------------------------------
 
    ##(1) - extract parameters from main fit
    fit_coef <- coef(fit)
@@ -285,23 +292,26 @@ fit_ThermalQuenching <- function(
    fit_coef_MC_full <- vapply(X = fit_MC, FUN = coef, FUN.VALUE = numeric(4))
    fit_coef_MC <- round(matrixStats::rowSds(fit_coef_MC_full),3)
 
+  ## (3) - unlog the C parameter before computing the standard deviation
+  fit_coef_MC[2] <- signif(sd(exp(fit_coef_MC_full[2, ])), 4)
+
    A_MC_X <- fit_coef_MC[1]
    C_MC_X <- fit_coef_MC[2]
    W_MC_X <- fit_coef_MC[3]
    c_MC_X <- fit_coef_MC[4]
 
-# Terminal output -----------------------------------------------------------------------------
-if(verbose){
+  ## Terminal output --------------------------------------------------------
+  if (verbose) {
 
     cat("\n[fit_ThermalQuenching()]\n\n")
-    cat(" A = ", A, " \u00b1 ",A_MC_X,"\n")
-    cat(" C = ", C, " \u00b1 ",C_MC_X,"\n")
-    cat(" W = ", W, " \u00b1 ",W_MC_X, " eV\n")
-    cat(" c = ", c, " \u00b1 ",c_MC_X, "\n")
+    cat(" A =", A, "\u00b1", A_MC_X, "\n")
+    cat(" C =", exp(C), "\u00b1", C_MC_X, "\n")
+    cat(" W =", W, "\u00b1", W_MC_X, "eV\n")
+    cat(" c =", c, "\u00b1", c_MC_X, "\n")
     cat(" --------------------------------\n")
-}
+  }
 
-# Potting -------------------------------------------------------------------------------------
+  ## Plotting ---------------------------------------------------------------
   if(plot) {
     ##plot settings
     plot_settings <- list(
@@ -337,20 +347,20 @@ if(verbose){
 
     ##add axis with correct temperature
     if(!is.null(plot_settings$xaxt) && plot_settings$xaxt == "n"){
-      at <- pretty(round(axTicks(side = 1) - 273.15))
-      axis(side = 1, at = at + 273.15, labels = at)
+      at <- pretty(round(axTicks(side = 1) - .const$C2K))
+      axis(side = 1, at = at + .const$C2K, labels = at)
     }
 
-    ##reset n.MC
-    if(!is.null(n.MC) && n.MC > 1){
-      ##add MC curves
+    ## add MC curves
+    if (n.MC > 1) {
       for(i in 1:n.MC){
         A <- fit_coef_MC_full[1,i]
         C <- fit_coef_MC_full[2,i]
         W <- fit_coef_MC_full[3,i]
         c <- fit_coef_MC_full[4,i]
         x <- 0
-        curve((A / (1 + C * (exp(-W / (k * x))))) + c, col = rgb(0,0,0,.1), add = TRUE)
+        curve(A / (1 + exp(C - W / (kB * x))) + c,
+              col = rgb(0, 0, 0, 0.1), add = TRUE)
       }
     }
 
@@ -366,49 +376,41 @@ if(verbose){
              )
 
     ##add central fit
-      A <- fit_coef[["A"]]
-      C <- fit_coef[["C"]]
-      W <- fit_coef[["W"]]
-      c <- fit_coef[["c"]]
+    A <- fit_coef[["A"]]
+    C <- fit_coef[["C"]]
+    W <- fit_coef[["W"]]
+    c <- fit_coef[["c"]]
 
     x <- 0
-    curve((A / (1 + C * (exp(
-      -W / (k * x)
-    )))) + c,
-    lty = plot_settings$lty,
-    lwd = plot_settings$lwd,
-    col = plot_settings$col_fit,
-    add = TRUE
-    )
+    curve(A / (1 + exp(C - W / (kB * x))) + c,
+          lty = plot_settings$lty,
+          lwd = plot_settings$lwd,
+          col = plot_settings$col_fit,
+          add = TRUE)
 
     ##add mtext
     mtext(side = 3, text = plot_settings$mtext)
   }
 
-
-  # Return --------------------------------------------------------------------------------------
-  output_df <- data.frame(
-    A = A,
-    A_X = A_MC_X,
-    C = C,
-    C_X = C_MC_X,
-    W = W,
-    W_X = W_MC_X,
-    c = c,
-    c_X = c_MC_X,
-    n.MC = n.MC
-  )
-
-  output <- set_RLum(
+  ## Return -----------------------------------------------------------------
+  set_RLum(
     class = "RLum.Results",
     data = list(
-      data = output_df,
+      data = data.frame(
+          A = A,
+          A_X = A_MC_X,
+          C = exp(C),
+          C_X = C_MC_X,
+          W = W,
+          W_X = W_MC_X,
+          c = c,
+          c_X = c_MC_X,
+          n.MC = n.MC
+      ),
       fit = fit
     ),
     info = list(
       call = sys.call()
     )
   )
-
-  return(output)
 }
