@@ -1,11 +1,13 @@
 #' @title Visualise dose recovery test results
 #'
-#' @description The function provides a standardised plot output for dose recovery test
+#' @description
+#' The function provides a standardised plot output for dose recovery test
 #' measurements.
 #'
-#' @details Procedure to test the accuracy of a measurement protocol to reliably
+#' @details
+#' The procedure tests the accuracy of a measurement protocol to reliably
 #' determine the dose of a specific sample. Here, the natural signal is erased
-#' and a known laboratory dose administered which is treated as unknown. Then
+#' and a known laboratory dose administered, which is treated as unknown. Then
 #' the De measurement is carried out and the degree of congruence between
 #' administered and recovered dose is a measure of the protocol's accuracy for
 #' this sample.\cr
@@ -18,19 +20,19 @@
 #'
 #' @param given.dose [numeric] (*optional*):
 #' given dose used for the dose recovery test to normalise data.
-#' If only one given dose is provided this given dose is valid for all input
-#' data sets (i.e., `values` is a list).  Otherwise a given dose for each input
+#' If only one given dose is provided, this given dose is valid for all input
+#' data sets (i.e., `values` is a list). Otherwise, a given dose for each input
 #' data set has to be provided (e.g., `given.dose = c(100,200)`).
-#' If `given.dose` in `NULL` the values are plotted without normalisation
+#' If `given.dose` is `NULL` or 0, the values are plotted without normalisation
 #' (might be useful for preheat plateau tests).
 #' **Note:** Unit has to be the same as from the input values (e.g., Seconds or
 #' Gray).
 #'
-#' @param error.range [numeric]:
+#' @param error.range [numeric] (*with default*):
 #' symmetric error range in percent will be shown as dashed lines in the plot.
-#' Set `error.range` to 0 to void plotting of error ranges.
+#' It can be set to 0 to remove the error ranges.
 #'
-#' @param preheat [numeric]:
+#' @param preheat [numeric] (*optional*):
 #' optional vector of preheat temperatures to be used for grouping the De values.
 #' If specified, the temperatures are assigned to the x-axis.
 #'
@@ -38,7 +40,7 @@
 #' plot values that are grouped by preheat temperature as boxplots.
 #' Only possible when `preheat` vector is specified.
 #'
-#' @param mtext [character]:
+#' @param mtext [character] (*with default*):
 #' additional text below the plot title.
 #'
 #' @param summary [character] (*optional*):
@@ -47,6 +49,7 @@
 #' - `"mean"` (mean De value),
 #' - `"weighted$mean"` (error-weighted mean),
 #' - `"median"` (median of the De values),
+#' - `"weighted$median"` (error-weighted median),
 #' - `"sd.rel"` (relative standard deviation in percent),
 #' - `"sd.abs"` (absolute standard deviation),
 #' - `"se.rel"` (relative standard error) and
@@ -75,8 +78,9 @@
 #' @param na.rm [logical] (*with default*): indicating whether `NA` values are
 #' removed before plotting from the input data set
 #'
-#' @param ... further arguments and graphical parameters passed to [plot], supported are:
-#' `xlab`, `ylab`, `xlim`, `ylim`, `main`, `cex`, `las` and `pch``
+#' @param ... further arguments and graphical parameters passed to [plot],
+#' supported are:
+#' `xlab`, `ylab`, `xlim`, `ylim`, `main`, `cex`, `las` and `pch`.
 #'
 #' @return A plot is returned.
 #'
@@ -84,7 +88,7 @@
 #' Further data and plot arguments can be added by using the appropriate R
 #' commands.
 #'
-#' @section Function version: 0.1.14
+#' @section Function version: 0.1.17
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)\cr
@@ -176,19 +180,18 @@
 #'   preheat = c(200, 200, 200, 240, 240),
 #'   boxplot = TRUE)
 #'
-#' @md
 #' @export
 plot_DRTResults <- function(
   values,
   given.dose = NULL,
   error.range = 10,
-  preheat,
+  preheat = NULL,
   boxplot = FALSE,
-  mtext,
-  summary,
-  summary.pos,
-  legend,
-  legend.pos,
+  mtext = "",
+  summary = "",
+  summary.pos = "topleft",
+  legend = NULL,
+  legend.pos = "topright",
   par.local = TRUE,
   na.rm  = FALSE,
   ...
@@ -199,30 +202,44 @@ plot_DRTResults <- function(
   ## Integrity checks -------------------------------------------------------
 
   .validate_not_empty(values)
+  .validate_class(given.dose, "numeric", null.ok = TRUE)
+  .validate_logical_scalar(boxplot)
 
   ##avoid crash for wrongly set boxlot argument
-  if(missing(preheat) & boxplot == TRUE){
+  if (is.null(preheat) && boxplot) {
     boxplot <- FALSE
-    .throw_warning("Option 'boxplot' requires a value in 'preheat', ",
-                   "reset to FALSE")
+    .throw_warning("'boxplot' requires a value in 'preheat', reset to FALSE")
   }
 
-  if(missing(summary) == TRUE) {summary <- NULL}
-  if(missing(summary.pos) == TRUE) {summary.pos <- "topleft"}
-  if(missing(legend.pos) == TRUE) {legend.pos <- "topright"}
-  if(missing(mtext) == TRUE) {mtext <- ""}
+  valid.pos <- c("left", "center", "right", "topleft", "top", "topright",
+                 "bottomleft", "bottom", "bottomright")
+  .validate_class(summary, "character")
+  if (is.numeric(summary.pos)) {
+    .validate_length(summary.pos, 2)
+  }
+  else {
+    summary.pos <- .validate_args(summary.pos, c("sub", valid.pos))
+  }
+  .validate_class(legend, "character", null.ok = TRUE)
+  if (is.numeric(legend.pos)) {
+    .validate_length(legend.pos, 2)
+  }
+  else {
+    legend.pos <- .validate_args(legend.pos, valid.pos)
+  }
 
   ## Homogenise and check input data
-  if(is(values, "list") == FALSE) {values <- list(values)}
+  if (!inherits(values, "list"))
+      values <- list(values)
 
   for (i in seq_along(values)) {
     .validate_class(values[[i]], c("data.frame", "RLum.Results"),
                     name = "'values'")
     if (inherits(values[[i]], "RLum.Results")) {
-      val <- get_RLum(values[[i]])[, 1:2]
-      if (is.null(val))
-        val <- NA
+      val <- get_RLum(values[[i]])[, 1:2] %||% NA
       values[[i]] <- val
+    } else if (ncol(values[[i]]) < 2) {
+      .throw_error("'values' should have 2 columns")
     }
   }
 
@@ -236,17 +253,15 @@ plot_DRTResults <- function(
   for (i in seq_along(values)) {
 
     ##check for preheat temperature values
-    if(missing(preheat) == FALSE) {
-      if(length(preheat) != nrow(values[[i]])){
-        .throw_error("Number of preheat temperatures != De values")
-      }
+    if (!missing(preheat) && length(preheat) < nrow(values[[i]])) {
+        .throw_error("'preheat' should have length equal to the number ",
+                     "of De values")
     }
 
     ##remove NA values; yes Micha, it is not that simple
-    if(na.rm  == TRUE){
-
+    if (!na.rm) {
       ##currently we assume that all input data sets comprise a similar of data
-      if(!missing(preheat) & i == length(values)){
+      if (!is.null(preheat) && i == length(values)) {
         ## remove preheat entries corresponding to NA values
         preheat <- preheat[!is.na(values[[i]][, 1]) &
                            !is.na(values[[i]][, 2])]
@@ -267,75 +282,58 @@ plot_DRTResults <- function(
   ## Set plot format parameters -----------------------------------------------
   extraArgs <- list(...) # read out additional arguments list
 
-  main <- if("main" %in% names(extraArgs)) {extraArgs$main} else
-  {"Dose recovery test"}
+  main <- extraArgs$main %||% "Dose recovery test"
+  xlab <- extraArgs$xlab %||% ifelse(is.null(preheat),
+                                     "# Aliquot", "Preheat temperature [\u00B0C]")
 
-  xlab <- if("xlab" %in% names(extraArgs)) {extraArgs$xlab} else {
-    ifelse(missing(preheat) == TRUE, "# Aliquot", "Preheat temperature [\u00B0C]")
-  }
+  ylab <- extraArgs$ylab %||% (
+    if (!is.null(given.dose) && length(given.dose) > 0 && given.dose[1] > 0)
+      expression(paste("Normalised ", D[e]))
+    else expression(paste(D[e], " [s]"))
+  )
 
-  ylab <- if("ylab" %in% names(extraArgs)) {extraArgs$ylab} else
-  {if(!is.null(given.dose)){
-    expression(paste("Normalised ", D[e], sep=""))
-  }else{expression(paste(D[e], " [s]"), sep = "")}}
-
-  xlim <- if("xlim" %in% names(extraArgs)) {extraArgs$xlim} else
-  {c(1, max(n.values) + 1)}
-
-  ylim <- if("ylim" %in% names(extraArgs)) {extraArgs$ylim} else
-  {c(0.75, 1.25)} #check below for further corrections if boundaries exceed set range
-
-  cex <- if("cex" %in% names(extraArgs)) {extraArgs$cex} else {1}
-
-  pch <- if("pch" %in% names(extraArgs)) {extraArgs$pch} else {
-    abs(seq(from = 20, to = -100))
-  }
-
-  ##axis labels
-  las <- if("las" %in% names(extraArgs)) extraArgs$las else 0
-
-  fun <- if ("fun" %in% names(extraArgs)) extraArgs$fun else FALSE # nocov
+  xlim <- extraArgs$xlim %||% (c(0, max(n.values)) + 0.5)
+  ylim <- extraArgs$ylim %||% c(0.75, 1.25) # check below for further corrections if boundaries exceed set range
+  cex <- extraArgs$cex %||% 1
+  pch <- extraArgs$pch %||% abs(seq(from = 20, to = -100))
+  las <- extraArgs$las %||% 0
+  fun <- isTRUE(extraArgs$fun)
 
   ## calculations and settings-------------------------------------------------
 
   ## normalise data if given.dose is given
-  if(!is.null(given.dose)){
+  if (!is.null(given.dose)) {
+    .validate_not_empty(given.dose)
+    if (length(given.dose) == 1) {
+      given.dose <- rep(given.dose, length(values))
+    }
+    else if (length(given.dose) != length(values)) {
+      .throw_error("'given.dose' should have length equal to the number ",
+                   "of input data sets")
+    }
 
-    if(length(given.dose) > 1){
-
-      if(length(values) < length(given.dose)){
-        .throw_error("'given.dose' > number of input data sets")
-      }
-
-      for(i in 1:length(values)) {
-        values[[i]] <- values[[i]]/given.dose[i]
-      }
-
-    }else{
-
-      for(i in 1:length(values)) {
-        values[[i]] <- values[[i]]/given.dose
+    if (all(given.dose > 0)) {
+      for (i in 1:length(values)) {
+        values[[i]] <- values[[i]] / given.dose[i]
       }
+    } else {
+      given.dose <- NULL
     }
   }
 
   ##correct ylim for data set which exceed boundaries
-  if((max(sapply(1:length(values), function(x){max(values[[x]][,1], na.rm = TRUE)}))>1.25 |
-        min(sapply(1:length(values), function(x){min(values[[x]][,1], na.rm = TRUE)}))<0.75) &
-       ("ylim" %in% names(extraArgs)) == FALSE){
-
+  if ((max(sapply(values, function(x) max(x[, 1], na.rm = TRUE))) > 1.25 ||
+       min(sapply(values, function(x) min(x[, 1], na.rm = TRUE))) < 0.75) &&
+       (!"ylim" %in% names(extraArgs))) {
     ylim <- c(
-      min(sapply(1:length(values), function(x){
-        min(values[[x]][,1], na.rm = TRUE) - max(values[[x]][,2], na.rm = TRUE)})),
-      max(sapply(1:length(values), function(x){
-        max(values[[x]][,1], na.rm = TRUE) + max(values[[x]][,2], na.rm = TRUE)})))
+      min(sapply(values, function(x) min(x[, 1], na.rm = TRUE) - max(x[, 2], na.rm = TRUE))),
+      max(sapply(values, function(x) max(x[, 1], na.rm = TRUE) + max(x[, 2], na.rm = TRUE))))
   }
 
   ## optionally group data by preheat temperature
-  if(missing(preheat) == FALSE) {
-    modes <- as.numeric(rownames(as.matrix(table(preheat))))
-    values.preheat <- list(NA)
-    values.boxplot <- list(NA)
+  if (!is.null(preheat)) {
+    modes <- as.numeric(names(table(preheat)))
+    values.preheat <- values.boxplot <- list()
     for(i in 1:length(modes)) {
       for(j in 1:length(values)) {
         values.preheat[[length(values.preheat) + 1]] <-
@@ -346,38 +344,39 @@ plot_DRTResults <- function(
       }
       j <- 1
     }
-    values.preheat[[1]] <- NULL
-    values.boxplot[[1]] <- NULL
     modes.plot <- rep(modes, each = length(values))
+    xlim <- c(min(modes.plot) * 0.9, max(modes.plot) * 1.1)
   } else {
     modes <- 1
   }
 
+  if (boxplot)
+    xlim <- c(0.5, length(unique(preheat)) + 0.5)
+
   ## assign colour indices
-  col <- if("col" %in% names(extraArgs)) {extraArgs$col} else {
-    if(missing(preheat) == TRUE) {
+  col <- extraArgs$col %||% (
+    if (is.null(preheat)) {
       rep(seq(from = 1, to = length(values)), each = length(modes))
     } else {
       rep(seq(from = 1, to = length(values)), length(modes))
     }
-  }
+  )
 
   ## calculate and paste statistical summary
   if(summary.pos[1] != "sub") {
     label.text <- lapply(1:length(values), function(i) {
       .create_StatisticalSummaryText(
-        x = calc_Statistics(values[[i]]),
+        calc_Statistics(values[[i]]),
         keywords = summary,
         digits = 2,
         sep = " \n",
-        prefix = paste(rep("\n", (i - 1) * length(summary)), collapse = "")
+        prefix = strrep("\n", (i - 1) * length(summary))
       )
     })
-
   }else{
-    label.text <- lapply(1:length(values), function(i) {
+    label.text <- lapply(values, function(x) {
       .create_StatisticalSummaryText(
-        x = calc_Statistics(values[[i]]),
+        calc_Statistics(x),
         keywords = summary,
         digits = 2,
         sep = " | "
@@ -385,33 +384,33 @@ plot_DRTResults <- function(
     })
   }
 
+  ## keep track if the summary is in the bottom row as we may need to compute
+  ## an adjustment further down, after the plot device has been opened
+  summary.pos_is_bottom <- grepl("bottom", summary.pos[1])
+
   ## convert keywords into summary and legend placement coordinates
   coords <- .get_keyword_coordinates(summary.pos, xlim, ylim)
   summary.pos <- coords$pos
-  summary.adj <- coords$adj
-  coords <- .get_keyword_coordinates(legend.pos, xlim, ylim)
-  legend.pos <- coords$pos
-  legend.adj <- coords$adj
+  summary.adj <- c(coords$adj[1], 1) # always top-aligned
 
   ## Plot output ------------------------------------------------------------
 
-  ## determine number of subheader lines to shif the plot
-  shift.lines <- if(summary.pos[1] == "sub" & mtext == "") {
-    length(label.text) - 1
-  } else {1}
+  ## determine number of subheader lines to shift the plot
+  shift.lines <- if (mtext == "") {
+                   if (summary.pos[1] == "sub") length(label.text) else 0
+                 } else 1
 
   ## setup plot area
   if(par.local){
-    if (shift.lines <= 0)
-    shift.lines <- 1
-    par.default <- par(mfrow = c(1, 1), cex = cex, oma = c(0, 1, shift.lines - 1, 1))
+    par.default <- par(mfrow = c(1, 1), cex = cex,
+                       mar = c(2.5, 2.5, shift.lines, 0) + 2.1)
     on.exit(par(par.default), add = TRUE)
   }
 
   ## optionally plot values and error bars
-  if(boxplot == FALSE) {
+  if (!boxplot) {
     ## plot data and error
-    if(missing(preheat) == TRUE) {
+    if (is.null(preheat)) {
       ## create empty plot
       plot(NA,NA,
            xlim = xlim,
@@ -427,7 +426,7 @@ plot_DRTResults <- function(
 
       ## add title
       title(main = main,
-            line = shift.lines + 2)
+            line = shift.lines + 0.5)
 
       ## add additional lines
       if (!is.null(given.dose)) {
@@ -442,55 +441,63 @@ plot_DRTResults <- function(
           text(
             par()$usr[2],
             (1 + error.range / 100) + 0.02,
-            paste("+", error.range , " %", sep = ""),
+            paste0("+", error.range , "%"),
             pos = 2,
             cex = 0.8
           )
           text(
             par()$usr[2],
             (1 - error.range / 100) - 0.02,
-            paste("-", error.range , "%", sep = ""),
+            paste0("-", error.range , "%"),
             pos = 2,
             cex = 0.8
           )
         }
-
       }
 
+      ## allow assigning a separate colour to each point, but only if there
+      ## is one input dataset
+      oneinput <- length(values) == 1
+      multicol <- oneinput && nrow(values[[1]]) == length(col)
+
       ## add data and error bars
       for(i in 1:length(values)) {
-
-        points(x = c(1:nrow(values[[i]])),
+        points(x = 1:nrow(values[[i]]),
                y = values[[i]][,1],
-               pch = if(nrow(values[[i]]) == length(pch)){ pch } else { pch[i] },
-               col = if(nrow(values[[i]]) == length(col)){ col } else { col[i] },
-               cex = 1.2 * cex)
+               pch = if (oneinput && nrow(values[[i]]) == length(pch)) pch else pch[i],
+               col = if (multicol) col else col[i],
+               cex = 1.2)
 
-        graphics::arrows(c(1:nrow(values[[i]])),
+        suppressWarnings( # zero-length arrow is of indeterminate angle and so skipped
+        graphics::arrows(1:nrow(values[[i]]),
                values[[i]][,1] + values[[i]][,2],
-               c(1:nrow(values[[i]])),
+               1:nrow(values[[i]]),
                values[[i]][,1] - values[[i]][,2],
                angle = 90,
                length = 0.075,
                code = 3,
-               col = if(nrow(values[[i]]) == length(col)){ col } else { col[i] })
+               col = if (multicol) col else col[i])
+        )
 
         ## add summary content
         if(summary.pos[1] != "sub") {
+          vadj <- 0
+          if (summary.pos_is_bottom) {
+            ## adjust the vertical coordinate by the height of the longest label
+            vadj <- graphics::strheight(tail(label.text, 1), cex = 0.8)
+          }
           text(x = summary.pos[1],
-               y = summary.pos[2],
+               y = summary.pos[2] + vadj,
                adj = summary.adj,
                labels = label.text[[i]],
-               cex = 0.8 * cex,
-               col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] })
-        } else {
-          if(mtext == "") {
+               cex = 0.8,
+               col = if (multicol) "black" else col[i])
+        } else if(mtext == "") {
             mtext(side = 3,
-                  line = - i + 2.5,
+                  line = shift.lines - i,
                   text = label.text[[i]],
-                  col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] },
+                  col = if (multicol) "black" else col[i],
                   cex = cex * 0.8)
-          }
         }
       }
     } else {
@@ -498,7 +505,7 @@ plot_DRTResults <- function(
       ## option for provided preheat data
       ## create empty plot
       plot(NA,NA,
-           xlim = c(min(modes.plot) * 0.9, max(modes.plot) * 1.1),
+           xlim = xlim,
            ylim = ylim,
            xlab = xlab,
            ylab = ylab,
@@ -515,7 +522,7 @@ plot_DRTResults <- function(
 
       ## add title
       title(main = main,
-            line = shift.lines + 2)
+            line = shift.lines + 0.5)
 
       ## add additional lines
       if (!is.null(given.dose)) {
@@ -530,14 +537,14 @@ plot_DRTResults <- function(
           text(
             par()$usr[2],
             (1 + error.range / 100) + 0.02,
-            paste("+", error.range , " %", sep = ""),
+            paste0("+", error.range , "%"),
             pos = 2,
             cex = 0.8
           )
           text(
             par()$usr[2],
             (1 - error.range / 100) - 0.02,
-            paste("-", error.range , "%", sep = ""),
+            paste0("-", error.range , "%"),
             pos = 2,
             cex = 0.8
           )
@@ -550,8 +557,9 @@ plot_DRTResults <- function(
                y = values.preheat[[i]][,1],
                pch = pch[i],
                col = col[i],
-               cex = 1.2 * cex)
+               cex = 1.2)
 
+        suppressWarnings( # zero-length arrow is of indeterminate angle and so skipped
         graphics::arrows(values.preheat[[i]][,3],
                values.preheat[[i]][,1] + values.preheat[[i]][,2],
                values.preheat[[i]][,3],
@@ -560,6 +568,7 @@ plot_DRTResults <- function(
                length = 0.075,
                code = 3,
                col = col[i])
+        )
       }
     }
   }
@@ -578,10 +587,10 @@ plot_DRTResults <- function(
             border = col)
 
     ## add axis label, if necessary
-    if (length(modes.plot) == 1) {
-      axis(side = 1, at = 1, labels = modes.plot, las = las)
-
-    } else if (length(modes.plot) > length(unique(modes.plot))){
+    if (length(modes.plot) <= length(unique(modes.plot))) {
+      axis(side = 1, at = 1:length(unique(modes.plot)),
+           labels = unique(modes.plot), las = las)
+    } else {
       ticks <- seq(from = 1 + ((length(values.boxplot)/length(unique(modes.plot)) - 1)/2),
                    to = length(values.boxplot),
                    by = length(values.boxplot)/length(unique(modes.plot)))
@@ -599,6 +608,8 @@ plot_DRTResults <- function(
       )
 
       polygon.step <- unique(diff(polygon.x) - 1)
+      if (length(polygon.step) == 0)
+        polygon.step <- 1
 
       for (x.plyg in polygon.x) {
         polygon(
@@ -611,17 +622,12 @@ plot_DRTResults <- function(
           ),
           col = "grey",
           border = "grey")
-
       }
-
-    }else{
-      axis(side = 1, at = 1:length(unique(modes.plot)), labels = unique(modes.plot), las = las)
-
     }
 
     ## add title
     title(main = main,
-          line = shift.lines + 2)
+          line = shift.lines + 0.5)
 
     ## add additional lines
     if(!is.null(given.dose)){
@@ -634,9 +640,9 @@ plot_DRTResults <- function(
 
         ## error range labels
         text(par()$usr[2], (1 + error.range / 100) + 0.02,
-             paste("+", error.range ," %", sep = ""), pos = 2, cex = 0.8)
+             paste0("+", error.range ,"%"), pos = 2, cex = 0.8)
         text(par()$usr[2], (1 - error.range / 100) - 0.02,
-             paste("-", error.range ,"%", sep = ""), pos = 2, cex = 0.8)
+             paste0("-", error.range ,"%"), pos = 2, cex = 0.8)
       }
     }
 
@@ -644,26 +650,32 @@ plot_DRTResults <- function(
     for(i in 1:length(values)) {
       ## add summary content
       if(summary.pos[1] != "sub") {
+        vadj <- 0
+        if (summary.pos_is_bottom) {
+          ## adjust the vertical coordinate by the height of the longest label
+          vadj <- graphics::strheight(tail(label.text, 1), cex = 0.8)
+        }
         text(x = summary.pos[1],
-             y = summary.pos[2],
+             y = summary.pos[2] + vadj,
              adj = summary.adj,
              labels = label.text[[i]],
-             cex = 0.8 * cex,
+             cex = 0.8,
              col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] })
-      } else {
-        if(mtext == "") {
+      } else if (mtext == "") {
           mtext(side = 3,
-                line = - i + 2.5,
+                line = shift.lines - i,
                 text = label.text[[i]],
                 col = if(nrow(values[[i]]) == length(col)){ "black" } else { col[i] },
                 cex = cex * 0.8)
-        }
       }
     }
   }
 
   ## optionally add legend content
-  if(missing(legend) == FALSE) {
+  if (!is.null(legend)) {
+    coords <- .get_keyword_coordinates(legend.pos, xlim, ylim)
+    legend.pos <- coords$pos
+    legend.adj <- coords$adj
     legend(x = legend.pos[1],
            y = legend.pos[2],
            xjust = legend.adj[1],
@@ -672,15 +684,14 @@ plot_DRTResults <- function(
            col = unique(col),
            pch = unique(pch),
            lty = 1,
-           cex = cex * 0.8)
+           cex = 0.8)
   }
 
   ## optionally add subheader text
   mtext(side = 3,
-        line = shift.lines,
         text = mtext,
         cex = 0.8 * cex)
 
   ##FUN by R Luminescence Team
-  if (fun == TRUE) sTeve() # nocov
+  if (fun) sTeve() # nocov
 }
