@@ -1,5 +1,6 @@
-#' Plot kernel density estimate with statistics
+#' @title Plot kernel density estimate with statistics
 #'
+#' @description
 #' Plot a kernel density estimate of measurement values in combination with the
 #' actual values and associated error bars in ascending order. If enabled, the
 #' boxplot will show the usual distribution parameters (median as
@@ -15,7 +16,6 @@
 #' calculation of the bin-width (parameter `bw`) see
 #' [density].
 #'
-#'
 #' A statistic summary, i.e. a collection of statistic measures of
 #' centrality and dispersion (and further measures) can be added by specifying
 #' one or more of the following keywords:
@@ -30,18 +30,16 @@
 #' - `"kurtosis"` (kurtosis)
 #' - `"skewness"` (skewness)
 #'
-#'
-#' **Note** that the input data for the statistic summary is sent to the function
-#' `calc_Statistics()` depending on the log-option for the z-scale. If
+#' **Note** that the input data for the statistic summary is sent to function
+#' [calc_Statistics] depending on the log-option for the z-scale. If
 #' `"log.z = TRUE"`, the summary is based on the logarithms of the input
-#' data. If `"log.z = FALSE"` the linearly scaled data is used.
+#' data. If `"log.z = FALSE"` the linearly-scaled data is used.
 #'
 #' **Note** as well, that `"calc_Statistics()"` calculates these statistic
 #' measures in three different ways: `unweighted`, `weighted` and
 #' `MCM-based` (i.e., based on Monte Carlo Methods). By default, the
-#' MCM-based version is used. If you wish to use another method, indicate this
-#' with the appropriate keyword using the argument `summary.method`.
-#'
+#' MCM-based version is used. This can be controlled via the `summary.method`
+#' argument.
 #'
 #' @param data [data.frame], [vector] or [RLum.Results-class] object (**required**):
 #' for `data.frame`: either two columns: De (`values[,1]`) and De error
@@ -68,11 +66,11 @@
 #' @param rug [logical] (*with default*):
 #' optionally add rug.
 #'
-#' @param summary [character] (*optional*):
+#' @param summary [character] (*with default*):
 #' add statistic measures of centrality and dispersion to the plot. Can be one
 #' or more of several keywords. See details for available keywords.
 #'
-#' @param summary.pos [numeric] or [character] (*optional*):
+#' @param summary.pos [numeric] or [character] (*with default*):
 #' optional position coordinates or keyword (e.g. `"topright"`)
 #' for the statistical summary. Alternatively, the keyword `"sub"` may be
 #' specified to place the summary below the plot header. However, this latter
@@ -81,16 +79,12 @@
 #'
 #' @param summary.method [character] (*with default*):
 #' keyword indicating the method used to calculate the statistic summary.
-#' One out of `"unweighted"`, `"weighted"` and `"MCM"`.
+#' One out of `"MCM"` (default), `"weighted"` or `"unweighted"`.
 #' See [calc_Statistics] for details.
 #'
-#' @param bw [character] (*with default*):
+#' @param bw [character], [numeric] (*with default*):
 #' bin-width, chose a numeric value for manual setting.
 #'
-#' @param output [logical] (*with default*):
-#' Optional output of numerical plot parameters. These can be useful to
-#' reproduce similar plots. Default is `TRUE`.
-#'
 #' @param ... further arguments and graphical parameters passed to [plot].
 #'
 #' @note
@@ -101,7 +95,7 @@
 #'
 #' @author
 #' Michael Dietze, GFZ Potsdam (Germany)\cr
-#' Geography & Earth Sciences, Aberystwyth University (United Kingdom)
+#' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
 #' @seealso [density], [plot]
 #'
@@ -156,10 +150,8 @@
 #'
 #' ## example of how to use the numerical output of the function
 #' ## return plot output to draw a thicker KDE line
-#' KDE_out <- plot_KDE(data = ExampleData.DeValues,
-#' output = TRUE)
+#' KDE_out <- plot_KDE(data = ExampleData.DeValues)
 #'
-#' @md
 #' @export
 plot_KDE <- function(
   data,
@@ -168,24 +160,23 @@ plot_KDE <- function(
   order = TRUE,
   boxplot = TRUE,
   rug = TRUE,
-  summary,
-  summary.pos,
+  summary = "",
+  summary.pos = "sub",
   summary.method = "MCM",
   bw = "nrd0",
-  output = TRUE,
   ...
 ) {
   .set_function_name("plot_KDE")
   on.exit(.unset_function_name(), add = TRUE)
 
-  ## check data and parameter consistency -------------------------------------
+  ## Integrity checks -------------------------------------------------------
 
-  if (is(data, "list") && length(data) == 0) {
+  if (inherits(data, "list") && length(data) == 0) {
     .throw_error("'data' is an empty list")
   }
 
   ## Homogenise input data format
-  if(is(data, "list") == FALSE) {
+  if (!inherits(data, "list")) {
     data <- list(data)
   }
 
@@ -237,49 +228,49 @@ plot_KDE <- function(
   if(length(data) == 0)
     .throw_error("Your input is empty due to Inf removal")
 
-  ## check/set function parameters
-  if(missing(summary) == TRUE) {
-    summary <- ""
+  .validate_logical_scalar(na.rm)
+  .validate_logical_scalar(values.cumulative)
+  .validate_logical_scalar(order)
+  .validate_logical_scalar(boxplot)
+  .validate_logical_scalar(rug)
+  .validate_args(summary.method, c("MCM", "weighted", "unweighted"))
+  .validate_class(summary, "character")
+  if (is.numeric(summary.pos)) {
+    .validate_length(summary.pos, 2)
   }
-
-  if(missing(summary.pos) == TRUE) {
-    summary.pos <- "sub"
+  else {
+    summary.pos <- .validate_args(summary.pos,
+                                  c("sub", "left", "center", "right",
+                                    "topleft", "top", "topright",
+                                    "bottomleft", "bottom", "bottomright"))
   }
+  .validate_class(bw, c("character", "numeric"), length = 1)
+  if (is.numeric(bw))
+    .validate_positive_scalar(bw)
 
   ## set mtext output
-  if("mtext" %in% names(list(...))) {
-    mtext <- list(...)$mtext
-  } else {
-    mtext <- ""
-  }
+  mtext <- list(...)$mtext %||% ""
 
   ## check/set layout definitions
-  if("layout" %in% names(list(...))) {
-    layout <- get_Layout(layout = list(...)$layout)
-  } else {
-    layout <- get_Layout(layout = "default")
-  }
+  layout <- get_Layout(layout = list(...)$layout %||% "default")
 
   ## data preparation steps ---------------------------------------------------
 
   ## optionally, count and exclude NA values and print result
-  if(na.rm == TRUE) {
+  if (na.rm) {
     for(i in 1:length(data)) {
       na.idx <- which(is.na(data[[i]][, 1]))
       n.NA <- length(na.idx)
-      if(n.NA == 1) {
-        message(paste("1 NA value excluded from data set", i, "."))
-      } else if(n.NA > 1) {
-        message(paste(n.NA, "NA values excluded from data set", i, "."))
-      }
       if (n.NA > 0) {
+        message(sprintf("%d NA value%s excluded from data set %d\n",
+                        n.NA, ifelse(n.NA > 1, "s", ""), i))
         data[[i]] <- data[[i]][-na.idx, ]
       }
     }
   }
 
   ## optionally, order data set
-  if(order == TRUE) {
+  if (order) {
     for(i in 1:length(data)) {
       data[[i]] <- data[[i]][order(data[[i]][,1]),]
     }
@@ -290,7 +281,7 @@ plot_KDE <- function(
   colnames(De.stats) <- c("n",
                           "mean",
                           "median",
-                          "kde.max",
+                          "kdemax",
                           "sd.abs",
                           "sd.rel",
                           "se.abs",
@@ -350,10 +341,6 @@ plot_KDE <- function(
 
       ## position of maximum KDE value
       De.stats[i,4] <- De.density[[i]]$x[which.max(De.density[[i]]$y)]
-
-    }else{
-      De.density.range[i,1:4] <- NA
-      De.stats[i,4] <- NA
     }
   }
 
@@ -363,305 +350,114 @@ plot_KDE <- function(
                         min(De.density.range[,3]),
                         max(De.density.range[,4]))
 
-  label.text = list(NA)
+  ## helper to generate an element of the statistical summary
+  .summary_line <- function(keyword, summary, val, label = keyword,
+                            percent = FALSE, sep = FALSE, digits = 2) {
+    ifelse(keyword %in% summary,
+           paste0(label, " = ", round(val, digits),
+                  if (percent) " %" else NULL, if (sep) " | " else  "\n"),
+           "")
+  }
 
-  if(summary.pos[1] != "sub") {
-    n.rows <- length(summary)
+  ## initialize list
+  label.text <- NULL
+  is.sub <- summary.pos[1] == "sub"
+  stops <- NULL
+  for (i in 1:length(data)) {
+    if (!is.sub)
+      stops <- strrep("\n", (i - 1) * length(summary))
 
-    for(i in 1:length(data)) {
-      stops <- paste(rep("\n", (i - 1) * n.rows), collapse = "")
-
-      summary.text <- character(0)
-
-      for(j in 1:length(summary)) {
-        summary.text <- c(summary.text,
-                          paste(
-                            "",
-                            ifelse("n" %in% summary[j] == TRUE,
-                                   paste("n = ",
-                                         De.stats[i,1],
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("mean" %in% summary[j] == TRUE,
-                                   paste("mean = ",
-                                         round(De.stats[i,2], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("median" %in% summary[j] == TRUE,
-                                   paste("median = ",
-                                         round(De.stats[i,3], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("kde.max" %in% summary[j] == TRUE,
-                                   paste("kdemax = ",
-                                         round(De.stats[i,4], 2),
-                                         " \n ",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sd.abs" %in% summary[j] == TRUE,
-                                   paste("sd = ",
-                                         round(De.stats[i,5], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("sd.rel" %in% summary[j] == TRUE,
-                                   paste("rel. sd = ",
-                                         round(De.stats[i,6], 2), " %",
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("se.abs" %in% summary[j] == TRUE,
-                                   paste("se = ",
-                                         round(De.stats[i,7], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("se.rel" %in% summary[j] == TRUE,
-                                   paste("rel. se = ",
-                                         round(De.stats[i,8], 2), " %",
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("skewness" %in% summary[j] == TRUE,
-                                   paste("skewness = ",
-                                         round(De.stats[i,11], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("kurtosis" %in% summary[j] == TRUE,
-                                   paste("kurtosis = ",
-                                         round(De.stats[i,12], 2),
-                                         "\n",
-                                         sep = ""),
-                                   ""),
-                            ifelse("in.2s" %in% summary[j] == TRUE,
-                                   paste("in 2 sigma = ",
-                                         round(sum(data[[i]][,1] >
-                                                     (De.stats[i,2] - 2 *
-                                                        De.stats[i,5]) &
-                                                     data[[i]][,1] <
-                                                     (De.stats[i,2] + 2 *
-                                                        De.stats[i,5])) /
-                                                 nrow(data[[i]]) * 100 , 1),
-                                         " %",
-                                         sep = ""),
-                                   ""),
-                            sep = ""))
-      }
-
-      summary.text <- paste(summary.text, collapse = "")
-
-      label.text[[length(label.text) + 1]] <- paste(stops,
-                                                    summary.text,
-                                                    stops,
-                                                    sep = "")
-    }
-  } else {
-    for(i in 1:length(data)) {
-
-      summary.text <- character(0)
-
-      for(j in 1:length(summary)) {
-        summary.text <- c(summary.text,
-                          ifelse("n" %in% summary[j] == TRUE,
-                                 paste("n = ",
-                                       De.stats[i,1],
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("mean" %in% summary[j] == TRUE,
-                                 paste("mean = ",
-                                       round(De.stats[i,2], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("median" %in% summary[j] == TRUE,
-                                 paste("median = ",
-                                       round(De.stats[i,3], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("kde.max" %in% summary[j] == TRUE,
-                                 paste("kdemax = ",
-                                       round(De.stats[i,4], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sd.rel" %in% summary[j] == TRUE,
-                                 paste("rel. sd = ",
-                                       round(De.stats[i,6], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("sd.abs" %in% summary[j] == TRUE,
-                                 paste("abs. sd = ",
-                                       round(De.stats[i,5], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("se.rel" %in% summary[j] == TRUE,
-                                 paste("rel. se = ",
-                                       round(De.stats[i,8], 2), " %",
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("se.abs" %in% summary[j] == TRUE,
-                                 paste("abs. se = ",
-                                       round(De.stats[i,7], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("skewness" %in% summary[j] == TRUE,
-                                 paste("skewness = ",
-                                       round(De.stats[i,11], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("kurtosis" %in% summary[j] == TRUE,
-                                 paste("kurtosis = ",
-                                       round(De.stats[i,12], 2),
-                                       " | ",
-                                       sep = ""),
-                                 ""),
-                          ifelse("in.2s" %in% summary[j] == TRUE,
-                                 paste("in 2 sigma = ",
-                                       round(sum(data[[i]][,1] >
-                                                   (De.stats[i,2] - 2 *
-                                                      De.stats[i,5]) &
-                                                   data[[i]][,1] <
-                                                   (De.stats[i,2] + 2 *
-                                                      De.stats[i,5])) /
-                                               nrow(data[[i]]) * 100 , 1),
-                                       " %   ",
-                                       sep = ""),
-                                 "")
-        )
-      }
-
-      summary.text <- paste(summary.text, collapse = "")
-
-      label.text[[length(label.text) + 1]]  <- paste(
-        "  ",
-        summary.text,
-        sep = "")
+    summary.text <- character(0)
+    for (j in 1:length(summary)) {
+      summary.text <-
+        c(summary.text,
+          .summary_line("n", summary[j], De.stats[i, 1], sep = is.sub),
+          .summary_line("mean", summary[j], De.stats[i, 2], sep = is.sub),
+          .summary_line("median", summary[j], De.stats[i, 3], sep = is.sub),
+          .summary_line("kdemax", summary[j], De.stats[i, 4], sep = is.sub),
+          .summary_line("sd.abs", summary[j], De.stats[i, 5], sep = is.sub,
+                        label = "sd"),
+          .summary_line("sd.rel", summary[j], De.stats[i, 6], sep = is.sub,
+                        label = "rel. sd", percent = TRUE),
+          .summary_line("se.abs", summary[j], De.stats[i, 7], sep = is.sub,
+                        label = "se"),
+          .summary_line("se.rel", summary[j], De.stats[i, 8], sep = is.sub,
+                        label = "rel. se", percent = TRUE),
+          .summary_line("skewness", summary[j], De.stats[i, 11], sep = is.sub),
+          .summary_line("kurtosis", summary[j], De.stats[i, 12], sep = is.sub),
+          .summary_line("in.2s", summary[j],
+                        sum(data[[i]][, 1] > (De.stats[i, 2] - 2 *
+                                              De.stats[i, 5]) &
+                            data[[i]][, 1] < (De.stats[i, 2] + 2 *
+                                              De.stats[i, 5])) /
+                        nrow(data[[i]]) * 100, sep = is.sub,
+                        label = "in 2 sigma", percent = TRUE, digits = 1))
     }
 
-    ## remove outer vertical lines from string
-    for(i in 2:length(label.text)) {
+    label.text[[i]] <- paste0(
+        if (is.sub) "" else stops,
+        paste(summary.text, collapse = ""),
+        stops)
+  }
+
+  ## remove outer vertical lines from string
+  if (is.sub) {
+    for (i in seq_along(label.text)) {
       label.text[[i]] <- substr(x = label.text[[i]],
-                                start = 3,
+                                start = 1,
                                 stop = nchar(label.text[[i]]) - 3)
     }
   }
 
-  ## remove dummy list element
-  label.text[[1]] <- NULL
-
   ## read out additional parameters -------------------------------------------
-  if("main" %in% names(list(...))) {
-    main <- list(...)$main
-  } else {
-    main <- expression(bold(paste(D[e], " distribution")))
-  }
+  main <- list(...)$main %||% expression(bold(paste(D[e], " distribution")))
+  sub <- list(...)$sub
+  xlab <- list(...)$xlab %||% expression(paste(D[e], " [Gy]"))
+  ylab <- list(...)$ylab %||% c("Density", "Cumulative frequency")
+  xlim.plot <- list(...)$xlim %||% c(min(c(De.global - De.error.global),
+                                         De.density.range[1],
+                                         na.rm = TRUE),
+                                     max(c(De.global + De.error.global),
+                                         De.density.range[2],
+                                         na.rm = TRUE))
 
-  if("sub" %in% names(list(...))) {
-    sub <- list(...)$sub
-  } else {
-    sub <- NULL
-  }
-
-  if("xlab" %in% names(list(...))) {
-    xlab <- list(...)$xlab
-  } else {
-    xlab <- expression(paste(D[e], " [Gy]"))
-  }
-
-  if("ylab" %in% names(list(...))) {
-    ylab <- list(...)$ylab
-  } else {
-    ylab <- c("Density", "Cumulative frequency")
-  }
-
-  if("xlim" %in% names(list(...))) {
-    xlim.plot <- list(...)$xlim
-  } else {
-    xlim.plot <- c(min(c(De.global - De.error.global),
-                       De.density.range[1],
-                       na.rm = TRUE),
-                   max(c(De.global + De.error.global),
-                       De.density.range[2],
-                       na.rm = TRUE))
-  }
-
-  if("ylim" %in% names(list(...))) {
+  if ("ylim" %in% ...names()) {
     ylim.plot <- list(...)$ylim
-    .validate_length(ylim.plot, 4, name = "'ylim'")
-  } else {
-    if(!is.na(De.density.range[1])){
+    .validate_class(ylim.plot, "numeric", length = 4, name = "'ylim'")
+  } else if (!is.na(De.density.range[1])) {
       ylim.plot <- c(De.density.range[3],
                      De.density.range[4],
                      0,
                      max(De.stats[,1]))
-
-    }else{
+  } else {
       ylim.plot <- c(0,
                      max(De.stats[,1]),
                      0,
                      max(De.stats[,1]))
-    }
   }
 
-  if("log" %in% names(list(...))) {
-    log.option <- list(...)$log
-  } else {
-    log.option <- ""
-  }
+  log.option <- list(...)$log %||% ""
+  lty <- list(...)$lty %||% rep(1, length(data))
+  lwd <- list(...)$lwd %||% rep(1, length(data))
+  cex <- list(...)$cex %||% 1
+  fun <- isTRUE(list(...)$fun)
 
-  if("col" %in% names(list(...))) {
-
-    col.main <- list(...)$col
-    col.xlab <- 1
-    col.ylab1 <- 1
-    col.ylab2 <- 1
-    col.xtck <- 1
-    col.ytck1 <- 1
-    col.ytck2 <- 1
-    col.box <- 1
-    col.mtext <- 1
+  if ("col" %in% ...names()) {
     col.stats <- list(...)$col
     col.kde.line <- list(...)$col
     col.kde.fill <- NA
     col.value.dot <- list(...)$col
     col.value.bar <- list(...)$col
     col.value.rug <- list(...)$col
-    col.boxplot <- list(...)$col
     col.boxplot.line <- list(...)$col
     col.boxplot.fill <- NA
-    col.mean.line <- adjustcolor(col = list(...)$col,
-                                 alpha.f = 0.4)
-    col.sd.bar <- adjustcolor(col = list(...)$col,
-                              alpha.f = 0.4)
-    col.background <- NA
   } else {
-
     .set_colour_value <- function(layout_value) {
       if (length(layout_value) == 1)
         c(layout_value, 2:length(data))
       else
         layout_value
     }
-    col.main <- .set_colour_value(layout$kde$colour$main)
-    col.xlab <- .set_colour_value(layout$kde$colour$xlab)
-    col.ylab1 <- .set_colour_value(layout$kde$colour$ylab1)
-    col.ylab2 <- .set_colour_value(layout$kde$colour$ylab2)
-    col.xtck <- .set_colour_value(layout$kde$colour$xtck)
-    col.ytck1 <- .set_colour_value(layout$kde$colour$ytck1)
-    col.ytck2 <- .set_colour_value(layout$kde$colour$ytck2)
-    col.box <- .set_colour_value(layout$kde$colour$box)
-    col.mtext <- .set_colour_value(layout$kde$colour$mtext)
     col.stats <- .set_colour_value(layout$kde$colour$stats)
     col.kde.line <- .set_colour_value(layout$kde$colour$kde.line)
     col.kde.fill <- .set_colour_value(layout$kde$colour$kde.fill)
@@ -670,68 +466,34 @@ plot_KDE <- function(
     col.value.rug <- .set_colour_value(layout$kde$colour$value.rug)
     col.boxplot.line <- .set_colour_value(layout$kde$colour$boxplot.line)
     col.boxplot.fill <- .set_colour_value(layout$kde$colour$boxplot.fill)
-    col.mean.line <- .set_colour_value(layout$kde$colour$mean.point)
-    col.sd.bar <- .set_colour_value(layout$kde$colour$sd.line)
-    col.background <- .set_colour_value(layout$kde$colour$background)
-  }
-
-  if("lty" %in% names(list(...))) {
-    lty <- list(...)$lty
-  } else {
-    lty <- rep(1, length(data))
-  }
-
-  if("lwd" %in% names(list(...))) {
-    lwd <- list(...)$lwd
-  } else {
-    lwd <- rep(1, length(data))
-  }
-
-  if("cex" %in% names(list(...))) {
-    cex <- list(...)$cex
-  } else {
-    cex <- 1
-  }
-
-  if("fun" %in% names(list(...))) {
-    fun <- list(...)$fun # nocov
-  } else {
-    fun <- FALSE
   }
 
   ## convert keywords into summary placement coordinates
-  coords <- .get_keyword_coordinates(summary.pos, xlim.plot, ylim.plot)
+  coords <- .get_keyword_coordinates(summary.pos, xlim.plot, ylim.plot[1:2])
   summary.pos <- coords$pos
   summary.adj <- coords$adj
 
   ## plot data sets -----------------------------------------------------------
+  par.default <- .par_defaults()
+  on.exit(par(par.default), add = TRUE)
 
   ## setup plot area
-  if(length(summary) >= 1 & summary.pos[1] == "sub") {
-
+  toplines <- 1
+  if (length(summary) >= 1 && is.sub) {
     toplines <- length(data)
-  } else {
-
-    toplines <- 1
   }
 
   ## extract original plot parameters
-  par(bg = layout$kde$colour$background)
-  bg.original <- par()$bg
-
   par(mar = c(5, 5.5, 2.5 + toplines, 4.5),
+      bg = layout$kde$colour$background,
       xpd = FALSE,
       cex = cex)
 
-  if(layout$kde$dimension$figure.width != "auto" |
-     layout$kde$dimension$figure.height != "auto") {
-    par(mai = layout$kde$dimension$margin / 25.4,
-        pin = c(layout$kde$dimension$figure.width / 25.4 -
-                  layout$kde$dimension$margin[2] / 25.4 -
-                  layout$kde$dimension$margin[4] / 25.4,
-                layout$kde$dimension$figure.height / 25.4 -
-                  layout$kde$dimension$margin[1] / 25.4 -
-                  layout$kde$dimension$margin[3]/25.4))
+  dim <- layout$kde$dimension
+  if (dim$figure.width != "auto" || dim$figure.height != "auto") {
+    par(mai = dim$margin / 25.4,
+        pin = c(dim$figure.width - dim$margin[2] - dim$margin[4],
+                dim$figure.height - dim$margin[1] - dim$margin[3]) / 25.4)
   }
 
   ## create empty plot to get plot dimensions
@@ -747,8 +509,7 @@ plot_KDE <- function(
   l_height <- par()$cxy[2]
 
   ## optionally update ylim
-  if(boxplot == TRUE) {
-
+  if (boxplot && values.cumulative) {
     ylim.plot[1] <- ylim.plot[1] - 1.4 * l_height
   }
 
@@ -834,19 +595,16 @@ plot_KDE <- function(
   }
 
   ## add plot title
-  cex.old <- par()$cex
-  par(cex = layout$kde$font.size$main / 12)
   title(main = main,
         family = layout$kde$font.type$main,
         font = (1:4)[c("plain", "bold", "italic", "bold italic") ==
                        layout$kde$font.deco$main],
         col.main = layout$kde$colour$main,
+        cex = layout$kde$font.size$main / 12,
         line = (toplines + 1.2) * layout$kde$dimension$main / 100)
-  par(cex = cex.old)
 
   ## optionally add mtext line
   if(mtext != "") {
-
     mtext(text = mtext,
           side = 3,
           line = 0.5,
@@ -854,34 +612,28 @@ plot_KDE <- function(
           font = (1:4)[c("plain", "bold", "italic", "bold italic") ==
                          layout$kde$font.deco$mtext],
           col.main = layout$kde$colour$mtext,
-          cex = layout$kde$font.size$mtext / 12)
+          cex = cex * layout$kde$font.size$mtext / 12)
   }
 
   ## add summary content
   for(i in 1:length(data)) {
-
-    if(summary.pos[1] != "sub") {
-
+    if (!is.sub) {
       text(x = summary.pos[1],
            y = summary.pos[2],
            adj = summary.adj,
            labels = label.text[[i]],
            col = col.stats[i],
            cex = layout$kde$font.size$stats / 12)
-    } else {
-
-      if(mtext == "") {
-
+    } else if (mtext == "") {
         mtext(side = 3,
               line = (toplines + 0.3 - i) * layout$kde$dimension$stats.line / 100,
               text = label.text[[i]],
               col = col.stats[i],
-              cex = layout$kde$font.size$stats / 12)
-      }
+              cex = cex * layout$kde$font.size$stats / 12)
     }
   }
 
-  if(values.cumulative == TRUE) {
+  if (values.cumulative) {
 
     ## create empty overlay plot
     par(new = TRUE) # adjust plot options
@@ -899,8 +651,7 @@ plot_KDE <- function(
     l_height <- par()$cxy[2]
 
     ## optionally update ylim
-    if(boxplot == TRUE) {
-
+    if (boxplot) {
       ylim.plot[3] <- ylim.plot[3] - 1.4 * l_height
     }
 
@@ -916,27 +667,17 @@ plot_KDE <- function(
          axes = FALSE)
 
     ## optionally add boxplot
-    if(boxplot == TRUE) {
-
-      ## add zero line
-      abline(h = 0)
-
+    if (boxplot) {
       ## get extended boxplot data
-      boxplot.data <- list(NA)
-
+      boxplot.data <- NULL
       for(i in 1:length(data)) {
         boxplot.i <- graphics::boxplot(x = data[[i]][,1],
                                        plot = FALSE)
-        boxplot.i$group <- mean(x = data[[i]][,1],
-                                                   na.rm = TRUE)
-        boxplot.i$names <- sd(x = data[[i]][,1],
-                                                   na.rm = TRUE)
-        boxplot.data[[length(boxplot.data) + 1]] <- boxplot.i
+        boxplot.i$group <- mean(data[[i]][, 1], na.rm = TRUE)
+        boxplot.i$names <- sd(data[[i]][, 1], na.rm = TRUE)
+        boxplot.data[[i]] <- boxplot.i
       }
 
-      ## remove dummy list object
-      boxplot.data[[1]] <- NULL
-
       ## get new line hights
       l_height <- par()$cxy[2]
 
@@ -958,8 +699,7 @@ plot_KDE <- function(
         ## draw median line
         lines(x = c(boxplot.data[[i]]$stats[3,1],
                     boxplot.data[[i]]$stats[3,1]),
-              y = c(-11/8 * l_height,
-                    -7/8 * l_height),
+              y = c(-11/8, -7/8) * l_height,
               lwd = 2,
               col = col.boxplot.line[i])
 
@@ -968,36 +708,29 @@ plot_KDE <- function(
                       boxplot.data[[i]]$stats[2,1],
                       boxplot.data[[i]]$stats[4,1],
                       boxplot.data[[i]]$stats[4,1]),
-                y = c(-11/8 * l_height,
-                      -7/8 * l_height,
-                      -7/8 * l_height,
-                      -11/8 * l_height),
+                y = c(-11/8, -7/8, -7/8, -11/8) * l_height,
                 col = col.boxplot.fill[i],
                 border = col.boxplot.line[i])
 
         ## draw whiskers
         lines(x = c(boxplot.data[[i]]$stats[2,1],
                     boxplot.data[[i]]$stats[1,1]),
-              y = c(-9/8 * l_height,
-                    -9/8 * l_height),
+              y = c(-9/8, -9/8) * l_height,
               col = col.boxplot.line[i])
 
         lines(x = c(boxplot.data[[i]]$stats[1,1],
                     boxplot.data[[i]]$stats[1,1]),
-              y = c(-10/8 * l_height,
-                    -8/8 * l_height),
+              y = c(-10/8, -8/8) * l_height,
               col = col.boxplot.line[i])
 
         lines(x = c(boxplot.data[[i]]$stats[4,1],
                     boxplot.data[[i]]$stats[5,1]),
-              y = c(-9/8 * l_height,
-                    -9/8 * l_height),
+              y = c(-9/8, -9/8) * l_height,
               col = col.boxplot.line[i])
 
         lines(x = c(boxplot.data[[i]]$stats[5,1],
                     boxplot.data[[i]]$stats[5,1]),
-              y = c(-10/8 * l_height,
-                    -8/8 * l_height),
+              y = c(-10/8, -8/8) * l_height,
               col = col.boxplot.line[i])
 
         ## draw outliers
@@ -1005,22 +738,22 @@ plot_KDE <- function(
                y = rep(-9/8 * l_height,
                        length(boxplot.data[[i]]$out)),
                col = col.boxplot.line[i],
-               cex = cex * 0.8)
+               cex = 0.8)
       }
     }
 
     ## optionally add rug
-    if(rug == TRUE) {
-
+    if (rug) {
       for(i in 1:length(data)) {
-
         for(j in 1:nrow(data[[i]])) {
-
-          lines(x = c(data[[i]][j,1],
-                      data[[i]][j,1]),
-                y = c(0,
-                      -2/8 * l_height),
-                col = col.value.rug[i])
+          suppressWarnings( # some values will be clipped
+          rug(data[[i]][j, 1],
+              ticksize = 0.01 * l_height,
+              side = 3,
+              pos = 0,
+              lwd = 1,
+              col = col.value.rug[i])
+          )
         }
       }
     }
@@ -1063,6 +796,7 @@ plot_KDE <- function(
     ## add De error bars
     for(i in 1:length(data)) {
       if (attr(data[[i]], "De.errors.available")) {
+        suppressWarnings( # zero-length arrow is of indeterminate angle and so skipped
         graphics::arrows(data[[i]][, 1] - data[[i]][, 2],
                1:length(data[[i]][,1]),
                data[[i]][, 1] + data[[i]][, 2],
@@ -1071,6 +805,7 @@ plot_KDE <- function(
                angle = 90,
                length = 0.05,
                col = col.value.bar[i])
+        )
       }
       ## add De measurements
       points(data[[i]][,1], 1:De.stats[i,1],
@@ -1079,25 +814,10 @@ plot_KDE <- function(
     }
   }
 
-  ## add empty plot
-  par(new = TRUE)
-  plot(NA,
-       ann = FALSE,
-       axes = FALSE,
-       xlim     = xlim.plot,
-       ylim     = ylim.plot[1:2],
-       log      = log.option,
-       cex      = cex,
-       cex.lab  = cex,
-       cex.main = cex,
-       cex.axis = cex)
-
   ## FUN by R Luminescence Team
-  if (fun == TRUE) sTeve() # nocov
+  if (fun) sTeve() # nocov
 
-  if(output == TRUE) {
-    return(invisible(list(De.stats = De.stats,
-                          summary.pos = summary.pos,
-                          De.density = De.density)))
-  }
+  invisible(list(De.stats = De.stats,
+                 summary.pos = summary.pos,
+                 De.density = De.density))
 }
