@@ -1,13 +1,12 @@
-#'@title Create Regions of Interest (ROI) Graphic
+#' @title Create Regions of Interest (ROI) Graphic
 #'
-#'@description Create ROI graphic with data extracted from the data imported
-#'via [read_RF2R]. This function is used internally by [analyse_IRSAR.RF] but
-#'might be of use to work with reduced data from spatially resolved measurements.
-#'The plot dimensions mimic the original image dimensions
+#' @description The function creates ROI graphic with data extracted from the
+#' data imported via [read_RF2R]. This function might be of use to work with
+#' reduced data from spatially resolved measurements.
+#' The plot dimensions mimic the original image dimensions.
 #'
-#'@param object [RLum.Analysis-class], [RLum.Results-class] or a [list] of such objects (**required**):
-#'data input. Please note that to avoid function errors, only input created
-#'by the functions [read_RF2R] or [extract_ROI] is accepted
+#' @param object [RLum.Analysis-class], [RLum.Results-class] or a [list] of such objects (**required**):
+#' input data created either by [read_RF2R] or [extract_ROI].
 #'
 #'@param exclude_ROI [numeric] (*with default*): option to remove particular ROIs from the
 #'analysis. Those ROIs are plotted but not coloured and not taken into account
@@ -22,11 +21,10 @@
 #'@param dim.CCD [numeric] (*optional*): metric x and y for the recorded (chip)
 #'surface in µm. For instance `c(8192,8192)`, if set additional x and y-axes are shown
 #'
-#'@param bg_image [RLum.Data.Image-class] (*optional*): background image object
-#'please note that the dimensions are not checked.
+#' @param bg_image [RLum.Data.Image-class] (*optional*): background image object
+#' (please note that dimensions are not checked).
 #'
-#'@param plot [logical] (*with default*): enable or disable plot output to use
-#'the function only to extract the ROI data
+#' @param plot [logical] (*with default*): enable/disable the plot output.
 #'
 #'@param ... further parameters to manipulate the plot. On top of all arguments of
 #'[graphics::plot.default] the following arguments are supported: `lwd.ROI`, `lty.ROI`,
@@ -41,7 +39,7 @@
 #'
 #'@author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
-#'@seealso [read_RF2R], [analyse_IRSAR.RF]
+#' @seealso [read_RF2R], [extract_ROI]
 #'
 #'@keywords datagen plot
 #'
@@ -58,66 +56,67 @@
 #'t <- extract_ROI(object = m, roi = roi)
 #'plot_ROI(t, bg_image = m)
 #'
-#'@md
 #'@export
 plot_ROI <- function(
   object,
-  exclude_ROI = c(1),
+  exclude_ROI = 1,
   dist_thre = -Inf,
   dim.CCD = NULL,
   bg_image = NULL,
   plot = TRUE,
-  ...) {
+  ...
+) {
+  .set_function_name("plot_ROI")
+  on.exit(.unset_function_name(), add = TRUE)
+
   ##helper function to extract content
   .spatial_data <- function(x) {
+    .validate_class(x, c("RLum.Analysis", "RLum.Results"),
+                    extra = "a 'list' of such objects",
+                    name = "'object'")
+
     ##ignore all none RLum.Analysis
-    if (!inherits(x, "RLum.Analysis") || x@originator != "read_RF2R")
-      stop("[plot_ROI()] Input for 'object' not supported, please check documentation!", call. = FALSE)
+    if (!inherits(x, "RLum.Analysis") || !.check_originator(x, "read_RF2R"))
+      .throw_error("Object originator '", x@originator, "' not supported")
 
     ##extract some of the elements
     info <- x@info
     info$ROI <- strsplit(split = " ", info$ROI, fixed = TRUE)[[1]][2]
-
-    c(ROI = info$ROI,
-      x = info$x,
-      y = info$y,
-      area = info$area,
-      width = info$width,
-      height = info$height,
-      img_width = info$image_width,
-      img_height = info$image_height,
-      grain_d = info$grain_d)
-
+    info <- info[c("ROI", "x", "y", "area", "width", "height",
+                    "image_width", "image_height", "grain_d")]
+    names(info) <- gsub("image_", "img_", names(info))
+    unlist(lapply(info, as.numeric))
   }
 
-  if(is(object, "RLum.Results") && object@originator == "extract_ROI") {
+  if (inherits(object, "RLum.Results") &&
+      .check_originator(object, "extract_ROI")) {
     m <- object@data$roi_coord
 
   } else {
     ## make sure the object is a list
     if(!is.list(object)) object <- list(object)
+    .validate_not_empty(object, "list")
 
     ##extract values and convert to numeric matrix
-    m <- t(vapply(object, .spatial_data, character(length = 9)))
-
-    ##make numeric
-    storage.mode(m) <- "numeric"
+    m <- t(vapply(object, .spatial_data, numeric(length = 9)))
 
     ## correct coordinates (they come in odd from the file)
     m[,"x"] <- m[,"x"] + m[,"width"] / 2
     m[,"y"] <- m[,"y"] + m[,"height"] / 2
   }
 
+  .validate_positive_scalar(exclude_ROI, int = TRUE, null.ok = TRUE)
+
   ##make sure the ROI selection works
-  if(is.null(exclude_ROI[1]) || exclude_ROI[1] <= 0)
+  if (is.null(exclude_ROI))
     exclude_ROI <- nrow(m) + 1
 
-  ## add mid_x and mid_y
-  m <- cbind(m, mid_x = c(m[,"x"] + m[,"width"] / 2), mid_y =  c(m[,"y"] + m[,"height"] / 2))
-  rownames(m) <- m[,"ROI"]
-
   ## distance calculation
-  euc_dist <- sel_euc_dist <- stats::dist(m[-exclude_ROI,c("mid_x","mid_y")])
+  euc_dist <- sel_euc_dist <- stats::dist(m[-exclude_ROI])
+
+  ## add half_width and half_height
+  m <- cbind(m, half_width = m[, "width"] / 2, half_height = m[, "height"] / 2)
+  rownames(m) <- m[, "ROI"]
 
   ## distance threshold selector
   sel_euc_dist[sel_euc_dist < dist_thre[1]] <- NA
@@ -148,12 +147,12 @@ plot_ROI <- function(
       legend.pos = "topright"
     ), val = list(...))
 
-
     ## set plot area
     do.call(
-      what = plot.default,
+      what = graphics::plot.default,
       args = c(x = NA, y = NA,
-               plot_settings[names(plot_settings) %in% methods::formalArgs(plot.default)])
+               plot_settings[names(plot_settings) %in%
+                             methods::formalArgs(graphics::plot.default)])
     )
 
     ## add background image if available
@@ -162,7 +161,8 @@ plot_ROI <- function(
         as(bg_image, "RLum.Data.Image")
       }, silent = TRUE)
       if(inherits(a, "try-error")) {
-        warning("[plot_ROI()] 'bg_image' is not of type RLum.Data.Image and cannot be converted into such; background image plot skipped!")
+        .throw_warning("'bg_image' is not of class 'RLum.Data.Image' and ",
+                       "cannot be converted into it, background image plot skipped")
       } else {
         a <- a@data
         graphics::image(
@@ -175,7 +175,8 @@ plot_ROI <- function(
       }
    }
 
-    if (plot_settings$grid) grid(nx = max(m[,"img_width"]), ny = max(m[,"img_height"]))
+    if (plot_settings$grid) graphics::grid(nx = max(m[, "img_width"]),
+                                           ny = max(m[, "img_height"]))
 
     ## plot metric scale
     if (!is.null(dim.CCD)) {
@@ -203,22 +204,25 @@ plot_ROI <- function(
     for (i in 1:nrow(m)) {
       if (!i%in%exclude_ROI) {
         ## mark selected pixels
+        x0 <- m[i, "x"] - m[i, "half_width"]
+        x1 <- m[i, "x"] + m[i, "half_width"]
+        y0 <- m[i, "y"] - m[i, "half_height"]
+        y1 <- m[i, "y"] + m[i, "half_height"]
         polygon(
-          x = c(m[i, "x"] - m[i, "width"]/ 2, m[i, "x"] - m[i, "width"]/ 2, m[i, "x"] + m[i, "width"]/2, m[i, "x"] + m[i, "width"]/2),
-          y = c(m[i, "y"] - m[i, "height"]/ 2, m[i, "y"] + m[i, "height"]/ 2, m[i, "y"] + m[i, "height"]/ 2, m[i, "y"] - m[i, "height"]/ 2),
+          x = c(x0, x0, x1, x1),
+          y = c(y0, y1, y1, y0),
           col = plot_settings$col.pixel
         )
       }
 
       ## add ROIs
       shape::plotellipse(
-        rx = m[i, "width"] / 2,
-        ry = m[i, "width"] / 2,
+        rx = m[i, "half_width"],
+        ry = m[i, "half_width"],
         mid = c(m[i, "x"], m[i, "y"]),
         lcol = plot_settings$col.ROI,
         lty = plot_settings$lty.ROI,
         lwd = plot_settings$lwd.ROI)
-
     }
 
     ## add distance marker
@@ -252,7 +256,6 @@ plot_ROI <- function(
         legend = plot_settings$legend.text,
         col = c(plot_settings$col.ROI, plot_settings$col.pixel, "red")
       )
-
     }
 
   }##end if plot
@@ -266,5 +269,4 @@ plot_ROI <- function(
     info = list(
       call = sys.call()
     )))
-
 }
