@@ -13,7 +13,7 @@
 #'
 #' @param file [character] or [list] (**required**): path and file name of the
 #' BIN/BINX file (URLs are supported). If input is a `list` it should comprise
-#' only `character`s representing each valid path and BIN/BINX-file names.
+#' only `character`s representing valid path and BIN/BINX-file names.
 #' Alternatively, the input character can be just a directory (path), in which
 #' case the function tries to detect and import all BIN/BINX files found in
 #' the directory.
@@ -70,9 +70,8 @@
 #' for import.
 #'
 #' @param pattern [character] (*optional*):
-#' argument that is used if only a path is provided. The argument will than be
-#' passed to the function [list.files] used internally to construct a `list`
-#' of wanted files
+#' regular expression pattern passed to [list.files] to construct a list of
+#' files to read (used only when a path is provided).
 #'
 #' @param verbose [logical] (*with default*):
 #' enable/disable output to the terminal.
@@ -123,7 +122,6 @@
 #'temp <- read_BIN2R(file)
 #'temp
 #'
-#' @md
 #' @export
 read_BIN2R <- function(
   file,
@@ -158,8 +156,8 @@ read_BIN2R <- function(
       ##If this is not really a path we skip this here
       if (all(dir.exists(file)) & length(dir(file)) > 0) {
         if (verbose)
-          message("[read_BIN2R()] Directory detected, trying to extract ",
-                  "'*.bin'/'*.binx' files ...\n")
+          .throw_message("Directory detected, trying to extract ",
+                         "'*.bin'/'*.binx' files ...\n", error = FALSE)
 
         ##get files
         file <- as.list(list.files(
@@ -293,8 +291,8 @@ read_BIN2R <- function(
     if(!is.null(forced.VersionNumber)){
       temp.VERSION <- as.raw(forced.VersionNumber)
       if (verbose)
-        message("[read_BIN2R()] 'forced.VersionNumber' set to ", temp.VERSION,
-                ", but this version may not match your input file")
+        .throw_message("'forced.VersionNumber' set to ", temp.VERSION,
+                       ", but this version may not match your input file", error = FALSE)
     }
 
     ##stop input if wrong VERSION
@@ -332,9 +330,11 @@ read_BIN2R <- function(
     if (num.toread > 0) {
       seek.connection(con, num.toread, origin = "current")
     } else {
-      if (verbose)
-        message("\n[read_BIN2R()] Record #", temp.ID + 1,
-                " skipped due to wrong record length")
+      if (verbose) {
+        message("") # add a newline
+        .throw_message("Record #", temp.ID + 1,
+                       " skipped due to wrong record length", error = FALSE)
+      }
       next()
     }
     temp.ID <- temp.ID + 1
@@ -404,7 +404,7 @@ read_BIN2R <- function(
         strlen <- force.size
       return(suppressWarnings(readChar(raw[-1], strlen, useBytes = TRUE)))
     }
-    return("")
+    return("") # nocov
   }
 
   ##PRESET VALUES
@@ -547,13 +547,11 @@ read_BIN2R <- function(
     }
 
     ##print record ID for debugging purposes
-    if(verbose){
-      if(show.record.number == TRUE){
+    if (show.record.number && verbose) {
         cat(temp.ID,",", sep = "")
         if(temp.ID%%10==0){
           cat("\n")
         }
-      }
     }
 
     #empty byte position
@@ -596,11 +594,13 @@ read_BIN2R <- function(
         ## we can check for a specific value for temp.RECTYPE
         if(inherits(ignore.RECTYPE[1], "numeric") && temp.RECTYPE == ignore.RECTYPE[1]) {
           seek.connection(con, temp.LENGTH - 15, origin = "current")
-            if(verbose)
-              message("\n[read_BIN2R()] Record #", temp.ID + 1,
-                      " skipped due to ignore.RECTYPE setting")
-            next()
+          if(verbose) {
+            message("") # add a newline
+            .throw_message("Record #", temp.ID + 1,
+                           " skipped due to ignore.RECTYPE setting", error = FALSE)
           }
+          next()
+        }
 
         if(temp.RECTYPE != 0 & temp.RECTYPE != 1 & temp.RECTYPE != 128) {
           ##jump to the next record by stepping the record length minus the already read bytes
@@ -612,8 +612,10 @@ read_BIN2R <- function(
           }
 
           ## skip to next record
-          if (verbose)
-            message("\n[read_BIN2R()] ", msg, ", record skipped")
+          if (verbose) {
+            message("") # add a newline
+            .throw_message(msg, ", record skipped", error = FALSE)
+          }
           temp.ID <- temp.ID + 1
           next()
         }
@@ -1097,13 +1099,13 @@ read_BIN2R <- function(
     ##check whether the position is valid at all
     if (results.METADATA[, all(position %in% POSITION)]) {
       keep.positions <- results.METADATA[, POSITION %in% position]
-      results.METADATA <- results.METADATA[keep.positions == TRUE, ]
+      results.METADATA <- results.METADATA[(keep.positions), ]
       results.DATA <- results.DATA[keep.positions]
       results.RESERVED <- results.RESERVED[keep.positions]
 
       if (verbose) {
-        message("[read_BIN2R()] Kept records matching 'position': ",
-                .collapse(position, quote = FALSE))
+        .throw_message("Kept records matching 'position': ",
+                       .collapse(position, quote = FALSE), error = FALSE)
       }
     }else{
       .throw_warning("At least one position number is not valid, ",
@@ -1131,7 +1133,7 @@ read_BIN2R <- function(
   ## if nothing is left, return an empty object
   if (nrow(results.METADATA) == 0) {
     if (verbose)
-      message("[read_BIN2R()] Empty object returned")
+      .throw_message("Empty object returned", error = FALSE)
     return(set_Risoe.BINfileData())
   }
 
@@ -1157,8 +1159,8 @@ read_BIN2R <- function(
 
         ##message
         if(verbose) {
-          message("[read_BIN2R()] Duplicated records detected and removed: ",
-                  .collapse(duplication.check, quote = FALSE))
+          .throw_message("Duplicated records detected and removed: ",
+                         .collapse(duplication.check, quote = FALSE), error = FALSE)
         }
 
       } else{
@@ -1174,7 +1176,7 @@ read_BIN2R <- function(
   if (results.METADATA[, .N != n.length || max(ID) > n.length]) {
     results.METADATA[, ID := 1:.N]
     if (verbose)
-      message("[read_BIN2R()] The record index has been recalculated")
+      .throw_message("The record index has been recalculated", error = FALSE)
   }
 
   # Convert Translation Matrix Values ---------------------------------------
@@ -1205,7 +1207,7 @@ read_BIN2R <- function(
 
   ## check for empty BIN-files names ... if so, set the name of the file as BIN-file name
   ## This can happen if the user uses different equipment
-  if (results.METADATA[, all(is.na(FNAME))]) {
+  if (results.METADATA[, !any(nzchar(FNAME))]) {
     results.METADATA[, FNAME := tools::file_path_sans_ext(basename(file))]
   }
 
@@ -1220,7 +1222,7 @@ read_BIN2R <- function(
   ## set fastForward to TRUE if arguments to Risoe.BINfileData2RLum.Analysis
   ## were specified
   if (!fastForward) {
-    dots <- names(list(...))
+    dots <- ...names()
     args <- dots[dots %in% names(formals(Risoe.BINfileData2RLum.Analysis))[-1]]
     if (length(args) > 0) {
       fastForward <- TRUE
