@@ -1,10 +1,9 @@
 #' @title Import Princeton Instruments (TM) SPE-file into R
 #'
-#' @description Function imports Princeton Instruments (TM) SPE-files into R environment and
-#' provides [RLum.Data.Image-class] objects as output.
-#'
-#' @details Function provides an R only import routine for the Princeton Instruments
-#' SPE format. Import functionality is based on the file format description provided by
+#' @description
+#' The function imports Princeton Instruments (TM) SPE-files into R and
+#' provides [RLum.Data.Image-class] objects as output. The import
+#' functionality is based on the file format description provided by
 #' Princeton Instruments and a MatLab script written by Carl Hall (see
 #' references).
 #'
@@ -12,14 +11,16 @@
 #' SPE-file name (including path), e.g.
 #' - `[WIN]`: `read_SPE2R("C:/Desktop/test.spe")`
 #' - `[MAC/LINUX]`: `read_SPE2R("/User/test/Desktop/test.spe")`.
-#' Additionally, it can be a URL starting with http:// or https://.
+#' Additionally, it can be a URL starting with `http://` or `https://`.
 #'
 #' @param output.object [character] (*with default*):
-#' set `RLum` output object.  Allowed types are `"RLum.Data.Spectrum"`,
-#' `"RLum.Data.Image"` or `"matrix"`
+#' set the output object type. Allowed types are `"RLum.Data.Spectrum"`,
+#' `"RLum.Data.Image"` or `"matrix"`.
 #'
-#' @param frame.range [vector] (*optional*):
-#' limit frame range, e.g. select first 100 frames by `frame.range = c(1,100)`
+#' @param frame.range [vector], [integer] (*optional*):
+#' range of frames to read. For example, `frame.range = c(1, 10)` selects only
+#' the first 10 frames. If not specifie, all available frames (up to a maximum
+#' of 100 if `output.object = "RLum.Data.Image"`) are read.
 #'
 #' @param txtProgressBar [logical] (*with default*):
 #' enable/disable the progress bar. Ignored if `verbose = FALSE`.
@@ -27,7 +28,7 @@
 #' @param verbose [logical] (*with default*): enable/disable output to the
 #' terminal.
 #'
-#' @param ... not used, for compatibility reasons only
+#' @param ... not used, for compatibility reasons only.
 #'
 #' @return
 #' Depending on the chosen option the functions returns three different
@@ -101,13 +102,11 @@
 #' #             file = "[your path and filename]",
 #' #             sep = ";", row.names = FALSE)
 #'
-#'
-#' @md
 #' @export
 read_SPE2R <- function(
   file,
   output.object = "RLum.Data.Image",
-  frame.range,
+  frame.range = NULL,
   txtProgressBar = TRUE,
   verbose = TRUE,
   ...
@@ -116,11 +115,13 @@ read_SPE2R <- function(
   on.exit(.unset_function_name(), add = TRUE)
 
   ## Integrity checks -------------------------------------------------------
-
   .validate_class(file, "character")
   .validate_length(file, 1)
-  valid.output.object <- c("RLum.Data.Image", "RLum.Data.Spectrum", "matrix")
-  .validate_args(output.object, valid.output.object)
+  .validate_args(output.object,
+                 c("RLum.Data.Image", "RLum.Data.Spectrum", "matrix"))
+  .validate_class(frame.range, c("integer", "numeric"), null.ok = TRUE)
+  if (anyNA(frame.range) || any(frame.range <= 0))
+    .throw_error("'frame.range' should contain positive values")
 
   ##check if file exists
   if(!file.exists(file)){
@@ -141,8 +142,8 @@ read_SPE2R <- function(
         download.file(file, destfile = file_link, quiet = !verbose, mode = "wb")
         file <- file_link
         failed <- FALSE
-      }else{
-        if (verbose) cat("FAILED\n")
+      } else if (verbose) {
+        cat("FAILED\n")
       }
     }
 
@@ -159,7 +160,6 @@ read_SPE2R <- function(
                    strsplit(file, split = "\\.")[[1]][2], sep = "")
   }}
 
-
   if (!verbose)
     txtProgressBar <- FALSE
 
@@ -201,7 +201,7 @@ read_SPE2R <- function(
   Date <- suppressWarnings(readChar(con, 10, useBytes=TRUE))
 
   ##jump
-  stepping <- readBin(con, what="raw", 4, size=1, endian="little", signed = TRUE)
+  readBin(con, what = "raw", 4, size = 1, endian = "little", signed = TRUE)
 
   #Old number of scans - should always be -1
   noscan <- readBin(con, what="int", 1, size=2, endian="little", signed = TRUE)
@@ -216,7 +216,7 @@ read_SPE2R <- function(
   xdim <- readBin(con, what="int", 1, size=2, endian="little", signed = FALSE)
 
   ##jump
-  stepping <- readBin(con, what="raw", 64, size=1, endian="little", signed = TRUE)
+  readBin(con, what = "raw", 64, size = 1, endian = "little", signed = TRUE)
 
   ##experiment data type
   ##0 = 32f (4 bytes)
@@ -226,7 +226,7 @@ read_SPE2R <- function(
   datatype <- readBin(con, what="int", 1, size=2, endian="little", signed = TRUE)
 
   ##jump
-  stepping <- readBin(con, what="raw", 546, size=1, endian="little")
+  readBin(con, what = "raw", 546, size = 1, endian = "little")
 
   #y dimension of raw data.
   ydim <- readBin(con, what="int", 1, size=2, endian="little", signed = FALSE)
@@ -235,7 +235,7 @@ read_SPE2R <- function(
   scramble <- readBin(con, what="int", 1, size=2, endian="little", signed = FALSE)
 
   ##jump
-  stepping <- readBin(con, what="raw", 4, size=1, endian="little")
+  readBin(con, what = "raw", 4, size = 1, endian = "little")
 
   #Number of scans (Early WinX)
   lnoscan <- readBin(con, what="int", 1, size=4, endian="little", signed = TRUE)
@@ -250,36 +250,41 @@ read_SPE2R <- function(
   TriggeredModeFlag <- readBin(con, what="int", 1, size=2, endian="little", signed = TRUE)
 
   ##jump
-  stepping <- readBin(con, what="raw", 768, size=1, endian="little")
+  readBin(con, what = "raw", 768, size = 1, endian = "little")
 
   ##number of frames in file.
   NumFrames <- readBin(con, what="int", 1, size=4, endian="little", signed = TRUE)
-
-  if(NumFrames > 100 & missing(frame.range) & output.object == "RLum.Data.Image"){
-    .throw_error("Import aborted: this file containes > 100 frames (",
-                 NumFrames, "). Use argument 'frame.range' to force import.")
+  if (NumFrames > 100 && is.null(frame.range) &&
+      output.object == "RLum.Data.Image") {
+    # nocov start
+    .throw_error("This file contains > 100 frames (", NumFrames, "), ",
+                 "use argument 'frame.range' to force import")
+    # nocov end
   }
 
-  ##set frame.range
-  if(missing(frame.range) == TRUE){frame.range <- c(1,NumFrames)}
+  ## set frame.range
+  frame.range <- if (is.null(frame.range))
+                   c(1, NumFrames)
+                 else
+                   pmin(pmax(range(frame.range), 1), NumFrames)
 
   ##jump
-  stepping <- readBin(con, what="raw", 542, size=1, endian="little")
+  readBin(con, what = "raw", 542, size = 1, endian = "little")
 
   #file_header_ver
   file_header_ver <- readBin(con, what="double", 1, size=4, endian="little")
 
   ##jump
-  stepping <- readBin(con, what="raw", 1000, size=1, endian="little")
+  readBin(con, what = "raw", 1000, size = 1, endian = "little")
 
   ##WinView_id - set to 19,088,743 (or 1234567 hex) (required for legacy reasons)
-  WinView_id <- readBin(con, what="integer", 1, size=4, endian="little", signed = TRUE)
+  readBin(con, what = "integer", 1, size = 4, endian = "little", signed = TRUE)
 
   ##jump
-  stepping <- readBin(con, what="raw", 1098, size=1, endian="little")
+  readBin(con, what = "raw", 1098, size = 1, endian = "little")
 
   ##lastvalue - set to 21,845 (or 5555 hex) (required for legacy reasons)
-  lastvalue <- readBin(con, what="integer", 1, size=2, endian="little", signed = TRUE)
+  readBin(con, what = "integer", 1, size = 2, endian = "little", signed = TRUE)
 
 
   ##end header
@@ -340,19 +345,12 @@ read_SPE2R <- function(
   ##stepping for frame range
   temp <- readBin(con, what = "raw", (min(frame.range)-1)*2, size = 1, endian = "little")
 
+  data.list <- NULL
   for(i in 1:(diff(frame.range)+1)){#NumFrames
     temp.data <- matrix(read.data(n.counts = (xdim * ydim)),
                         ncol = ydim,
                         nrow = xdim)
-
-    if(exists("data.list") == FALSE){
-
-      data.list <- list(temp.data)
-
-    }else{
-
-      data.list <- c(data.list, list(temp.data))
-    }
+    data.list <- c(data.list, list(temp.data))
 
     ##update progress bar
     if (txtProgressBar) {
@@ -368,7 +366,7 @@ read_SPE2R <- function(
 
   # Output ------------------------------------------------------------------
 
-  if(output.object == "RLum.Data.Spectrum" | output.object == "matrix"){
+  if (output.object %in% c("RLum.Data.Spectrum", "matrix")) {
     ##to create a spectrum object the matrix has to transposed and
     ##the row sums are needed
 
@@ -385,7 +383,6 @@ read_SPE2R <- function(
     colnames(data.spectrum.matrix) <- as.character(1:ncol(data.spectrum.matrix))
     rownames(data.spectrum.matrix) <- as.character(1:nrow(data.spectrum.matrix))
 
-
     ##set output object
     object <- set_RLum(
       class = "RLum.Data.Spectrum",
@@ -400,11 +397,10 @@ read_SPE2R <- function(
       object <- get_RLum(object)
     }
 
-
   }else if(output.object == "RLum.Data.Image"){
     object <- as(data.list, "RLum.Data.Image")
     object@originator <- "read_SPE2R"
-    object@recordType = "Image"
+    object@recordType <- "Image"
     object@curveType <- "measured"
     object@info <- temp.info
   }
