@@ -1,16 +1,17 @@
-#' Use DRAC to calculate dose rate data
-#'
-#' The function provides an interface from R to DRAC. An R-object or a
-#' pre-formatted XLS/XLSX file is passed to the DRAC website and the
-#' results are re-imported into R.
+#' @title Use DRAC to calculate dose rate data
 #'
+#' @description The function provides an interface from R to DRAC. An R-object or a
+#' CSV file is passed to the DRAC website and results are re-imported into R.
 #'
 #' @param file [character] (**required**):
-#' spreadsheet to be passed to the DRAC website for calculation. Can also be a
-#' DRAC template object obtained from `template_DRAC()`.
+#' name of a CSV file (formatted according to the DRAC v1.2 CSV template) to
+#' be sent to the DRAC website for calculation. It can also be a DRAC template
+#' object obtained from [template_DRAC()], which supports also import from
+#' CSV-files.
 #'
 #' @param name [character] (*with default*):
-#' Optional user name submitted to DRAC. If omitted, a random name will be generated
+#' Optional user name submitted to DRAC. If `NULL`, a random name will be
+#' generated.
 #'
 #' @param print_references (*with default*):
 #' Print all references used in the input data table to the console.
@@ -20,9 +21,13 @@
 #' used references. Valid options are `"Bibtex"`, `"citation"`, `"html"`, `"latex"`
 #' or `"R"`. Default is `"text"`.
 #'
-#' @param ... Further arguments.
+#' @param ... further arguments:
 #'
 #' - `url` [character]: provide an alternative URL to DRAC
+#' - `ignore_version` [logical]: ignores the version check, this might come in handy
+#' if the version has changed, but not the column order
+#' - `user` [character]: option to provide username for secured site
+#' - `password` [character]: password for secured site, only works jointly with `user`
 #' - `verbose` [logical]: show or hide console output
 #'
 #' @return Returns an [RLum.Results-class] object containing the following elements:
@@ -46,7 +51,9 @@
 #'
 #' The output should be accessed using the function [get_RLum].
 #'
-#' @section Function version: 0.14
+#' @seealso [template_DRAC], [.as.latex.table]
+#'
+#' @section Function version: 0.17
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)\cr
@@ -68,10 +75,7 @@
 #' use_DRAC(file = file)
 #' }
 #'
-#'
-#'
 #' ## (2) Method using an R template object
-#'
 #' # Create a template
 #' input <- template_DRAC(preset = "DRAC-example_quartz")
 #'
@@ -105,82 +109,95 @@
 #' # use DRAC
 #' \dontrun{
 #' output <- use_DRAC(input)
+#'
+#' ## export as LaTeX table
+#' .as.latex.table(output)
 #' }
 #'
-#' @md
 #' @export
 use_DRAC <- function(
   file,
-  name,
+  name = NULL,
   print_references = TRUE,
   citation_style = "text",
   ...
-){
+) {
+  .set_function_name("use_DRAC")
+  on.exit(.unset_function_name(), add = TRUE)
+
   ## TODO:
   ## (1) Keep the data set as unmodified as possible. Check structure and order of parameters
-  ## for meaningful cominbination.
+  ## for meaningful combination.
   ##
   ## (2)
   ## Leave it to the user where the calculations made in our package should be used
+  ##
+ # Settings ------------------------------------------------------------------------------------
+  settings <- modifyList(list(
+    name = name %||% paste(sample(c(LETTERS, letters), runif(1, 2, 4)),
+                           collapse = ""),
+    verbose = TRUE,
+    url = "https://www.aber.ac.uk/en/dges/research/quaternary/luminescence-research-laboratory/dose-rate-calculator/?show=calculator",
+    ignore_version = FALSE,
+    user = NULL,
+    password = NULL),
+    list(...),
+    keep.null = TRUE)
+
+  ## double check URL for validity
+  if(!grepl(pattern = "?show=calculator", settings$url, fixed = TRUE))
+     .throw_warning("URL is missing '?show=calculator', please check validity!")
+
+  ## Integrity checks -------------------------------------------------------
+  .validate_class(file, c("character", "DRAC.list", "DRAC.data.frame"))
+  .validate_not_empty(file)
 
-  # Integrity tests -----------------------------------------------------------------------------
   if (inherits(file, "character")) {
+    .validate_length(file, 1)
     if(!file.exists(file)){
-      stop("[use_DRAC()] It seems that the file doesn't exist!", call. = FALSE)
-
+      .throw_error("Input file does not exist")
     }
 
-    # Import data ---------------------------------------------------------------------------------
+    if (tools::file_ext(file) == "xls" || tools::file_ext(file) == "xlsx") {
+      .throw_error("XLS/XLSX format no longer supported, use CSV instead")
+    }
+
+    ## Import data ----------------------------------------------------------
 
     ## Import and skip the first rows and remove NA lines and the 2 row, as this row contains
     ## only meta data
 
-    ## DRAC v1.1 - XLS sheet
-    ##check if is the original DRAC table
-    if (tools::file_ext(file) == "xls" || tools::file_ext(file) == "xlsx") {
-      if (readxl::excel_sheets(file)[1] != "DRAC_1.1_input")
-        stop("[use_DRAC()] It looks like that you are not using the original DRAC v1.1 XLSX template. This is currently not supported!", call. = FALSE)
-
-      warning("\n[use_DRAC()] The current DRAC version is 1.2, but you provided the v1.1 excel input template.",
-              "\nPlease transfer your data to the new CSV template introduced with DRAC v1.2.", call. = FALSE)
-      input.raw <- na.omit(as.data.frame(readxl::read_excel(path = file, sheet = 1, skip = 5)))[-1, ]
-    }
-
     ## DRAC v1.2 - CSV sheet
-    if (tools::file_ext(file) == "csv") {
-      if (read.csv(file, nrows = 1, header = FALSE)[1] != "DRAC v.1.2 Inputs")
-        stop("[use_DRAC()] It looks like that you are not using the original DRAC v1.2 CSV template. This is currently not supported!", call. = FALSE)
+    if (read.csv(file, nrows = 1, header = FALSE)[1] != "DRAC v.1.2 Inputs" &
+        !settings$ignore_version)
+      .throw_error("It looks like that you are not using the original ",
+                   "DRAC v1.2 CSV template, this is currently not supported")
 
-      input.raw <- read.csv(file, skip = 8, check.names = FALSE, header = TRUE, stringsAsFactors = FALSE)[-1, ]
-    }
+    input.raw <- read.csv(file, skip = 8, check.names = FALSE, header = TRUE,
+                          stringsAsFactors = FALSE)[-1, ]
 
   } else if (inherits(file, "DRAC.list")) {
     input.raw <- as.data.frame(file)
 
   } else if (inherits(file, "DRAC.data.frame")) {
     input.raw <- file
-
-  } else {
-    stop("The provided data object is not a valid DRAC template.", call. = FALSE)
   }
 
+  ## NOTE: if this limit is ever raised above 9999, the generation of
+  ## random identifiers below at (4) must be adjusted accordingly to
+  ## guarantee uniqueness of identifiers
   if (nrow(input.raw) > 5000)
-    stop("[use_DRAC()] The limit of allowed datasets is 5000!", call. = FALSE)
+    .throw_error("The limit of allowed datasets is 5000!")
 
-  # Settings ------------------------------------------------------------------------------------
-  settings <- list(
-    name = ifelse(missing(name),
-                  paste0(sample(if(runif(1,-10,10)>0){LETTERS}else{letters}, runif(1, 2, 4))), name),
-   verbose = TRUE,
-   url = "https://www.aber.ac.uk/en/dges/research/quaternary/luminescence-research-laboratory/dose-rate-calculator/?show=calculator")
+  .validate_class(name, "character", null.ok = TRUE)
+  .validate_logical_scalar(print_references)
+  citation_style <- .validate_args(citation_style,
+                                   c("text", "Bibtex", "citation", "html",
+                                     "latex", "R"))
 
-  # override defaults with args in ...
-  settings <- modifyList(settings, list(...))
 
-  # Set helper function -------------------------------------------------------------------------
-  ## The real data are transferred without any encryption, so we have to mask the original
-
-  ##(0) set masking function
+  ## Helper function --------------------------------------------------------
+  ## The data are transferred without encryption, so we mask the original
   .masking <- function(mean, sd, n) {
     temp <- rnorm(n = 30 * n, mean = mean, sd = sd)
     t(vapply(seq(1, length(temp), 30), function(x) {
@@ -197,7 +214,7 @@ use_DRAC <- function(
 
   ##(2) generate some meaningful random variables
   mask.df <- lapply(seq(1, nrow(input.raw), 3), function(x) {
-    if (mask.df[x,"TI:52"] != "X") {
+    if (!is.na(mask.df[x,"TI:52"]) && mask.df[x,"TI:52"] != "X") {
       ##replace some values - the De value
       mask.df[x:(x + 2), c("TI:52","TI:53")] <- .masking(
         mean = as.numeric(mask.df[x,"TI:52"]),
@@ -205,82 +222,96 @@ use_DRAC <- function(
         n = 3)
       return(mask.df)
     }
-
   })
 
-  ##(3) bin values
+  ##(3) bind actual and masked values
   DRAC_submission.df <- rbind(input.raw, mask.df[[1]])
+  num.rows.df <- nrow(DRAC_submission.df)
 
   ##(4) replace ID values
   DRAC_submission.df$`TI:1` <- paste0(
-    paste0(
-      paste0(
-        sample(if(runif(1,-10,10)>0) LETTERS else letters, runif(1, 2, 4))),
-      ifelse(runif(1,-10,10)>0, "-", "")),
-    gsub(" ", "0", prettyNum(seq(sample(1:50, 1, prob = 50:1/50, replace = FALSE),
-      by = 1, length.out = nrow(DRAC_submission.df)), width = 2)))
-
+      sample(c(LETTERS, letters), max(4, num.rows.df), replace = TRUE),
+      sample(c("", "-", "_"), num.rows.df, replace = TRUE),
+      sprintf("%04d", sample(0:9999, num.rows.df)))[1:num.rows.df]
 
   ##(5) store the real IDs in a separate object
   DRAC_results.id <-  DRAC_submission.df[1:nrow(input.raw), "TI:1"]
 
   ##(6) create DRAC submission string
-  DRAC_submission.df <- DRAC_submission.df[sample(x = 1:nrow(DRAC_submission.df), nrow(DRAC_submission.df),
-                                                  replace = FALSE), ]
-
-  ##convert all columns of the data.frame to class 'character'
-  for (i in 1:ncol(DRAC_submission.df))
-    DRAC_submission.df[ ,i] <- as.character(DRAC_submission.df[, i])
-
   if (settings$verbose) message("\t Creating submission string...")
-  ##get line by line and remove unwanted characters
-  DRAC_submission.string <- sapply(1:nrow(DRAC_submission.df), function(x) {
-    paste0(gsub(",", "", toString(DRAC_submission.df[x, ])), "\n")
-  })
+
+  ## shuffle the rows of the submission
+  DRAC_submission.df <- DRAC_submission.df[sample(num.rows.df), ]
+
+  ## convert all columns of the data.frame to character
+  DRAC_submission.df[] <- lapply(DRAC_submission.df, as.character)
 
   ##paste everything together to get the format we want
-  DRAC_input <- paste(DRAC_submission.string, collapse = "")
+  DRAC_input <- paste0(apply(DRAC_submission.df, 1, paste, collapse = " "),
+                       "\n", collapse = "")
 
   # Send data to DRAC ---------------------------------------------------------------------------
-  if (settings$verbose) message(paste("\t Establishing connection to", settings$url))
+  if (settings$verbose)
+    message("\t Establishing connection to ", settings$url)
+
+  ## create config list
+  conf_l <- list()
+  # nocov start
+  if(!is.null(settings$user) & !is.null(settings$password))
+    conf_l <- httr::authenticate(settings$user, settings$password, type = "basic")
+  # nocov end
 
   ## send data set to DRAC website and receive response
-  DRAC.response <- httr::POST(settings$url,
-                              body = list("drac_data[name]"  = settings$name,
-                                          "drac_data[table]" = DRAC_input))
+  DRAC.response <- try(httr::POST(
+    url = settings$url,
+    config = conf_l,
+    body = list("drac_data[name]"  = settings$name,
+                "drac_data[table]" = DRAC_input)),
+    silent = TRUE)
+
   ## check for correct response
-  if (DRAC.response$status_code != 200) {
-    stop(paste0("[use_DRAC()] transmission failed with HTTP status code: ",
-                DRAC.response$status_code))
-  } else {
-    if (settings$verbose) message("\t The request was successful, processing the reply...")
+  if (inherits(DRAC.response, "try-error")) {
+    .throw_error("Transmission failed with error: ",
+                 attr(DRAC.response, "condition")$message)
+  } else if (DRAC.response$status_code != 200) {
+    .throw_error("Transmission failed with HTTP status code: ",
+                 DRAC.response$status_code)
   }
 
+  if (settings$verbose)
+    message("\t The request was successful, processing the reply...")
+
   ## assign DRAC response data to variables
-  http.header <- DRAC.response$header
   DRAC.content <- httr::content(x = DRAC.response, as = "text")
 
   ## if the input was valid from a technical standpoint, but not with regard
   ## contents, we indeed get a valid response, but no DRAC output
   if (!grepl("DRAC Outputs", DRAC.content)) {
-    error_start <- max(gregexpr("drac_field_error", DRAC.content)[[1]])
-    error_end <- regexec('textarea name=', DRAC.content)[[1]]
-    error_msg <- substr(DRAC.content, error_start, error_end)
-
     on.exit({
-      reply <- readline("Do you want to see the DRAC error message (Y/N)?")
-      if (reply == "Y" || reply == "y" || reply == 1)
-        cat(error_msg)
-    })
+      reply <- readline("Do you want to see the DRAC error message (y/N)? ")
+      if (reply %in% c("Y", "y", "1")) {
+        ## convert to html and extract the error fields
+        html <- XML::htmlParse(DRAC.content, asText = TRUE)
+        node <- XML::getNodeSet(html, "//div[@class='drac_field_error']")
+        error <- sapply(node, function(x) {
+          line <- paste0(sapply(XML::xmlChildren(x), XML::saveXML), collapse = "")
+          trimws(gsub("\\s*<br/>", "\n", line))
+        })
 
-    stop(paste("\n\t We got a response from the server, but it\n",
-               "\t did not contain DRAC output. Please check\n",
-               "\t your data and verify its validity.\n"),
-         call. = FALSE)
-  } else {
-    if (settings$verbose) message("\t Finalising the results...")
+        ## extract the fragment we are given if no drac_field_error is found
+        if (length(error) == 0)
+          error <- XML::xpathSApply(html, "//body", XML::xmlValue) # nocov
+
+        cat("\n", paste(error, collapse = "\n"), "\n", sep = "")
+      }
+    }, add = TRUE)
+
+    .throw_error("The response from the server did not contain DRAC output, ",
+                 "please check your data and verify its validity.")
   }
 
+  if (settings$verbose) message("\t Finalising the results...")
+
   ## split header and content
   DRAC.content.split <- strsplit(x = DRAC.content,
                                  split = "DRAC Outputs\n\n")
@@ -296,19 +327,23 @@ use_DRAC <- function(
   ## remove first two lines
   DRAC.content <- data.table::fread(as.character(DRAC.content.split[[1]][2]),
                                     sep = ",", skip = 2,
-                                    stringsAsFactors = FALSE, colClasses = c(V3 = "character"),
+                                    stringsAsFactors = FALSE,
+                                    colClasses = c(V3 = "character"),
                                     data.table = FALSE)
 
   ##Get rid of all the value we do not need anymore
   DRAC.content <-  subset(DRAC.content, DRAC.content$V1 %in% DRAC_results.id)
-  DRAC.content <- DRAC.content[with(DRAC.content, order(V1)), ]
+  DRAC.content <- DRAC.content[match(DRAC_results.id, DRAC.content$V1),]
 
   ##replace by original names
-  DRAC.content[ ,1] <- input.raw[ ,1]
+  DRAC.content[[1]] <- input.raw[[1]]
 
   ## assign column names
   colnames(DRAC.content) <- DRAC.raw[1, ]
 
+  ## reset rownames
+  rownames(DRAC.content) <- seq_len(nrow(DRAC.content))
+
   ## save column labels and use them as attributes for the I/O table columns
   DRAC.labels <- DRAC.raw[2, ]
   for (i in 1:length(DRAC.content)) {
@@ -319,7 +354,7 @@ use_DRAC <- function(
   DRAC.content.input <- DRAC.content[ ,grep("TI:", names(DRAC.content))]
   DRAC.content.output <- DRAC.content[ ,grep("TO:", names(DRAC.content))]
 
-  ## The DRAC ouput also contains a hightlight table, which results in
+  ## The DRAC ouput also contains a highlight table, which results in
   ## duplicate columns. When creating the data.frame duplicate columns
   ## are automatically appended '.1' in their names, so we can identify
   ## and remove them easily
@@ -363,7 +398,7 @@ use_DRAC <- function(
   if (settings$verbose) lapply(messages, message)
 
   ## Get and print used references
-  references <- get_DRAC_references(DRAC.content.input)
+  references <- .get_DRAC_references(DRAC.content.input)
 
   if (print_references && settings$verbose) {
     for (i in 1:length(references$refs)) {
@@ -372,9 +407,8 @@ use_DRAC <- function(
     }
   }
 
-
   ## return output
-  DRAC.return <- set_RLum(
+  invisible(set_RLum(
     "RLum.Results",
     data = list(
     DRAC = list(highlights = DRAC.highlights,
@@ -387,6 +421,5 @@ use_DRAC <- function(
         data = file,
         call = sys.call(),
         args = as.list(sys.call()[-1])))
-
-  invisible(DRAC.return)
+    )
 }
