@@ -21,31 +21,30 @@
 #' @param path [character] (*optional*):
 #' character string naming folder for the output to be written. If nothing
 #' is provided `path` will be set to the working directory.
-#' **Note:** this argument is ignored if the the argument `export` is set to `FALSE`.
+#' **Note:** this argument is ignored when `export = FALSE`.
 #'
 #' @param prefix [character] (*with default*):
 #' optional prefix to name the files. This prefix is valid for all written files
 #'
 #' @param export [logical] (*with default*):
-#' enable or disable the file export. If set to `FALSE` nothing is written to
+#' enable/disable the file export. If set to `FALSE` nothing is written to
 #' the file connection, but a list comprising objects of type [data.frame] and [matrix]
-#' is returned instead
+#' is returned instead.
 #'
-#' @param compact [logical] (*with default*): if `TRUE` (the default) the output will be more
-#' simple but less comprehensive, means not all elements in the objects will be fully broken down.
-#' This is in particular useful for writing `RLum.Results` objects to CSV-files, such objects
-#' can be rather complex and not all information are needed in a CSV-file or can be meaningful translated
-#' to it.
+#' @param compact [logical] (*with default*): if `TRUE` (default) the output
+#' will be simpler but less comprehensive, that is not all elements in the
+#' objects will be fully broken down. This is in particular useful for writing
+#' `RLum.Results` objects to CSV files, as such objects can be rather complex
+#' and not all information are needed in a CSV file or can be meaningfully
+#' translated to CSV format.
 #'
 #' @param ... further arguments that will be passed to the function
 #' [utils::write.table]. All arguments except the argument `file` are supported
 #'
-#'
 #' @return
 #' The function returns either a CSV-file (or many of them) or for the
 #' option `export == FALSE` a list comprising objects of type [data.frame] and [matrix]
 #'
-#'
 #' @section Function version: 0.2.2
 #'
 #' @author
@@ -58,23 +57,20 @@
 #'
 #' @examples
 #'
-#' ##transform values to a list (and do not write)
+#' ## transform values to a list (and do not write)
 #' data(ExampleData.BINfileData, envir = environment())
 #' object <- Risoe.BINfileData2RLum.Analysis(CWOSL.SAR.Data)[[1]]
 #' write_RLum2CSV(object, export = FALSE)
 #'
 #' \dontrun{
-#'
-#' ##create temporary filepath
-#' ##(for usage replace by own path)
+#' ## write to a temporary file
 #' temp_file <- tempfile(pattern = "output", fileext = ".csv")
+#' write_RLum2CSV(object, temp_file)
 #'
-#' ##write CSV-file to working directory
-#' write_RLum2CSV(temp_file)
-#'
+#' ## write to the working directory
+#' write_RLum2CSV(object)
 #' }
 #'
-#' @md
 #' @export
 write_RLum2CSV <- function(
   object,
@@ -83,33 +79,32 @@ write_RLum2CSV <- function(
   export = TRUE,
   compact = TRUE,
   ...
+) {
+  .set_function_name("write_RLum2CSV")
+  on.exit(.unset_function_name(), add = TRUE)
 
-){
-  # General tests -------------------------------------------------------------------------------
-  if(missing(object)){
-    stop("[write_RLum2CSV()] input object is missing!", call. = FALSE)
-
+  ## support using `file` for the output path to help RLumShiny
+  if ("file" %in% ...names() && is.null(path)) {
+    path <- list(...)$file
   }
 
   # Self-call -----------------------------------------------------------------------------------
   ##this option allows to work on a list of RLum-objects
-  if(is.list(object) && !is.data.frame(object)){
-    ##extent the list of arguments if set
-      ##path
-      path <- rep(list(path), length = length(object))
+  if (inherits(object, "list")) {
+    ## expand input arguments
+    rep.length <- length(object)
+    path <- .listify(path, rep.length)
+    export <- .listify(export, rep.length)
 
       ##prefix ... create automatic prefix if nothing is provided
       prefix <- as.list(paste0(prefix[1], "[[",1:length(object),"]]_"))
 
-      ##export
-      export <- rep(list(export), length = length(object))
-
       ## write list name to object
-      for (i in 1:length(object))
+      for (i in seq_along(object))
         attr(object[[i]], "list_name") <- names(object)[i]
 
     ##execute the self-call function
-      temp <- lapply(1:length(object), function(x){
+      temp <- lapply(seq_along(object), function(x) {
         write_RLum2CSV(
           object = object[[x]],
           path = path[[x]],
@@ -117,77 +112,66 @@ write_RLum2CSV <- function(
           export = export[[x]],
           ...
         )
-
       })
 
-      ##this prevents that we get a list of NULL
-      if(is.null(unlist(temp))){
-        return(NULL)
-
-      }else{
-        return(temp)
-
-      }
-
+    ## this prevents that we get a list of NULL
+    if (is.null(unlist(temp)))
+      return(NULL)
+    return(temp)
   }
 
-  # Integrity tests -----------------------------------------------------------------------------
-  ##check path
-    ##if NULL condition
-    if(export == TRUE && is.null(path)){
+  ## Integrity checks -------------------------------------------------------
+  .validate_class(object, c("RLum", "data.frame"),
+                  extra = "a 'list' of such objects")
+  .validate_not_empty(object)
+  .validate_class(path, "character", null.ok = TRUE)
+  .validate_class(prefix, "character")
+  .validate_logical_scalar(export)
+  .validate_logical_scalar(compact)
+
+  ## check export path
+  if (export) {
+    if (is.null(path) || !nzchar(path)) {
       path <- getwd()
-      message(paste0("[write_RLum2CSV()] Path automatically set to: ", path))
-
-    }
-
-    ##non NULL conditon
-    if(export == TRUE && !dir.exists(path)){
-      stop("[write_RLum2CSV()] Diretory provided via the argument 'path' does not exist!", call. = FALSE)
-
+      .throw_message("Path automatically set to: ", path, error = FALSE)
+    } else if (!dir.exists(path)) {
+      .throw_error("Path '", path, "' does not exist")
     }
+  }
 
   ## What do we need at the end of the day is a named list of data.frames or matrices we can export
   ## using the function write.table; the name of the list elements will become the file names
-  if(inherits(object, "RLum")){
-    if(is(object, "RLum.Analysis") ||
-       is(object, "RLum.Data.Curve") ||
-       is(object, "RLum.Data.Spectrum") || is(object, "RLum.Data.Image")){
 
-      ##extract all elements ... depending on the input
-      if(is(object, "RLum.Analysis")){
-        ##tricky, we cannot use get_RLum() as the function lapply calls as.list() for an object!
-        object_list <- lapply(object, function(x){get_RLum(x)})
+  ## extract all elements ... depending on the input
+  if (inherits(object, "RLum.Analysis")) {
+        object_list <- lapply(object, get_RLum)
 
         ##change names of the list and produce the right format straight away
         names(object_list) <- paste0(1:length(object_list),"_",names(object))
 
-      } else {
-
+  } else if (inherits(object, "RLum.Data")) {
         ##get object and make list
         object_list <- list(get_RLum(object))
 
         ##set new name
         names(object_list) <- paste0("1_",object@recordType)
 
-      }
-
-    } else if (is(object, "RLum.Results")){
+  } else if (inherits(object, "RLum.Results")) {
       ##unlist what ever comes, but do not break structures like matrices, numerics and
       names <- names(object@data)
 
       ##get elements
       object_list <- lapply(object@data, function(e){
         ##only run something on the list of it is worth it and pack it in the list
-        if(inherits(e, "matrix") || inherits(e, "numeric") || inherits(e, "data.frame"))
+        if (inherits(e, c("matrix", "numeric", "data.frame")))
           return(list(e))
 
         ##unlist the rest until the end
         if(!compact)
-          return(unlist(e))
+          e <- unlist(e)
 
         ##now we return whatever we have
         return(e)
-
       })
 
       ##now unlist again one level
@@ -195,41 +179,35 @@ write_RLum2CSV <- function(
 
       ##sort out objects we do not like and we cannot procede ...
       object_list_rm <- vapply(object_list, function(x) {
-         inherits(x, "matrix") || inherits(x, "numeric") || inherits(x, "data.frame")
-
-      }, vector(mode = "logical", length = 1))
+         inherits(x, c("matrix", "numeric", "data.frame"))
+      }, FUN.VALUE = logical(1))
 
       ##remove unwanted objects
       object_list <- object_list[object_list_rm]
 
-
       ##set warning
-      if(any(!object_list_rm))
-        warning(paste0("[write_RLum2CSV()] ", length(which(!object_list_rm)), " elements could not be converted to a CSV-structure!"), call. = FALSE)
+      if (!all(object_list_rm))
+        .throw_warning(length(which(!object_list_rm)),
+                       " elements could not be converted to CSV")
+
+      if (length(object_list) == 0) {
+        .throw_error("No valid records in 'object'")
+      }
 
       ##adjust the names
       names(object_list) <- paste0(1:length(object_list),"_",names(object_list))
 
-
-    } else {
-      try(stop("[write_RLum2CSV()] One particular RLum-object is not yet supported! NULL returned!", call. = FALSE))
-      return(NULL)
-
-    }
-
   } else if (inherits(object, "data.frame")) {
     object_list <- list(object)
-    if(!is.null(attr(object, "filename"))) filename <- attr(object, "filename") else  filename <- ""
-
+    filename <- attr(object, "filename") %||% ""
     names(object_list) <- paste0("conv_", attr(object, "list_name"), filename)
-
-  }else{
-   stop("[write_RLum2CSV()] Object needs to be a member of the object class RLum!", call. = FALSE)
-
   }
 
   # Export --------------------------------------------------------------------------------------
-  if(export){
+  if (!export) {
+    return(object_list)
+  }
+
     ##set export settings for write.table
     export_settings.default <- list(
       append = FALSE,
@@ -242,7 +220,6 @@ write_RLum2CSV <- function(
       col.names = FALSE,
       qmethod = c("escape", "double"),
       fileEncoding = ""
-
     )
 
     ##modify on demand
@@ -263,12 +240,5 @@ write_RLum2CSV <- function(
         col.names =  export_settings$col.names,
         qmethod =  export_settings$qmethod,
         fileEncoding =  export_settings$fileEncoding)
-
     }
-
-  }else{
-    return(object_list)
-
-  }
-
 }
