@@ -8,7 +8,7 @@ test_that("input validation", {
   expect_error(analyse_ThermochronometryData(list()),
                "'object' should be of class 'character'")
   expect_error(analyse_ThermochronometryData("error"),
-               "File does not exist")
+               "File 'error' does not exist")
   expect_error(analyse_ThermochronometryData(input.csv[1], verbose = "error"),
                "'verbose' should be a single logical value")
   expect_error(analyse_ThermochronometryData(test_path("_data/CLBR.xlsx")),
@@ -25,8 +25,20 @@ test_that("check functionality", {
   expect_s4_class(analyse_ThermochronometryData(input.csv[1],
                                                 verbose = FALSE),
                   "RLum.Results")
+  SW({
   expect_s4_class(analyse_ThermochronometryData(input.csv[1], ITL_model = "BTS",
-                                                verbose = FALSE,
+                                                verbose = TRUE,
                                                 num_s_values_bts = 10),
                   "RLum.Results")
+  })
+})
+
+test_that("graphical snapshot tests", {
+  testthat::skip_on_cran()
+  testthat::skip_if_not_installed("vdiffr")
+
+  SW({
+  vdiffr::expect_doppelganger("default",
+                              analyse_ThermochronometryData(input.csv[1]))
+  })
 })
