@@ -19,7 +19,7 @@ test_that("input validation", {
   expect_error(analyse_baSAR(list(CWOSL.sub, "error"), verbose = FALSE),
                "'object' only accepts a list of objects of the same type")
   expect_error(analyse_baSAR(CWOSL.sub, n.MCMC = NULL),
-               "'n.MCMC' should be a positive integer scalar")
+               "'n.MCMC' should be a single positive integer value")
   expect_error(analyse_baSAR(CWOSL.sub, verbose = FALSE),
                "'source_doserate' is missing, but the current implementation")
   expect_error(analyse_baSAR(CWOSL.sub, fit.method = "error"),
@@ -46,7 +46,7 @@ test_that("input validation", {
                              signal.integral = c(1:2),
                              background.integral = c(80:100),
                              CSV_file = "error"),
-               "'CSV_file' does not exist")
+               "does not exist or is non-readable")
   data.table::fwrite(data.frame(BIN_file = "a", DISC = 1), file = csv.file)
   expect_error(analyse_baSAR(CWOSL.sub, verbose = FALSE,
                              source_doserate = c(0.04, 0.001),
@@ -95,6 +95,16 @@ test_that("input validation", {
   expect_error(suppressWarnings(
     analyse_baSAR(obj, recordType = "NONE", verbose = TRUE)),
     "No records of the appropriate type were found")
+
+  empty <- CWOSL.sub
+  empty@METADATA <- empty@METADATA[integer(0), ]
+  expect_error(expect_warning(
+      analyse_baSAR(list(empty),
+                    source_doserate = c(0.04, 0.001),
+                    signal.integral = c(1:2),
+                    background.integral = c(80:100)),
+      "No data selected from BIN-file 1, BIN-file removed from input"),
+      "All provided objects were removed")
   })
 
   expect_warning(expect_output(
@@ -134,7 +144,8 @@ test_that("input validation", {
                     signal.integral = c(1:2),
                     background.integral = c(80:100),
                     distribution = "user_defined")),
-      "Channel numbers of Lx and Tx data differ")
+      "Different number of channels for Lx (250) and Tx (1000)",
+      fixed = TRUE)
 
   SW({
   data(ExampleData.RLum.Analysis, envir = environment())
@@ -149,7 +160,7 @@ test_that("input validation", {
                     fit.method = "LIN", fit.force_through_origin = FALSE,
                     distribution = "normal",
                     n.MCMC = 75)),
-      "'1' is a duplicate and therefore removed from the input")
+      "'ExampleData.BINfileData' is a duplicate and therefore removed from the input")
 
   CWOSL.min <- subset(CWOSL.sub, subset = ID < 20)
   expect_warning(expect_error(
@@ -161,7 +172,37 @@ test_that("input validation", {
       "In input 1 the number of data points (19) is not a multiple of the",
       fixed = TRUE),
       "Only multiple grain data provided, automatic selection skipped")
+
+  CWOSL.sub2 <- CWOSL.sub
+  CWOSL.sub2@METADATA$FNAME <- "Other file"
+  expect_warning(analyse_baSAR(list(CWOSL.sub, CWOSL.sub2),
+                               source_doserate = list(c(0.04, 0.001), c(0.05, 0.02)),
+                               signal.integral = c(1:2),
+                               background.integral = c(5:15),
+                               method_control = list(n.chains = 1),
+                               n.MCMC = 10),
+                 "Provided source dose rate errors differ")
   })
+
+  CWOSL.mod <- CWOSL.sub
+  CWOSL.mod@METADATA$POSITION[c(1:nrow(CWOSL.mod@METADATA))] <- 2.5
+  expect_warning(expect_message(expect_null(
+      analyse_baSAR(CWOSL.mod, verbose = FALSE,
+                    source_doserate = c(0.04, 0.001),
+                    signal.integral = 1:2,
+                    background.integral = 80:100)),
+      "position number 2 does not exist, NULL returned"),
+      "Only multiple grain data provided, automatic selection skipped")
+
+  CWOSL.mod <- CWOSL.sub
+  CWOSL.mod@METADATA$GRAIN[c(1:nrow(CWOSL.mod@METADATA))] <- 2
+  expect_warning(expect_message(expect_null(
+      analyse_baSAR(CWOSL.mod, verbose = FALSE,
+                    source_doserate = c(0.04, 0.001),
+                    signal.integral = 1:2,
+                    background.integral = 80:100)),
+      "grain number 0 does not exist, NULL returned"),
+      "Only multiple grain data provided, automatic selection skipped")
 })
 
 test_that("Full check of analyse_baSAR function", {
@@ -222,16 +263,16 @@ test_that("Full check of analyse_baSAR function", {
       n.MCMC = 100),
      "Number of aliquots used:.+3/3")
 
-  expect_warning(analyse_baSAR(
+  expect_output(analyse_baSAR(
       object = results,
       plot = FALSE,
       verbose = FALSE,
       txtProgressBar = FALSE,
-      method_control = list(upper_centralD = 200),
-      n.MCMC = 100),
-      "You have modified the upper central_D boundary")
+      baSAR_model = "empty",
+      n.MCMC = 10),
+      "Error parsing model file")
 
-    expect_warning(analyse_baSAR(
+  expect_warning(analyse_baSAR(
       object = results,
       plot = FALSE,
       verbose = FALSE,
@@ -297,6 +338,19 @@ test_that("Full check of analyse_baSAR function", {
       fixed = TRUE),
       "'source_doserate' is ignored in this mode as it was already set")
 
+  ## more coverage
+  SW({
+  expect_warning(analyse_baSAR(
+      object = CWOSL.sub,
+      source_doserate = c(NaN, 0.001),
+      signal.integral = 1:2,
+      background.integral = 80:100,
+      plot = FALSE,
+      verbose = TRUE,
+      n.MCMC = 100),
+      "'Nb_aliquots' corrected due to NaN or Inf values")
+  })
+
   CWOSL.mod <- CWOSL.sub
   CWOSL.mod@METADATA$SEL <- FALSE
   expect_warning(expect_null(analyse_baSAR(CWOSL.mod, verbose = TRUE,
@@ -315,7 +369,7 @@ test_that("Full check of analyse_baSAR function", {
                                background.integral = c(80:100),
                                method_control = list(n.chains = 1),
                                n.MCMC = 100),
-                 "Record pre-selection in BIN-file detected")
+                 "Record pre-selection in BIN-file detected, record reduced to")
 
   CWOSL.mod <- CWOSL.sub
   CWOSL.mod@METADATA$GRAIN[-c(19:24)] <- 2
@@ -330,7 +384,8 @@ test_that("Full check of analyse_baSAR function", {
 
   results2@originator <- "unknown"
   expect_error(analyse_baSAR(object = results2),
-               "'object' is of type 'RLum.Results', but was not produced by")
+               "'object' has an unsupported originator (expected 'analyse_baSAR'",
+               fixed = TRUE)
 
   results2@originator <- "analyse_baSAR"
   results2@data$input_object <- results2$input_object[1:2, ]
@@ -345,8 +400,9 @@ test_that("Full check of analyse_baSAR function", {
                                n.MCMC = 10),
                  "Number of background channels for Tx < 25")
 
+  df <- CWOSL.sub@METADATA[, c("FNAME", "POSITION", "GRAIN")]
   analyse_baSAR(CWOSL.sub,
-                CSV_file = CWOSL.sub@METADATA[, c("FNAME", "POSITION", "GRAIN")],
+                CSV_file = df,
                 source_doserate = c(0.04, 0.001),
                 signal.integral = c(1:2),
                 background.integral = c(8:10),
@@ -354,6 +410,16 @@ test_that("Full check of analyse_baSAR function", {
                 aliquot_range = 1:2,
                 n.MCMC = 10)
 
+  df$FNAME[3] <- NA
+  analyse_baSAR(CWOSL.sub,
+                CSV_file = df,
+                source_doserate = c(0.04, 0.001),
+                signal.integral = 1:2,
+                background.integral = 8:40,
+                method_control = list(n.chains = 1),
+                aliquot_range = 1:4,
+                n.MCMC = 10)
+
   vnames <- c("central_D", "sigma_D", "")
   expect_message(analyse_baSAR(CWOSL.sub,
                                source_doserate = c(0.04, 0.001),
@@ -374,6 +440,31 @@ test_that("Full check of analyse_baSAR function", {
                                                      variable.names = vnames),
                                n.MCMC = 10, verbose = FALSE),
                  "Plots for 'central_D' and 'sigma_D' could not be produced")
+
+  ## user model just for coverage
+  expect_output(analyse_baSAR(CWOSL.sub, verbose = FALSE,
+                              source_doserate = c(0.04, 0.001),
+                              signal.integral = 1:2,
+                              background.integral = 80:100,
+                              distribution = "cauchy",
+                              baSAR_model =
+                                "model {
+                                  central_D ~ dunif(lower_centralD, upper_centralD)
+                                  sigma_D ~ dunif(0.01, 1 * central_D)
+                                  for (i in 1:Nb_aliquots) {
+                                    a[i] ~ dnorm(6.5 , 1/(9.2^2))
+                                    b[i] ~ dnorm(50 , 1/(1000^2))
+                                    c[i] ~ dnorm(1.002 , 1/(0.9^2))
+                                    g[i] ~ dnorm(0.5 , 1/(2.5^2))
+                                    sigma_f[i] ~ dexp(20)
+                                    D[i] ~ dnorm( central_D  + Dose[1, i], 1/(sigma_D^2))
+                                    S_y[1,i] <- 1 / (sLum[1,i]^2 + sigma_f[i]^2)
+                                    Lum[1,i] ~ dnorm( Q[1,i] , S_y[1,i])
+                                    Q[1,i] <- GC_Origin * LinGC * ExpoGC * Limited_cycles[1]
+                                  }
+                                }",
+                              method_control = list(n.chains = 1),
+                              n.MCMC = 10))
   })
 })
 
@@ -391,6 +482,20 @@ test_that("regression tests", {
                     n.MCMC = 60),
       "RLum.Results"),
       "'thin = 60' is too high for 'n.MCMC = 60', reset to 30")
+
+  ## issue 834
+  df <- CWOSL.sub@METADATA[, c("FNAME", "POSITION", "GRAIN")]
+  expect_warning(expect_s4_class(
+      analyse_baSAR(CWOSL.sub,
+                    CSV_file = df,
+                    source_doserate = c(0.04, 0.001),
+                    signal.integral = 1:2,
+                    background.integral = 8:40,
+                    method_control = list(n.chains = 1),
+                    aliquot_range = 1,
+                    n.MCMC = 10),
+      "RLum.Results"),
+      "Adaptation incomplete")
   })
 
   ## check parameters irradiation times
