@@ -2,29 +2,32 @@ test_that("input validation", {
   testthat::skip_on_cran()
 
   expect_error(calc_FadingCorr("error"),
-               "'age.faded' should be of class 'numeric' or 'integer'")
+               "'age.faded' should be of class 'numeric' or 'integer' and have length 2")
   expect_error(calc_FadingCorr(1),
-               "'age.faded' should have length 2")
+               "'age.faded' should be of class 'numeric' or 'integer' and have length 2")
   expect_error(calc_FadingCorr(c(0.1, 0), "error"),
                "'g_value' should be of class 'numeric', 'integer' or 'RLum.Results'")
+  expect_error(calc_FadingCorr(age.faded = c(0.1,0), tc = 2592000,
+                               g_value = set_RLum("RLum.Results")),
+               "'g_value' has an unsupported originator")
   expect_error(calc_FadingCorr(c(0.1, 0), 1),
                "'g_value' should have length 2")
   expect_error(calc_FadingCorr(age.faded = c(0.1, 0), g_value = c(5.0, 1.0)),
-               "[calc_FadingCorr()] 'tc' should be of class 'numeric' or",
+               "[calc_FadingCorr()] 'tc' should be a single positive value",
                fixed = TRUE)
   expect_error(calc_FadingCorr(age.faded = c(0.1, 0), g_value = c(5.0, 1.0),
                                tc = c(1000, 2000)),
-               "[calc_FadingCorr()] 'tc' should be a positive scalar",
+               "[calc_FadingCorr()] 'tc' should be a single positive value",
                fixed = TRUE)
   expect_error(calc_FadingCorr(age.faded = c(0.1, 0), g_value = c(5.0, 1.0),
                                tc = 2592000, n.MC = "error"),
-               "'n.MC' should be a positive integer scalar")
+               "'n.MC' should be a single positive integer value")
   expect_error(calc_FadingCorr(age.faded = c(0.1, 0), g_value = c(5.0, 1.0),
                                tc = 2592000, interval = "error"),
-               "'interval' should be of class 'numeric'")
+               "'interval' should be of class 'numeric' and have length 2")
   expect_error(calc_FadingCorr(age.faded = c(0.1, 0), g_value = c(5.0, 1.0),
                                tc = 2592000, interval = 1),
-               "'interval' should have length 2")
+               "'interval' should be of class 'numeric' and have length 2")
   expect_error(calc_FadingCorr(age.faded = c(0.1, 0), g_value = c(5.0, 1.0),
                                tc = 2592000, txtProgressBar = "error"),
                "'txtProgressBar' should be a single logical value")
@@ -63,12 +66,6 @@ test_that("check class and length of output", {
                   "RLum.Results")
   })
 
-  fading@originator <- "unexpected"
-  expect_message(
-      expect_null(calc_FadingCorr(age.faded = c(0.1,0),
-                               g_value = fading, tc = 2592000)),
-               "Unknown originator for the 'g_value' object provided")
-
   ## auto, seed (Note: this is slow!)
   SW({
   calc_FadingCorr(
