@@ -5,10 +5,6 @@ test_that("input validation", {
                "'object' should be of class 'RLum.Analysis'")
   expect_error(convert_PSL2Risoe.BINfileData(set_RLum("RLum.Analysis")),
                "'object' cannot be an empty RLum.Analysis")
-
-  ## defunct name
-  expect_error(PSL2Risoe.BINfileData(set_RLum("RLum.Analysis")),
-               "'PSL2Risoe.BINfileData' is defunct")
 })
 
 test_that("check functionality", {
@@ -31,6 +27,5 @@ test_that("check functionality", {
   fake <- merged
   fake@records[10][[1]]@originator <- "unexpected-originator"
   expect_error(convert_PSL2Risoe.BINfileData(fake),
-               "Only objects originating from 'read_PSL2R()' are allowed",
-               fixed=TRUE)
+               "At least one element of 'object' has an unsupported originator")
 })
