@@ -1,5 +1,3 @@
-context("smooth_RLum")
-
 data(ExampleData.CW_OSL_Curve, envir = environment())
 temp <-
   set_RLum(
@@ -15,24 +13,27 @@ test_that("check class and length of output", {
   testthat::skip_on_cran()
 
   ##standard tests
-  expect_is(temp, class = "RLum.Data.Curve", info = NULL, label = NULL)
-  expect_is(smooth_RLum(temp), class = "RLum.Data.Curve", info = NULL, label = NULL)
+  expect_s4_class(temp, class = "RLum.Data.Curve")
+  expect_snapshot_RLum(smooth_RLum(temp))
 
   ##test on a list
     ##RLum list
-    expect_is(smooth_RLum(list(temp, temp)), "list")
+    expect_type(smooth_RLum(list(temp, temp)), "list")
 
     ##normal list
-    expect_is(smooth_RLum(list(a = 1, b = 2)), "list")
+    expect_type(smooth_RLum(list(a = 1, b = 2)), "list")
 
   ##test on an RLum.Analysis-object
   expect_s4_class(smooth_RLum(temp_analysis), "RLum.Analysis")
 
 })
 
-test_that("check values from output example", {
+test_that("snapshot tests", {
  testthat::skip_on_cran()
- expect_equivalent(round(mean(smooth_RLum(temp, k = 5)[,2], na.rm = TRUE), 0), 100)
- expect_equivalent(round(mean(smooth_RLum(temp, k = 10)[,2], na.rm = TRUE), 0), 85)
 
+ small <-set_RLum(class = "RLum.Data.Curve", recordType = "OSL",
+                  data = as.matrix(ExampleData.CW_OSL_Curve[1:150, ]))
+ expect_snapshot_RLum(smooth_RLum(small, k = 5))
+ expect_snapshot_RLum(smooth_RLum(small, k = 10))
+ expect_snapshot_RLum(smooth_RLum(small, k = 11, method = "median"))
 })
