@@ -1,10 +1,13 @@
 ##Full check
 test_that("Check template creation ", {
   testthat::skip_on_cran()
-  local_edition(3)
 
   ## test output class
-  expect_s3_class(template_DRAC(), "DRAC.list")
+  SW({
+  expect_message(res <- template_DRAC(),
+                 "IMPORTANT NOTE")
+  })
+  expect_s3_class(res, "DRAC.list")
   expect_s3_class(template_DRAC(notification = FALSE), "DRAC.list")
   expect_s3_class(template_DRAC(nrow = 10, notification = FALSE), "DRAC.list")
 
@@ -26,14 +29,27 @@ test_that("Check template creation ", {
   expect_true(!all(is.na(template_DRAC(preset = "DRAC-example_polymineral", notification = FALSE))))
   expect_equal(length(template_DRAC(notification = FALSE)), 53)
   expect_equal(length(template_DRAC(nrow = 10, notification = FALSE)[[1]]), 10)
-  expect_s3_class(template_DRAC(nrow = -1, notification = FALSE), "DRAC.list")
+
+  ## use the file_input option
+  SW({
+    tmp_file <- tempfile(fileext = ".csv")
+    write.csv(x = as.data.frame(template_DRAC(nrow = 12, preset = 'quartz_coarse')), file = tmp_file, row.names = FALSE)
+    t <- expect_s3_class(suppressWarnings(template_DRAC(file_input = tmp_file, notification = FALSE)), "DRAC.list")
+    expect_length(t[[1]], 3)
+  })
 
   ## expect failure
+  expect_error(template_DRAC(nrow = -1),
+               "'nrow' should be a single positive integer value")
+  expect_error(template_DRAC("preset"),
+               "'nrow' should be a single positive integer value")
   expect_warning(template_DRAC(nrow = 5001, notification = FALSE),
-                 regexp = "\\[template_DRAC\\(\\)\\] More than 5000 datasets might not be supported!")
-  expect_error(template_DRAC(preset = "this_one_does_not_exist"))
-  expect_error(template_DRAC(preset = c("this_one_does_not_exist", "this_one_neither")))
-  expect_error(template_DRAC(preset = 999))
-
+                 "[template_DRAC()] More than 5000 datasets might not be supported",
+                 fixed = TRUE)
+  expect_error(template_DRAC(preset = "does_not_exist"),
+               "'preset' should be one of 'quartz_coarse', 'quartz_fine'")
+  expect_error(template_DRAC(preset = c("does_not_exist", "neither_this_one")),
+               "'preset' contains multiple values but not all of them match 'choices'")
+  expect_error(template_DRAC(preset = 999),
+               "'preset' should be one of 'quartz_coarse', 'quartz_fine'")
 })
-
